# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from typing import List, Optional
from pydantic import BaseModel, Field, StrictBool, StrictStr, conlist, validator
from com.zuora.models.add_subscription_component import AddSubscriptionComponent

class ChangeSubscriptionRatePlan(BaseModel):
    """
    ChangeSubscriptionRatePlan
    """
    booking_date: Optional[date] = Field(None, alias="bookingDate", description="The booking date that you want to set for the amendment contract. The booking date of an amendment is the equivalent of the order date of an order. This field must be in the `yyyy-mm-dd` format. The default value is the current date when you make the API call.  ")
    charge_overrides: Optional[conlist(AddSubscriptionComponent)] = Field(None, alias="chargeOverrides", description="This optional container is used to override one or more product rate plan charges for this subscription.")
    contract_effective_date: Optional[date] = Field(None, alias="contractEffectiveDate", description="Effective date of the new subscription, as yyyy-mm-dd.")
    customer_acceptance_date: Optional[date] = Field(None, alias="customerAcceptanceDate", description="The date when the customer accepts the contract in yyyy-mm-dd format. When this field is not set: * If the `serviceActivationDate` field is not set, the value of this field is set to be the contract effective date. * If the `serviceActivationDate` field is set, the value of this field is set to be the service activation date.  The billing trigger dates must follow this rule: contractEffectiveDate <= serviceActivationDate <= contractAcceptanceDate ")
    effective_policy: Optional[StrictStr] = Field(None, alias="effectivePolicy", description="The default value for the `effectivePolicy` field is as follows:   * If the rate plan change (from old to new) is an upgrade, the effective policy is `EffectiveImmediately` by default.   * If the rate plan change (from old to new) is a downgrade, the effective policy is `EffectiveEndOfBillingPeriod` by default.   * Otherwise, the effective policy is `SpecificDate` by default.  Note that if the `effectivePolicy` field is set to `EffectiveEndOfBillingPeriod`, you cannot set the <a href=\"https://knowledgecenter.zuora.com/Zuora_Billing/Subscriptions/Subscriptions/W_Subscription_and_Amendment_Dates#Billing_Trigger_Dates\" target=\"_blank\">billing trigger dates</a> for the subscription as the system will automatically set the trigger dates to the end of billing period. ")
    external_catalog_plan_id: Optional[StrictStr] = Field(None, alias="externalCatalogPlanId", description="An external ID of the rate plan to be removed. You can use this field to specify an existing rate plan in your subscription. The value of the `externalCatalogPlanId` field must match one of the values that are predefined in the `externallyManagedPlanIds` field on a product rate plan. However, if there are multiple rate plans with the same `productRatePlanId` value existing in the subscription, you must use the `ratePlanId` field to remove the rate plan. The `externalCatalogPlanId` field cannot be used to distinguish multiple rate plans in this case.  **Note:** Provide only one of `externalCatalogPlanId`, `ratePlanId` or `productRatePlanId`. If more than one field is provided then the request would fail. ")
    external_id_source_system: Optional[StrictStr] = Field(None, alias="externalIdSourceSystem", description="The ID of the external source system. You can use this field and `externalCatalogPlanId` to specify a product rate plan that is imported from an external system.  **Note:** If both `externalCatalogPlanId`, `externalIdSourceSystem` and `productRatePlanId` are provided. They must point to the same product rate plan. Otherwise, the request would fail. ")
    new_external_catalog_plan_id: Optional[StrictStr] = Field(None, alias="newExternalCatalogPlanId", description="An external ID of the product rate plan to be added. You can use this field to specify a product rate plan that is imported from an external system. The value of the `externalCatalogPlanId` field must match one of the values that are predefined in the `externallyManagedPlanIds` field on a product rate plan.  **Note:** Provide only one of `newExternalCatalogPlanId` or `newProductRatePlanId`. If both fields are provided then the request would fail. ")
    new_external_id_source_system: Optional[StrictStr] = Field(None, alias="newExternalIdSourceSystem", description="The ID of the external source system. You can use this field and `newExternalCatalogPlanId` to specify a product rate plan that is imported from an external system.  **Note:** If both `newExternalCatalogPlanId`, `newExternalIdSourceSystem` and `newProductRatePlanId` are provided. They must point to the same product rate plan. Otherwise, the request would fail. ")
    new_product_rate_plan_id: Optional[StrictStr] = Field(None, alias="newProductRatePlanId", description="ID of a product rate plan for this subscription.")
    new_product_rate_plan_number: Optional[StrictStr] = Field(None, alias="newProductRatePlanNumber", description="Number of a product rate plan for this subscription.")
    product_rate_plan_id: Optional[StrictStr] = Field(None, alias="productRatePlanId", description="ID of the product rate plan that the removed rate plan is based on. ")
    product_rate_plan_number: Optional[StrictStr] = Field(None, alias="productRatePlanNumber", description="Number of a product rate plan for this subscription.    ")
    rate_plan_id: Optional[StrictStr] = Field(None, alias="ratePlanId", description="ID of a rate plan to remove. Note that the removal of a rate plan through the Change Plan amendment supports the function of <a href=\"https://knowledgecenter.zuora.com/Zuora_Billing/Subscriptions/Subscriptions/Subscribe_and_Amend/E_Amendments/EB_Remove_rate_plan_on_subscription_before_future-dated_removals\" target=\"_blank\">removal before future-dated removals</a>, as in a Remove Product amendment. ")
    reset_bcd: Optional[StrictBool] = Field(False, alias="resetBcd", description="If resetBcd is true then reset the Account BCD to the effective date; if it is false keep the original BCD. ")
    service_activation_date: Optional[date] = Field(None, alias="serviceActivationDate", description="The date when the change in the subscription is activated in yyyy-mm-dd format. You must specify a Service Activation date if the Customer Acceptance date is set. If the Customer Acceptance date is not set, the value of the `serviceActivationDate` field defaults to be the Contract Effective Date. The billing trigger dates must follow this rule: contractEffectiveDate <= serviceActivationDate <= contractAcceptanceDate")
    sub_type: Optional[StrictStr] = Field(None, alias="subType", description="Use this field to choose the sub type for your change plan amendment.   However, if you do not set this field, the field will be automatically generated by the system according to the following rules:  When the old and new rate plans are within the same Grading catalog group: * If the grade of new plan is greater than that of the old plan, this is an \"Upgrade\". * If the grade of new plan is less than that of the old plan, this is a \"Downgrade\". * If the grade of new plan equals that of the old plan, this is a \"Crossgrade\".  When the old and new rate plans are not in the same Grading catalog group, or either has no group, this is \"PlanChanged\". ")
    subscription_rate_plan_number: Optional[StrictStr] = Field(None, alias="subscriptionRatePlanNumber", description="Number of a rate plan for this subscription.  ")
    __properties = ["bookingDate", "chargeOverrides", "contractEffectiveDate", "customerAcceptanceDate", "effectivePolicy", "externalCatalogPlanId", "externalIdSourceSystem", "newExternalCatalogPlanId", "newExternalIdSourceSystem", "newProductRatePlanId", "newProductRatePlanNumber", "productRatePlanId", "productRatePlanNumber", "ratePlanId", "resetBcd", "serviceActivationDate", "subType", "subscriptionRatePlanNumber"]

    @validator('effective_policy')
    def effective_policy_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('EffectiveImmediately', 'EffectiveEndOfBillingPeriod', 'SpecificDate'):
            raise ValueError("must be one of enum values ('EffectiveImmediately', 'EffectiveEndOfBillingPeriod', 'SpecificDate')")
        return value

    @validator('sub_type')
    def sub_type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('Upgrade', 'Downgrade', 'Crossgrade', 'PlanChanged'):
            raise ValueError("must be one of enum values ('Upgrade', 'Downgrade', 'Crossgrade', 'PlanChanged')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ChangeSubscriptionRatePlan:
        """Create an instance of ChangeSubscriptionRatePlan from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in charge_overrides (list)
        _items = []
        if self.charge_overrides:
            for _item in self.charge_overrides:
                if _item:
                    _items.append(_item.to_dict())
            _dict['chargeOverrides'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> ChangeSubscriptionRatePlan:
        """Create an instance of ChangeSubscriptionRatePlan from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return ChangeSubscriptionRatePlan.parse_obj(obj)

        _obj = ChangeSubscriptionRatePlan.parse_obj({
            "booking_date": obj.get("bookingDate"),
            "charge_overrides": [AddSubscriptionComponent.from_dict(_item) for _item in obj.get("chargeOverrides")] if obj.get("chargeOverrides") is not None else None,
            "contract_effective_date": obj.get("contractEffectiveDate"),
            "customer_acceptance_date": obj.get("customerAcceptanceDate"),
            "effective_policy": obj.get("effectivePolicy"),
            "external_catalog_plan_id": obj.get("externalCatalogPlanId"),
            "external_id_source_system": obj.get("externalIdSourceSystem"),
            "new_external_catalog_plan_id": obj.get("newExternalCatalogPlanId"),
            "new_external_id_source_system": obj.get("newExternalIdSourceSystem"),
            "new_product_rate_plan_id": obj.get("newProductRatePlanId"),
            "new_product_rate_plan_number": obj.get("newProductRatePlanNumber"),
            "product_rate_plan_id": obj.get("productRatePlanId"),
            "product_rate_plan_number": obj.get("productRatePlanNumber"),
            "rate_plan_id": obj.get("ratePlanId"),
            "reset_bcd": obj.get("resetBcd") if obj.get("resetBcd") is not None else False,
            "service_activation_date": obj.get("serviceActivationDate"),
            "sub_type": obj.get("subType"),
            "subscription_rate_plan_number": obj.get("subscriptionRatePlanNumber")
        })
        return _obj


