# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, StrictBool, StrictStr, conlist
from com.zuora.models.failed_reason import FailedReason

class CreateAccountResponse(BaseModel):
    """
    CreateAccountResponse
    """
    process_id: Optional[StrictStr] = Field(None, alias="processId", description="The Id of the process that handle the operation. ")
    request_id: Optional[StrictStr] = Field(None, alias="requestId", description="Unique request identifier. If you need to contact us about a specific request, providing the request identifier will ensure the fastest possible resolution. ")
    reasons: Optional[conlist(FailedReason)] = None
    success: Optional[StrictBool] = Field(None, description="Returns `true` if the request was processed successfully. ")
    account_id: Optional[StrictStr] = Field(None, alias="accountId", description="Auto-generated account ID. ")
    account_number: Optional[StrictStr] = Field(None, alias="accountNumber", description="Account number. ")
    bill_to_contact_id: Optional[StrictStr] = Field(None, alias="billToContactId", description="The ID of the bill-to contact. ")
    contracted_mrr: Optional[StrictStr] = Field(None, alias="contractedMrr", description="Contracted monthly recurring revenue of the subscription. ")
    credit_memo_id: Optional[StrictStr] = Field(None, alias="creditMemoId", description="The credit memo ID, if a credit memo is generated during the subscription process.  **Note:** This field is only available if you have [Invoice Settlement](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/Invoice_Settlement) enabled. The Invoice Settlement feature is generally available as of Zuora Billing Release 296 (March 2021). This feature includes Unapplied Payments, Credit and Debit Memo, and Invoice Item Settlement. If you want to enable Invoice Settlement, see [Invoice Settlement Enablement and Checklist Guide](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/Invoice_Settlement/Invoice_Settlement_Migration_Checklist_and_Guide) for more information. ")
    invoice_id: Optional[StrictStr] = Field(None, alias="invoiceId", description="ID of the invoice generated at account creation, if applicable. ")
    paid_amount: Optional[StrictStr] = Field(None, alias="paidAmount", description="Amount collected on the invoice generated at account creation, if applicable. ")
    payment_id: Optional[StrictStr] = Field(None, alias="paymentId", description="ID of the payment collected on the invoice generated at account creation, if applicable. ")
    payment_method_id: Optional[StrictStr] = Field(None, alias="paymentMethodId", description="ID of the payment method that was set up at account creation, which automatically becomes the default payment method for this account. ")
    sold_to_contact_id: Optional[StrictStr] = Field(None, alias="soldToContactId", description="The ID of the sold-to contact. ")
    subscription_id: Optional[StrictStr] = Field(None, alias="subscriptionId", description="ID of the subscription that was set up at account creation, if applicable. ")
    subscription_number: Optional[StrictStr] = Field(None, alias="subscriptionNumber", description="Number of the subscription that was set up at account creation, if applicable. ")
    total_contracted_value: Optional[StrictStr] = Field(None, alias="totalContractedValue", description="Total contracted value of the subscription. ")
    additional_properties: Dict[str, Any] = {}
    __properties = ["processId", "requestId", "reasons", "success", "accountId", "accountNumber", "billToContactId", "contractedMrr", "creditMemoId", "invoiceId", "paidAmount", "paymentId", "paymentMethodId", "soldToContactId", "subscriptionId", "subscriptionNumber", "totalContractedValue"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CreateAccountResponse:
        """Create an instance of CreateAccountResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in reasons (list)
        _items = []
        if self.reasons:
            for _item in self.reasons:
                if _item:
                    _items.append(_item.to_dict())
            _dict['reasons'] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> CreateAccountResponse:
        """Create an instance of CreateAccountResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return CreateAccountResponse.parse_obj(obj)

        _obj = CreateAccountResponse.parse_obj({
            "process_id": obj.get("processId"),
            "request_id": obj.get("requestId"),
            "reasons": [FailedReason.from_dict(_item) for _item in obj.get("reasons")] if obj.get("reasons") is not None else None,
            "success": obj.get("success"),
            "account_id": obj.get("accountId"),
            "account_number": obj.get("accountNumber"),
            "bill_to_contact_id": obj.get("billToContactId"),
            "contracted_mrr": obj.get("contractedMrr"),
            "credit_memo_id": obj.get("creditMemoId"),
            "invoice_id": obj.get("invoiceId"),
            "paid_amount": obj.get("paidAmount"),
            "payment_id": obj.get("paymentId"),
            "payment_method_id": obj.get("paymentMethodId"),
            "sold_to_contact_id": obj.get("soldToContactId"),
            "subscription_id": obj.get("subscriptionId"),
            "subscription_number": obj.get("subscriptionNumber"),
            "total_contracted_value": obj.get("totalContractedValue")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj


