# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional, Union
from pydantic import BaseModel, Field, StrictBool, StrictFloat, StrictInt, StrictStr, conlist
from com.zuora.models.create_batch_query_request import CreateBatchQueryRequest

class CreateBatchQueryJobRequest(BaseModel):
    """
    CreateBatchQueryJobRequest
    """
    date_time_utc: Optional[StrictBool] = Field(None, alias="dateTimeUtc", description="When using WSDL 69 and later you can ensure that the exported output of dateTime records are rendered according to ISO-8601 generic UTC form by setting `dateTimeUtc` to `true`.  When `dateTimeUtc` is set to `true`, exports of dateTime data types will be rendered in the following generic format: `YYYY-MM-DDThh:mm:ss-hhmm` or `YYYY-MM-DDThh:mm:ss+hhmm`.  **Note**: Regardless of what batchType query is used (`zoql` or `zoqlexport`), the query response output for datetime data types can be standardized by setting dateTimeUtc to `true`. When `true`, the results will display datetime types with the format: YYYY-MM-DDThh:mm:ss+/-hhmm. ")
    format: Optional[StrictStr] = None
    name: Optional[StrictStr] = Field(None, description="The name of the job. 32 character limit. ")
    notify_url: Optional[StrictStr] = Field(None, alias="notifyUrl", description="If URL is provided, the AQuA job will call this `notifyUrl` once the job has completed. The value of `notifyUrl` needs to have `${JOBID}` and `${STATUS}` placeholders. These placeholders will be replaced by the actual job ID and status when returned in the response. Status will be `Completed` after the AQuA job is done.  If you submit an AQuA query with `notifyUrl` specified, the value of `notifyUrl` will be ignored if your organization has already <a href=\"https://knowledgecenter.zuora.com/Zuora_Central_Platform/API/AB_Aggregate_Query_API/Callout_Notification_for_Completed_AQuA_Jobs\" target=\"_blank\">configured a callout notification through the Zuora user interface</a>.  ")
    null_replacement: Optional[StrictStr] = Field(None, alias="nullReplacement", description="The string used to represent null values in the query results. If you do not set this parameter, null values are represented by the empty string in the query results. ")
    offset: Optional[Union[StrictFloat, StrictInt]] = Field(None, description="This field specifies the time offset for AQuA queries in stateful mode. It is an integer in the range 0 to 3,600 seconds.  For example, if you set this field to 600 seconds and you post a query in stateful mode at 2:00 AM, it will query against data created or updated between the completion time of the previous query and 1:50 AM.  The value of this field will override the value you configured in **Settings** > **Administration** > **AQuA API Stateful Mode Time Offset**.         ")
    partner: Optional[StrictStr] = Field(None, description="The partner field indicates the unique ID of a data integration partner. The dropdown list of this field displays partner IDs for the past thirty days. It must be used together with \"project\" field to uniquely identify a data integration target.  For example, if a continuous AQuA session is to retrieve data incrementally for a Salesforce.com Org 00170000011K3Ub, you can use partner as \"Salesforce\", and \"project\" as \"00170000011K3Ub.\"  This field is required only if you are using AQuA in stateful mode. Otherwise, if you are using AQuA in stateless mode, partner field can be null.  **Note**: Zuora highly recommends you use the stateless mode instead of the stateful mode to extract bulk data. See <a href=\"https://knowledgecenter.zuora.com/Zuora_Central_Platform/API/AB_Aggregate_Query_API/Bulk_data__extraction_from_Zuora_using_AQuA\" target=\"_blank\">Bulk data extraction from Zuora using AQuA</a> for best practices. **Note**: Submit a request at <a href=\"http://support.zuora.com\" target=\"_blank\">Zuora Global Support</a> to obtain a partner ID. ")
    project: Optional[StrictStr] = Field(None, description="The project field contains the unique ID of a data integration project for a particular partner. The dropdown list of this field displays project IDs for the past thirty days.  This field must be used together with partner field to uniquely identify a data integration target.   This field is required only if you are using AQuA in stateful mode. Otherwise, if you are using AQuA in stateless mode, partner field can be null. ")
    queries: Optional[conlist(CreateBatchQueryRequest)] = Field(None, description="A JSON array object that contains a list of batch objects. ")
    use_query_labels: Optional[StrictBool] = Field(None, alias="useQueryLabels", description="When this optional flag is set to `true` the request will use object and field API names for the CSV header output instead of the field labels. Data integration projects should set `useQueryLabels` to `true` so that API names remain the same.  By default `useQueryLabels` is `false`, so that output CSV headers display the more user-friendly object and field labels.  ")
    version: Optional[Union[StrictFloat, StrictInt]] = Field(None, description="The API version you want to use.   The supported versions are as follows:   - `1.1`. It supports both modes   - `1.0`. Default. It supports stateless modes only.  See <a href=\"https://knowledgecenter.zuora.com/Zuora_Central_Platform/API/AB_Aggregate_Query_API/BA_Stateless_and_Stateful_Modes\" target=\"_blank\">Stateless and stateful modes</a> for more information. ")
    __properties = ["dateTimeUtc", "format", "name", "notifyUrl", "nullReplacement", "offset", "partner", "project", "queries", "useQueryLabels", "version"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CreateBatchQueryJobRequest:
        """Create an instance of CreateBatchQueryJobRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in queries (list)
        _items = []
        if self.queries:
            for _item in self.queries:
                if _item:
                    _items.append(_item.to_dict())
            _dict['queries'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> CreateBatchQueryJobRequest:
        """Create an instance of CreateBatchQueryJobRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return CreateBatchQueryJobRequest.parse_obj(obj)

        _obj = CreateBatchQueryJobRequest.parse_obj({
            "date_time_utc": obj.get("dateTimeUtc"),
            "format": obj.get("format"),
            "name": obj.get("name"),
            "notify_url": obj.get("notifyUrl"),
            "null_replacement": obj.get("nullReplacement"),
            "offset": obj.get("offset"),
            "partner": obj.get("partner"),
            "project": obj.get("project"),
            "queries": [CreateBatchQueryRequest.from_dict(_item) for _item in obj.get("queries")] if obj.get("queries") is not None else None,
            "use_query_labels": obj.get("useQueryLabels"),
            "version": obj.get("version")
        })
        return _obj


