# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from typing import List, Optional
from pydantic import BaseModel, Field, StrictBool, StrictInt, StrictStr, conint, conlist
from com.zuora.models.bill_run_filter import BillRunFilter
from com.zuora.models.bill_run_schedule import BillRunSchedule

class CreateBillRunRequest(BaseModel):
    """
    CreateBillRunRequest
    """
    name: Optional[StrictStr] = Field(None, description="The name of the bill run. ")
    batches: Optional[conlist(StrictStr)] = Field(None, description="The batch of accounts for this bill run.   You can only specify either this field or the `billRunFilters` field.  **Values:** `AllBatches` or an array of `Batch*n*` where *n* is one of numbers 1 - 50, for example, `Batch7`. ")
    bill_cycle_day: Optional[StrictStr] = Field(None, alias="billCycleDay", description="The day of the bill cycle. This field is only valid if the `batches` field is specified.  **Values:**  - `AllBillCycleDays` or one of numbers 1 - 31 for an ad-hoc bill run - `AllBillCycleDays`, one of numbers 1 - 31, or `AsRunDay` for a scheduled bill run ")
    bill_run_filters: Optional[conlist(BillRunFilter)] = Field(None, alias="billRunFilters", description="The target account or subscriptions for this bill run. You can only specify either this field or the `batches` field. ")
    bill_run_type: Optional[StrictStr] = Field(None, alias="billRunType")
    charge_type_to_exclude: Optional[conlist(StrictStr)] = Field(None, alias="chargeTypeToExclude", description="The types of the charges to be excluded from the generation of billing documents. You can specify at most two charge types in the array. ")
    auto_email: Optional[StrictBool] = Field(False, alias="autoEmail", description="Whether to automatically send emails after Auto-Post is complete.  **Note:** To use this field, you must first set the <a href=\"https://knowledgecenter.zuora.com/CB_Billing/Billing_Settings/Define_Billing_Rules\" target=\"_blank\">Support Bill Run Auto-Post?</a> billing rule to **Yes** through the Zuora UI. ")
    auto_post: Optional[StrictBool] = Field(False, alias="autoPost", description="Whether to automatically post the bill run after the bill run is created.  **Note:** To use this field, you must first set the <a href=\"https://knowledgecenter.zuora.com/CB_Billing/Billing_Settings/Define_Billing_Rules\" target=\"_blank\">Support Bill Run Auto-Post?</a> billing rule to **Yes** through the Zuora UI. ")
    auto_renewal: Optional[StrictBool] = Field(False, alias="autoRenewal", description="Whether to automatically renew auto-renew subscriptions that are up for renewal. ")
    no_email_for_zero_amount_invoice: Optional[StrictBool] = Field(False, alias="noEmailForZeroAmountInvoice", description="Whether to suppress emails for invoices with zero total amount generated in this bill run after the bill run is complete.   It is best practice to not send emails for invoices with zero amount. ")
    schedule: Optional[BillRunSchedule] = None
    invoice_date: Optional[date] = Field(None, alias="invoiceDate", description="The invoice date for the bill run.   - When creating an ad-hoc bill run, if you do not specify any value for this field, the default value is the current date. - When creating a scheduled bill run, if you do not specify any value for this field, the invoice date is the value of the `repeatFrom` field. ")
    invoice_date_month_offset: Optional[StrictInt] = Field(None, alias="invoiceDateMonthOffset", description="The month offset of invoice date for this bill run based on run date, only valid for monthly scheduled bill runs. invoiceDateOffset and invoiceDateMonthOffset/invoiceDateDayOfMonth are mutually exclusive. invoiceDateMonthOffset and invoiceDateDayOfMonth coexist. ")
    invoice_date_day_of_month: Optional[conint(strict=True, le=31, ge=1)] = Field(None, alias="invoiceDateDayOfMonth", description="The day of month of invoice date for this bill run, only valid for monthly scheduled bill runs. Specify any day of the month (1-31, where 31 = end-of-month). invoiceDateOffset and invoiceDateMonthOffset/invoiceDateDayOfMonth are mutually exclusive. invoiceDateMonthOffset and invoiceDateDayOfMonth coexist. ")
    target_date: Optional[date] = Field(None, alias="targetDate", description="The target date for this bill run.   - You must specify this field when creating an ad-hoc bill run. - For scheduled bill runs, if you do not specify any value for this field, the target date is the value of the `repeatFrom` field. ")
    target_date_month_offset: Optional[StrictInt] = Field(None, alias="targetDateMonthOffset", description="The month offset of target date for this bill run based on run date, only valid for monthly scheduled bill runs. targetDateOffset and targetDateMonthOffset/targetDateDayOfMonth are mutually exclusive. targetDateMonthOffset and targetDateDayOfMonth coexist. ")
    target_date_day_of_month: Optional[conint(strict=True, le=31, ge=1)] = Field(None, alias="targetDateDayOfMonth", description="The day of month of target date for this bill run, only valid for monthly scheduled bill runs. Specify any day of the month (1-31, where 31 = end-of-month). targetDateOffset and targetDateMonthOffset/targetDateDayOfMonth are mutually exclusive. targetDateMonthOffset and targetDateDayOfMonth coexist. ")
    __properties = ["name", "batches", "billCycleDay", "billRunFilters", "billRunType", "chargeTypeToExclude", "autoEmail", "autoPost", "autoRenewal", "noEmailForZeroAmountInvoice", "schedule", "invoiceDate", "invoiceDateMonthOffset", "invoiceDateDayOfMonth", "targetDate", "targetDateMonthOffset", "targetDateDayOfMonth"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CreateBillRunRequest:
        """Create an instance of CreateBillRunRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in bill_run_filters (list)
        _items = []
        if self.bill_run_filters:
            for _item in self.bill_run_filters:
                if _item:
                    _items.append(_item.to_dict())
            _dict['billRunFilters'] = _items
        # override the default output from pydantic by calling `to_dict()` of schedule
        if self.schedule:
            _dict['schedule'] = self.schedule.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> CreateBillRunRequest:
        """Create an instance of CreateBillRunRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return CreateBillRunRequest.parse_obj(obj)

        _obj = CreateBillRunRequest.parse_obj({
            "name": obj.get("name"),
            "batches": obj.get("batches"),
            "bill_cycle_day": obj.get("billCycleDay"),
            "bill_run_filters": [BillRunFilter.from_dict(_item) for _item in obj.get("billRunFilters")] if obj.get("billRunFilters") is not None else None,
            "bill_run_type": obj.get("billRunType"),
            "charge_type_to_exclude": obj.get("chargeTypeToExclude"),
            "auto_email": obj.get("autoEmail") if obj.get("autoEmail") is not None else False,
            "auto_post": obj.get("autoPost") if obj.get("autoPost") is not None else False,
            "auto_renewal": obj.get("autoRenewal") if obj.get("autoRenewal") is not None else False,
            "no_email_for_zero_amount_invoice": obj.get("noEmailForZeroAmountInvoice") if obj.get("noEmailForZeroAmountInvoice") is not None else False,
            "schedule": BillRunSchedule.from_dict(obj.get("schedule")) if obj.get("schedule") is not None else None,
            "invoice_date": obj.get("invoiceDate"),
            "invoice_date_month_offset": obj.get("invoiceDateMonthOffset"),
            "invoice_date_day_of_month": obj.get("invoiceDateDayOfMonth"),
            "target_date": obj.get("targetDate"),
            "target_date_month_offset": obj.get("targetDateMonthOffset"),
            "target_date_day_of_month": obj.get("targetDateDayOfMonth")
        })
        return _obj


