# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from typing import Optional
from pydantic import BaseModel, Field, StrictBool, StrictStr, constr

class CreateBillingPreviewRunRequest(BaseModel):
    """
    CreateBillingPreviewRunRequest
    """
    assume_renewal: Optional[StrictStr] = Field(None, alias="assumeRenewal", description="Indicates whether to generate a preview of future invoice items and credit memo items with the assumption that the subscriptions are renewed.  Set one of the following values in this field to decide how the assumption is applied in the billing preview.    * **All:** The assumption is applied to all the subscriptions. Zuora generates preview invoice item data and credit memo item data from the first day of the customer's next billing period to the target date.      * **None:** (Default) The assumption is not applied to the subscriptions. Zuora generates preview invoice item data and credit memo item data based on the current term end date and the target date.        * If the target date is later than the current term end date, Zuora generates preview invoice item data and credit memo item data from the first day of the customer's next billing period to the current term end date.      * If the target date is earlier than the current term end date, Zuora generates preview invoice item data and credit memeo item data from the first day of the customer's next billing period to the target date.    * **Autorenew:** The assumption is applied to the subscriptions that have auto-renew enabled. Zuora generates preview invoice item data and credit memo item data from the first day of the customer's next billing period to the target date.    **Note:**    - This field can only be used if the subscription renewal term is not set to 0.           - The credit memo item data is only available if you have Invoice Settlement feature enabled. The Invoice Settlement feature is generally available as of Zuora Billing Release 296 (March 2021). This feature includes Unapplied Payments, Credit and Debit Memo, and Invoice Item Settlement. If you want to enable Invoice Settlement, see [Invoice Settlement Enablement and Checklist Guide](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/Invoice_Settlement/Invoice_Settlement_Migration_Checklist_and_Guide) for more information.  ")
    batch: Optional[constr(strict=True, max_length=255)] = Field(None, description="The customer batch to include in the billing preview run. If not specified, all customer batches are included.   **Note**: This field is not available if you set the `zuora-version` request header to `314.0` or later. ")
    batches: Optional[constr(strict=True, max_length=1000)] = Field(None, description="The customer batches to include in the billing preview run. You can specify multiple batches separated by comma. If not specified, all customer batches are included.  **Note**: This field is only available if you set the `zuora-version` request header to `314.0` or later. ")
    charge_type_to_exclude: Optional[StrictStr] = Field(None, alias="chargeTypeToExclude", description="The charge types to exclude from the forecast run.  **Possible values:** OneTime, Recurring, Usage, and any comma-separated combination of these values. ")
    including_draft_items: Optional[StrictBool] = Field(None, alias="includingDraftItems", description="Whether draft document items are included in the billing preview run. By default, draft document items are not included.  This field loads draft invoice items and credit memo items. The `chargeTypeToExclude`, `targetDate`, `includingEvergreenSubscription`, and `assumeRenewal` fields do not affect the behavior of the `includingDraftItems` field. ")
    including_evergreen_subscription: Optional[StrictBool] = Field(None, alias="includingEvergreenSubscription", description="Whether evergreen subscriptions are included in the billing preview run. By default, evergreen subscriptions are not included. ")
    storage_option: Optional[StrictStr] = Field(None, alias="storageOption")
    target_date: date = Field(..., alias="targetDate", description="The target date for the billing preview run. The billing preview run generates preview invoice item data and credit memo item data from the first day of the customer's next billing period to the target date.   The value for the `targetDate` field must be in _`YYYY-MM-DD`_ format.  If the target date is later than the subscription current term end date, the preview invoice item data and credit memo item data is generated from the first day of the customer's next billing period to the current term end date. If you want to generate preview invoice item data and credit memo item data past the end of the subscription current term, specify the AssumeRenewal field in the request.  **Note:** The credit memo item data is only available if you have Invoice Settlement feature enabled. The Invoice Settlement feature is generally available as of Zuora Billing Release 296 (March 2021). This feature includes Unapplied Payments, Credit and Debit Memo, and Invoice Item Settlement. If you want to enable Invoice Settlement, see [Invoice Settlement Enablement and Checklist Guide](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/Invoice_Settlement/Invoice_Settlement_Migration_Checklist_and_Guide) for more information. ")
    compared_billing_preview_run_id: Optional[StrictStr] = Field(None, alias="comparedBillingPreviewRunId", description="The ID of the billing preview run to compare with the current billing preview run.")
    store_difference: Optional[StrictBool] = Field(None, alias="storeDifference", description="Whether to store the difference between the current billing preview run and the compared billing preview run. The default value is `false`.")
    __properties = ["assumeRenewal", "batch", "batches", "chargeTypeToExclude", "includingDraftItems", "includingEvergreenSubscription", "storageOption", "targetDate", "comparedBillingPreviewRunId", "storeDifference"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CreateBillingPreviewRunRequest:
        """Create an instance of CreateBillingPreviewRunRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> CreateBillingPreviewRunRequest:
        """Create an instance of CreateBillingPreviewRunRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return CreateBillingPreviewRunRequest.parse_obj(obj)

        _obj = CreateBillingPreviewRunRequest.parse_obj({
            "assume_renewal": obj.get("assumeRenewal"),
            "batch": obj.get("batch"),
            "batches": obj.get("batches"),
            "charge_type_to_exclude": obj.get("chargeTypeToExclude"),
            "including_draft_items": obj.get("includingDraftItems"),
            "including_evergreen_subscription": obj.get("includingEvergreenSubscription"),
            "storage_option": obj.get("storageOption"),
            "target_date": obj.get("targetDate"),
            "compared_billing_preview_run_id": obj.get("comparedBillingPreviewRunId"),
            "store_difference": obj.get("storeDifference")
        })
        return _obj


