# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictStr, constr

class CreateContactRequest(BaseModel):
    """
    CreateContactRequest
    """
    account_id: Optional[StrictStr] = Field(None, alias="accountId", description="The ID of the account associated with the contact.   **Note**: When creating a contact, you must specify `accountNumber`, `accountId`, or both in the request body. If both fields are specified, they must correspond to the same account. ")
    account_number: Optional[StrictStr] = Field(None, alias="accountNumber", description="The number of the customer account associated with the contact.   **Note**: When creating a contact, you must specify `accountNumber`, `accountId`, or both in the request body. If both fields are specified, they must correspond to the same account. ")
    address1: Optional[constr(strict=True, max_length=255)] = Field(None, description="The first line of the contact's address, which is often a street address or business name. ")
    address2: Optional[constr(strict=True, max_length=255)] = Field(None, description="The second line of the contact's address. ")
    city: Optional[constr(strict=True, max_length=40)] = Field(None, description="The city of the contact's address. ")
    contact_description: Optional[constr(strict=True, max_length=100)] = Field(None, alias="contactDescription", description="A description for the contact. ")
    country: Optional[constr(strict=True, max_length=64)] = Field(None, description="The country of the contact's address. ")
    county: Optional[constr(strict=True, max_length=32)] = Field(None, description="The county. May optionally be used by Zuora Tax to calculate county tax. ")
    fax: Optional[constr(strict=True, max_length=40)] = Field(None, description="The contact's fax number. ")
    first_name: constr(strict=True, max_length=100) = Field(..., alias="firstName", description="The contact's first name. ")
    home_phone: Optional[constr(strict=True, max_length=40)] = Field(None, alias="homePhone", description="The contact's home phone number. ")
    last_name: constr(strict=True, max_length=100) = Field(..., alias="lastName", description="The contact's last name. ")
    mobile_phone: Optional[constr(strict=True, max_length=100)] = Field(None, alias="mobilePhone", description="The mobile phone number of the contact. ")
    nickname: Optional[constr(strict=True, max_length=100)] = Field(None, description="A nickname for the contact. ")
    other_phone: Optional[constr(strict=True, max_length=40)] = Field(None, alias="otherPhone", description="An additional phone number for the contact. ")
    other_phone_type: Optional[StrictStr] = Field(None, alias="otherPhoneType")
    personal_email: Optional[constr(strict=True, max_length=80)] = Field(None, alias="personalEmail", description="The contact's personal email address. ")
    state: Optional[constr(strict=True, max_length=40)] = Field(None, description="The state or province of the contact's address. ")
    tax_region: Optional[constr(strict=True, max_length=32)] = Field(None, alias="taxRegion", description="If using Zuora Tax, a region string as optionally defined in your tax rules. Not required. ")
    work_email: Optional[constr(strict=True, max_length=80)] = Field(None, alias="workEmail", description="The contact's business email address. ")
    work_phone: Optional[constr(strict=True, max_length=40)] = Field(None, alias="workPhone", description="The contact's business phone number. ")
    zip_code: Optional[constr(strict=True, max_length=20)] = Field(None, alias="zipCode", description="The zip code for the contact's address. ")
    additional_properties: Dict[str, Any] = {}
    __properties = ["accountId", "accountNumber", "address1", "address2", "city", "contactDescription", "country", "county", "fax", "firstName", "homePhone", "lastName", "mobilePhone", "nickname", "otherPhone", "otherPhoneType", "personalEmail", "state", "taxRegion", "workEmail", "workPhone", "zipCode"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CreateContactRequest:
        """Create an instance of CreateContactRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> CreateContactRequest:
        """Create an instance of CreateContactRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return CreateContactRequest.parse_obj(obj)

        _obj = CreateContactRequest.parse_obj({
            "account_id": obj.get("accountId"),
            "account_number": obj.get("accountNumber"),
            "address1": obj.get("address1"),
            "address2": obj.get("address2"),
            "city": obj.get("city"),
            "contact_description": obj.get("contactDescription"),
            "country": obj.get("country"),
            "county": obj.get("county"),
            "fax": obj.get("fax"),
            "first_name": obj.get("firstName"),
            "home_phone": obj.get("homePhone"),
            "last_name": obj.get("lastName"),
            "mobile_phone": obj.get("mobilePhone"),
            "nickname": obj.get("nickname"),
            "other_phone": obj.get("otherPhone"),
            "other_phone_type": obj.get("otherPhoneType"),
            "personal_email": obj.get("personalEmail"),
            "state": obj.get("state"),
            "tax_region": obj.get("taxRegion"),
            "work_email": obj.get("workEmail"),
            "work_phone": obj.get("workPhone"),
            "zip_code": obj.get("zipCode")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj


