# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from typing import Optional
from pydantic import BaseModel, Field, StrictStr

class CreateInvoiceCollectRequest(BaseModel):
    """
    CreateInvoiceCollectRequest
    """
    account_key: StrictStr = Field(..., alias="accountKey", description="Customer account ID or account number. ")
    document_date: Optional[date] = Field(None, alias="documentDate", description="The date that should appear on the invoice and credit memo being generated, in `yyyy-mm-dd` format. If this field is omitted and `invoiceId` is not specified, the current date is used by default.    **Note:** The credit memo is only available if you have the Invoice Settlement feature enabled.   This field is in Zuora REST API version control. Supported minor versions are `215.0` and later. To use this field in the method, you must set the  `zuora-version` parameter to the minor version number in the request header. ")
    invoice_date: Optional[date] = Field(None, alias="invoiceDate", description="**Note:** This field has been replaced by the `documentDate` field in Zuora REST API version `215.0` and later. The `invoiceDate` field is only available for backward compatibility.  The date that should appear on the invoice being generated, in `yyyy-mm-dd` format. If this field is omitted and `invoiceId` is not specified, the current date is used by default.   This field is in Zuora REST API version control. Supported minor versions are `214.0` and earlier. ")
    invoice_id: Optional[StrictStr] = Field(None, alias="invoiceId", description="The ID of an existing invoice for which to collect payment using the account's default payment method. If this value is specified, no new invoice is generated, and the following fields are ignored:   - `invoiceDate` and `invoiceTargetDate` (if the Zuora REST API version is 214.0 or earlier)   - `documentDate` and `targetDate` (if the Zuora REST API version is 215.0 or later) ")
    invoice_number: Optional[StrictStr] = Field(None, alias="invoiceNumber", description="The number of an existing invoice for which to collect payment using the account's default payment method. If this value is specified, no new invoice is generated, and the following fields are ignored:   - `invoiceDate` and `invoiceTargetDate` (if the Zuora REST API version is 214.0 or earlier)   - `documentDate` and `targetDate` (if the Zuora REST API version is 215.0 or later) ")
    invoice_target_date: Optional[date] = Field(None, alias="invoiceTargetDate", description="**Note:** This field has been replaced by the `targetDate` field in Zuora REST API version `215.0` and later. The `invoiceTargetDate` field is only available for backward compatibility.   The date through which to calculate charges on this account if an invoice is generated, in `yyyy-mm-dd` format. If this field is omitted and `invoiceId` is not specified, the current date is used by default.    This field is in Zuora REST API version control. Supported minor versions are `214.0` and earlier.   ")
    payment_gateway: Optional[StrictStr] = Field(None, alias="paymentGateway", description="The name of the gateway that will be used for the payment. Must be a valid gateway name and the gateway must support the specific payment method. If a value is not specified, the default gateway on the Account will be used. ")
    target_date: Optional[date] = Field(None, alias="targetDate", description="The date through which to calculate charges on this account if an invoice or a credit memo is generated,  in `yyyy-mm-dd` format. If this field is omitted and `invoiceId` is not specified, the current date is used by default.    **Note:** The credit memo is only available if you have the Invoice Settlement feature enabled.   This field is in Zuora REST API version control. Supported minor versions are `215.0` and later. To use this field in the method, you must set the  `zuora-version` parameter to the minor version number in the request header. ")
    payment_method_id: Optional[StrictStr] = Field(None, alias="paymentMethodId", description="paymentMethodId. ")
    __properties = ["accountKey", "documentDate", "invoiceDate", "invoiceId", "invoiceNumber", "invoiceTargetDate", "paymentGateway", "targetDate", "paymentMethodId"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CreateInvoiceCollectRequest:
        """Create an instance of CreateInvoiceCollectRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> CreateInvoiceCollectRequest:
        """Create an instance of CreateInvoiceCollectRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return CreateInvoiceCollectRequest.parse_obj(obj)

        _obj = CreateInvoiceCollectRequest.parse_obj({
            "account_key": obj.get("accountKey"),
            "document_date": obj.get("documentDate"),
            "invoice_date": obj.get("invoiceDate"),
            "invoice_id": obj.get("invoiceId"),
            "invoice_number": obj.get("invoiceNumber"),
            "invoice_target_date": obj.get("invoiceTargetDate"),
            "payment_gateway": obj.get("paymentGateway"),
            "target_date": obj.get("targetDate"),
            "payment_method_id": obj.get("paymentMethodId")
        })
        return _obj


