# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, StrictStr, conlist, constr
from com.zuora.models.open_payment_method_type_request_fields import OpenPaymentMethodTypeRequestFields

class CreateOpenPaymentMethodTypeRequest(BaseModel):
    """
    CreateOpenPaymentMethodTypeRequest
    """
    entity_id: Optional[StrictStr] = Field(None, alias="entityId", description="If this custom payment method type is specific to one entity only, provide the entity ID in this field in UUID format, such as `123e4567-e89b-12d3-a456-426614174000`. If no entity UUID is provided, the custom payment method type is available to the global entity and all the sub entities in the tenant.  You can get the entity ID through the [Multi-entity: List entities](https://www.zuora.com/developer/api-references/older-api/operation/Get_Entities/) API operation or the **Manage Entity Profile** administration setting in the UI. To convert the format of the entity ID to UUID, separate the entity ID string in five groups with hyphens, in the form `<8-characters>-<4-characters>-<4-characters>-<4-characters>-<12-characters>` for a total of 36 characters.  Note: After the custom payment method type is created, you can only update this field to be empty. ")
    fields: conlist(OpenPaymentMethodTypeRequestFields) = Field(..., description="An array containing field metadata of the custom payment method type.  Notes:   - All the following nested metadata must be provided in the request to define a field.    - At least one field must be defined in the fields array for a custom payment method type.    - Up to 20 fields can be defined in the fields array for a custom payment method type. ")
    internal_name: constr(strict=True, max_length=19) = Field(..., alias="internalName", description="A string to identify the custom payment method type in the API name of the payment method type.  This field must be alphanumeric, starting with a capital letter, excluding JSON preserved characters such as  * \\ ’ ”. Additionally, '_' or '-' is not allowed.  This field must be unique in a tenant.  This field is used along with the `tenantId` field by the system to construct and generate the API name of the custom payment method type in the following way:  `<internalName>__c_<tenantId>`  For example, if `internalName` is `AmazonPay`, and `tenantId` is `12368`, the API name of the custom payment method type will be `AmazonPay__c_12368`.  This field cannot be updated after the creation of the custom payment method type. ")
    label: constr(strict=True, max_length=40) = Field(..., description="The label that is used to refer to this type in the Zuora UI.  This value must be alphanumeric, excluding JSON preserved characters such as  * \\ ’ ”  ")
    method_reference_id_field: StrictStr = Field(..., alias="methodReferenceIdField", description="The identification reference of the custom payment method.  This field should be mapped to a field name defined in the `fields` array for the purpose of being used as a filter in reporting tools such as Payment Method Data Source Exports and Data Query.  This field cannot be updated after the creation of the custom payment method type. ")
    sub_type_field: Optional[StrictStr] = Field(None, alias="subTypeField", description="The identification reference indicating the subtype of the custom payment method.  This field should be mapped to a field name defined in the `fields` array for the purpose of being used as a filter in reporting tools such as Data Source Exports and Data Query.  This field cannot be updated after the creation of the custom payment method type. ")
    tenant_id: StrictStr = Field(..., alias="tenantId", description="Zuora tenant ID. If multi-entity is enabled in your tenant, this is the ID of the parent tenant of all the sub entities.  This field cannot be updated after the creation of the custom payment method type. ")
    user_reference_id_field: Optional[StrictStr] = Field(None, alias="userReferenceIdField", description="The identification reference of the user or customer account.  This field should be mapped to a field name defined in the `fields` array for the purpose of being used as a filter in reporting tools such as Data Source Exports and Data Query.  This field cannot be updated after the creation of the custom payment method type. ")
    __properties = ["entityId", "fields", "internalName", "label", "methodReferenceIdField", "subTypeField", "tenantId", "userReferenceIdField"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CreateOpenPaymentMethodTypeRequest:
        """Create an instance of CreateOpenPaymentMethodTypeRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in fields (list)
        _items = []
        if self.fields:
            for _item in self.fields:
                if _item:
                    _items.append(_item.to_dict())
            _dict['fields'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> CreateOpenPaymentMethodTypeRequest:
        """Create an instance of CreateOpenPaymentMethodTypeRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return CreateOpenPaymentMethodTypeRequest.parse_obj(obj)

        _obj = CreateOpenPaymentMethodTypeRequest.parse_obj({
            "entity_id": obj.get("entityId"),
            "fields": [OpenPaymentMethodTypeRequestFields.from_dict(_item) for _item in obj.get("fields")] if obj.get("fields") is not None else None,
            "internal_name": obj.get("internalName"),
            "label": obj.get("label"),
            "method_reference_id_field": obj.get("methodReferenceIdField"),
            "sub_type_field": obj.get("subTypeField"),
            "tenant_id": obj.get("tenantId"),
            "user_reference_id_field": obj.get("userReferenceIdField")
        })
        return _obj


