# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from typing import Any, Dict, List, Optional
from pydantic import BaseModel, Field, StrictStr, conlist, constr
from com.zuora.models.create_order_order_line_item import CreateOrderOrderLineItem
from com.zuora.models.create_order_subscription import CreateOrderSubscription
from com.zuora.models.order_create_account import OrderCreateAccount
from com.zuora.models.order_scheduling_options import OrderSchedulingOptions
from com.zuora.models.processing_options_with_delayed_capture_payment import ProcessingOptionsWithDelayedCapturePayment

class CreateOrderRequest(BaseModel):
    """
    CreateOrderRequest
    """
    category: Optional[StrictStr] = None
    custom_fields: Optional[Dict[str, Dict[str, Any]]] = Field(None, alias="customFields", description="Container for custom fields of an Order object. ")
    description: Optional[constr(strict=True, max_length=500)] = Field(None, description="A description of the order.")
    existing_account_number: Optional[constr(strict=True, max_length=70)] = Field(None, alias="existingAccountNumber", description="The account number that this order will be created under. It can be either the accountNumber or the account info provided. It will return an error if both are specified. Note that this actually specifies the invoice owner account of the subscriptions included in this order. ")
    externally_managed_by: Optional[StrictStr] = Field(None, alias="externallyManagedBy")
    new_account: Optional[OrderCreateAccount] = Field(None, alias="newAccount")
    order_date: date = Field(..., alias="orderDate", description="The date when the order is signed. All the order actions under this order will use this order date as the contract effective date if the contract effective date field is skipped or its value is left as null.")
    order_line_items: Optional[conlist(CreateOrderOrderLineItem)] = Field(None, alias="orderLineItems", description="[Order Line Items](https://knowledgecenter.zuora.com/Billing/Subscriptions/Orders/Order_Line_Items/AA_Overview_of_Order_Line_Items) are non subscription based items created by an Order, representing transactional charges such as one-time fees, physical goods, or professional service charges that are not sold as subscription services.   With the Order Line Items feature enabled, you can now launch non-subscription and unified monetization business models in Zuora, in addition to subscription business models.  **Note:** The [Order Line Items](https://knowledgecenter.zuora.com/Billing/Subscriptions/Orders/Order_Line_Items/AA_Overview_of_Order_Line_Items) feature is now generally available to all Zuora customers. You need to enable the [Orders](https://knowledgecenter.zuora.com/BC_Subscription_Management/Orders/AA_Overview_of_Orders#Orders) feature to access the [Order Line Items](https://knowledgecenter.zuora.com/Billing/Subscriptions/Orders/Order_Line_Items/AA_Overview_of_Order_Line_Items) feature. As of Zuora Billing Release 313 (November 2021), new customers who onboard on [Orders](https://knowledgecenter.zuora.com/Billing/Subscriptions/Orders/AA_Overview_of_Orders) will have the [Order Line Items](https://knowledgecenter.zuora.com/Billing/Subscriptions/Orders/Order_Line_Items) feature enabled by default.         ")
    order_number: Optional[constr(strict=True, max_length=100)] = Field(None, alias="orderNumber", description="The order number of the new order. If not provided, system will auto-generate a number for this order.    **Note:** Make sure the order number does not contain a slash.  ")
    processing_options: Optional[ProcessingOptionsWithDelayedCapturePayment] = Field(None, alias="processingOptions")
    reason_code: Optional[constr(strict=True, max_length=255)] = Field(None, alias="reasonCode", description="Values of reason code configured in **Billing Settings** > **Configure Reason Codes** through Zuora UI. Indicates the reason when a return order line item occurs. ")
    scheduling_options: Optional[OrderSchedulingOptions] = Field(None, alias="schedulingOptions")
    status: Optional[StrictStr] = None
    subscriptions: Optional[conlist(CreateOrderSubscription)] = Field(None, description="Each item includes a set of order actions, which will be applied to the same base subscription.")
    __properties = ["category", "customFields", "description", "existingAccountNumber", "externallyManagedBy", "newAccount", "orderDate", "orderLineItems", "orderNumber", "processingOptions", "reasonCode", "schedulingOptions", "status", "subscriptions"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CreateOrderRequest:
        """Create an instance of CreateOrderRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of new_account
        if self.new_account:
            _dict['newAccount'] = self.new_account.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in order_line_items (list)
        _items = []
        if self.order_line_items:
            for _item in self.order_line_items:
                if _item:
                    _items.append(_item.to_dict())
            _dict['orderLineItems'] = _items
        # override the default output from pydantic by calling `to_dict()` of processing_options
        if self.processing_options:
            _dict['processingOptions'] = self.processing_options.to_dict()
        # override the default output from pydantic by calling `to_dict()` of scheduling_options
        if self.scheduling_options:
            _dict['schedulingOptions'] = self.scheduling_options.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in subscriptions (list)
        _items = []
        if self.subscriptions:
            for _item in self.subscriptions:
                if _item:
                    _items.append(_item.to_dict())
            _dict['subscriptions'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> CreateOrderRequest:
        """Create an instance of CreateOrderRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return CreateOrderRequest.parse_obj(obj)

        _obj = CreateOrderRequest.parse_obj({
            "category": obj.get("category"),
            "custom_fields": obj.get("customFields"),
            "description": obj.get("description"),
            "existing_account_number": obj.get("existingAccountNumber"),
            "externally_managed_by": obj.get("externallyManagedBy"),
            "new_account": OrderCreateAccount.from_dict(obj.get("newAccount")) if obj.get("newAccount") is not None else None,
            "order_date": obj.get("orderDate"),
            "order_line_items": [CreateOrderOrderLineItem.from_dict(_item) for _item in obj.get("orderLineItems")] if obj.get("orderLineItems") is not None else None,
            "order_number": obj.get("orderNumber"),
            "processing_options": ProcessingOptionsWithDelayedCapturePayment.from_dict(obj.get("processingOptions")) if obj.get("processingOptions") is not None else None,
            "reason_code": obj.get("reasonCode"),
            "scheduling_options": OrderSchedulingOptions.from_dict(obj.get("schedulingOptions")) if obj.get("schedulingOptions") is not None else None,
            "status": obj.get("status"),
            "subscriptions": [CreateOrderSubscription.from_dict(_item) for _item in obj.get("subscriptions")] if obj.get("subscriptions") is not None else None
        })
        return _obj


