# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from typing import List, Optional, Union
from pydantic import BaseModel, Field, StrictFloat, StrictInt, StrictStr, conlist, constr
from com.zuora.models.apply_payment_debit_memo_application import ApplyPaymentDebitMemoApplication
from com.zuora.models.apply_payment_invoice_application import ApplyPaymentInvoiceApplication
from com.zuora.models.gateway_options import GatewayOptions
from com.zuora.models.refund_request_finance_information import RefundRequestFinanceInformation

class CreateRefundwithAutoUnapply(BaseModel):
    """
    CreateRefundwithAutoUnapply
    """
    comment: Optional[constr(strict=True, max_length=255, min_length=0)] = Field(None, description="Comments about the refund. ")
    debit_memos: Optional[conlist(ApplyPaymentDebitMemoApplication)] = Field(None, alias="debitMemos", description="Container for debit memos. The maximum number of debit memos is 1,000. ")
    finance_information: Optional[RefundRequestFinanceInformation] = Field(None, alias="financeInformation")
    gateway_options: Optional[GatewayOptions] = Field(None, alias="gatewayOptions")
    invoices: Optional[conlist(ApplyPaymentInvoiceApplication)] = Field(None, description="Container for invoices. The maximum number of invoices is 1,000. ")
    method_type: Optional[StrictStr] = Field(None, alias="methodType")
    reason_code: Optional[StrictStr] = Field(None, alias="reasonCode", description="A code identifying the reason for the transaction. The value must be an existing reason code or empty. If you do not specify a value, Zuora uses the default reason code. ")
    reference_id: Optional[constr(strict=True, max_length=100, min_length=0)] = Field(None, alias="referenceId", description="The transaction ID returned by the payment gateway for an electronic refund. Use this field to reconcile refunds between your gateway and Zuora Payments. ")
    refund_date: Optional[date] = Field(None, alias="refundDate", description="The date when the refund takes effect, in `yyyy-mm-dd` format. The date of the refund cannot be before the payment date. Specify this field only for external refunds. Zuora automatically generates this field for electronic refunds. ")
    second_refund_reference_id: Optional[constr(strict=True, max_length=100, min_length=0)] = Field(None, alias="secondRefundReferenceId", description="The transaction ID returned by the payment gateway if there is an additional transaction for the refund. Use this field to reconcile payments between your gateway and Zuora Payments. ")
    soft_descriptor: Optional[constr(strict=True, max_length=35)] = Field(None, alias="softDescriptor", description="A payment gateway-specific field that maps to Zuora for the gateways, Orbital, Vantiv and Verifi.")
    soft_descriptor_phone: Optional[constr(strict=True, max_length=20)] = Field(None, alias="softDescriptorPhone", description="A payment gateway-specific field that maps to Zuora for the gateways, Orbital, Vantiv and Verifi.")
    total_amount: Union[StrictFloat, StrictInt] = Field(..., alias="totalAmount", description="The total amount of the refund.     - `Full Refund`: If the refund amount and debit memo/ invoice are not specified, then the payment will be unapplied completely, followed by processing a full refund.   - `Partial Refund`:        - If the total amount is specified, and the debit memo/invoice is not specified, you can unapply the refund amount from the available debit memo/invoice and refund the unapplied payment to the customer.        - If the total amount is specified, along with the debit memo and the invoice, you can unapply the applied payments from the mentioned invoices and debit memos, and refund the unapplied payments to customers.   ")
    type: StrictStr = Field(...)
    refund_transaction_type: Optional[StrictStr] = Field(None, alias="refundTransactionType")
    additional_properties: Dict[str, Any] = {}
    __properties = ["comment", "debitMemos", "financeInformation", "gatewayOptions", "invoices", "methodType", "reasonCode", "referenceId", "refundDate", "secondRefundReferenceId", "softDescriptor", "softDescriptorPhone", "totalAmount", "type", "refundTransactionType"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CreateRefundwithAutoUnapply:
        """Create an instance of CreateRefundwithAutoUnapply from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in debit_memos (list)
        _items = []
        if self.debit_memos:
            for _item in self.debit_memos:
                if _item:
                    _items.append(_item.to_dict())
            _dict['debitMemos'] = _items
        # override the default output from pydantic by calling `to_dict()` of finance_information
        if self.finance_information:
            _dict['financeInformation'] = self.finance_information.to_dict()
        # override the default output from pydantic by calling `to_dict()` of gateway_options
        if self.gateway_options:
            _dict['gatewayOptions'] = self.gateway_options.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in invoices (list)
        _items = []
        if self.invoices:
            for _item in self.invoices:
                if _item:
                    _items.append(_item.to_dict())
            _dict['invoices'] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> CreateRefundwithAutoUnapply:
        """Create an instance of CreateRefundwithAutoUnapply from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return CreateRefundwithAutoUnapply.parse_obj(obj)

        _obj = CreateRefundwithAutoUnapply.parse_obj({
            "comment": obj.get("comment"),
            "debit_memos": [ApplyPaymentDebitMemoApplication.from_dict(_item) for _item in obj.get("debitMemos")] if obj.get("debitMemos") is not None else None,
            "finance_information": RefundRequestFinanceInformation.from_dict(obj.get("financeInformation")) if obj.get("financeInformation") is not None else None,
            "gateway_options": GatewayOptions.from_dict(obj.get("gatewayOptions")) if obj.get("gatewayOptions") is not None else None,
            "invoices": [ApplyPaymentInvoiceApplication.from_dict(_item) for _item in obj.get("invoices")] if obj.get("invoices") is not None else None,
            "method_type": obj.get("methodType"),
            "reason_code": obj.get("reasonCode"),
            "reference_id": obj.get("referenceId"),
            "refund_date": obj.get("refundDate"),
            "second_refund_reference_id": obj.get("secondRefundReferenceId"),
            "soft_descriptor": obj.get("softDescriptor"),
            "soft_descriptor_phone": obj.get("softDescriptorPhone"),
            "total_amount": obj.get("totalAmount"),
            "type": obj.get("type"),
            "refund_transaction_type": obj.get("refundTransactionType")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj


