# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from typing import Optional, Union
from pydantic import BaseModel, Field, StrictFloat, StrictInt, StrictStr
from com.zuora.models.create_taxation_item_for_credit_memo_finance_information import CreateTaxationItemForCreditMemoFinanceInformation

class CreateTaxationItemForCreditMemoRequest(BaseModel):
    """
    CreateTaxationItemForCreditMemoRequest
    """
    exempt_amount: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="exemptAmount", description="The calculated tax amount excluded due to the exemption. ")
    finance_information: Optional[CreateTaxationItemForCreditMemoFinanceInformation] = Field(None, alias="financeInformation")
    jurisdiction: StrictStr = Field(..., description="The jurisdiction that applies the tax or VAT. This value is typically a state, province, county, or city. ")
    location_code: Optional[StrictStr] = Field(None, alias="locationCode", description="The identifier for the location based on the value of the `taxCode` field. ")
    memo_item_id: Optional[StrictStr] = Field(None, alias="memoItemId", description="The ID of the credit memo that the taxation item is created for. ")
    name: StrictStr = Field(..., description="The name of the taxation item. ")
    source_tax_item_id: Optional[StrictStr] = Field(None, alias="sourceTaxItemId", description="The ID of the taxation item of the invoice, which the credit memo is created from.   If you want to use this REST API to create taxation items for a credit memo created from an invoice, the taxation items of the invoice must be created or imported through the SOAP API call.  **Note:**    - This field is only used if the credit memo is created from an invoice.    - If you do not contain this field in the request body, Zuora will automatically set a value for the `sourceTaxItemId` field based on the tax location code, tax jurisdiction, and tax rate. ")
    tax_amount: Union[StrictFloat, StrictInt] = Field(..., alias="taxAmount", description="The amount of the tax applied to the credit memo. ")
    applicable_tax_un_rounded: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="applicableTaxUnRounded", description="The unrounded amount of the tax. ")
    country: Optional[StrictStr] = Field(None, description="The field which contains country code. ")
    tax_code: Optional[StrictStr] = Field(None, alias="taxCode", description="The tax code identifies which tax rules and tax rates to apply to a specific credit memo. ")
    tax_code_description: Optional[StrictStr] = Field(None, alias="taxCodeDescription", description="The description of the tax code. ")
    tax_date: Optional[date] = Field(None, alias="taxDate", description="The date when the tax is applied to the credit memo. ")
    tax_rate: Union[StrictFloat, StrictInt] = Field(..., alias="taxRate", description="The tax rate applied to the credit memo. ")
    tax_rate_description: Optional[StrictStr] = Field(None, alias="taxRateDescription", description="The description of the tax rate. ")
    tax_rate_type: StrictStr = Field(..., alias="taxRateType")
    additional_properties: Dict[str, Any] = {}
    __properties = ["exemptAmount", "financeInformation", "jurisdiction", "locationCode", "memoItemId", "name", "sourceTaxItemId", "taxAmount", "applicableTaxUnRounded", "country", "taxCode", "taxCodeDescription", "taxDate", "taxRate", "taxRateDescription", "taxRateType"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CreateTaxationItemForCreditMemoRequest:
        """Create an instance of CreateTaxationItemForCreditMemoRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of finance_information
        if self.finance_information:
            _dict['financeInformation'] = self.finance_information.to_dict()
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> CreateTaxationItemForCreditMemoRequest:
        """Create an instance of CreateTaxationItemForCreditMemoRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return CreateTaxationItemForCreditMemoRequest.parse_obj(obj)

        _obj = CreateTaxationItemForCreditMemoRequest.parse_obj({
            "exempt_amount": obj.get("exemptAmount"),
            "finance_information": CreateTaxationItemForCreditMemoFinanceInformation.from_dict(obj.get("financeInformation")) if obj.get("financeInformation") is not None else None,
            "jurisdiction": obj.get("jurisdiction"),
            "location_code": obj.get("locationCode"),
            "memo_item_id": obj.get("memoItemId"),
            "name": obj.get("name"),
            "source_tax_item_id": obj.get("sourceTaxItemId"),
            "tax_amount": obj.get("taxAmount"),
            "applicable_tax_un_rounded": obj.get("applicableTaxUnRounded"),
            "country": obj.get("country"),
            "tax_code": obj.get("taxCode"),
            "tax_code_description": obj.get("taxCodeDescription"),
            "tax_date": obj.get("taxDate"),
            "tax_rate": obj.get("taxRate"),
            "tax_rate_description": obj.get("taxRateDescription"),
            "tax_rate_type": obj.get("taxRateType")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj


