# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from typing import Optional, Union
from pydantic import BaseModel, Field, StrictFloat, StrictInt, StrictStr, constr

class CreateUsageRequest(BaseModel):
    """
    CreateUsageRequest
    """
    account_id: Optional[StrictStr] = Field(None, alias="AccountId", description=" The ID of the account associated with the usage data. This field is only required if no value is specified for the `AccountNumber` field. **Character limit**: 32 **Values**: a valid account ID.")
    account_number: Optional[StrictStr] = Field(None, alias="AccountNumber", description=" The number of the account associated with the usage data. This field is only required if no value is specified for the `AccountId` field. **Character limit**: 50 **Values**: a valid account number.")
    subscription_id: Optional[constr(strict=True, max_length=32)] = Field(None, alias="SubscriptionId", description="The original ID of the subscription that contains the fees related to the usage data.   The ID of a subscription might change when you create amendments to the subscription. It is good practice to use the unique subscription number that you can specify in the `SubscriptionNumber` field. ")
    subscription_number: Optional[constr(strict=True, max_length=100)] = Field(None, alias="SubscriptionNumber", description="The unique identifier number of the subscription that contains the fees related to the usage data.  It is good practice to use this field when creating usage records. ")
    charge_id: Optional[StrictStr] = Field(None, alias="ChargeId", description=" The OrginalId of the rate plan charge related to the usage record, e.g., `2c9081a03c63c94c013c6873357a0117` **Character limit**: 32 **Values**: a valid rate plan charge OriginalID. ")
    charge_number: Optional[constr(strict=True, max_length=50)] = Field(None, alias="ChargeNumber", description="A unique number for the rate plan charge related to the usage record. For example, C-00000007. ")
    start_date_time: datetime = Field(..., alias="StartDateTime", description=" The start date and time of a range of time when usage is tracked. Zuora uses this field value to determine the usage date. Unlike the `EndDateTime`, the `StartDateTime` field does affect usage calculation. **Character limit**: 29 **Values**: a valid date and time value")
    end_date_time: Optional[datetime] = Field(None, alias="EndDateTime", description=" The end date and time of a range of time when usage is tracked. Use this field for reporting; this field doesn't affect usage calculation. **Character limit**: 29 **Values**: a valid date and time value.")
    uom: StrictStr = Field(..., alias="UOM", description=" Specifies the units to measure usage. Units of measure are configured in the web-based UI. Your values depend on your configuration in **Billing Settings**. **Character limit**: **Values**: a valid unit of measure")
    quantity: Union[StrictFloat, StrictInt] = Field(..., alias="Quantity", description=" Indicates the number of units used. **Character limit**: 16 **Values**: a valid decimal amount equal to or greater than 0")
    description: Optional[constr(strict=True, max_length=200)] = Field(None, alias="Description", description="A description of the usage record. ")
    additional_properties: Dict[str, Any] = {}
    __properties = ["AccountId", "AccountNumber", "SubscriptionId", "SubscriptionNumber", "ChargeId", "ChargeNumber", "StartDateTime", "EndDateTime", "UOM", "Quantity", "Description"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CreateUsageRequest:
        """Create an instance of CreateUsageRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> CreateUsageRequest:
        """Create an instance of CreateUsageRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return CreateUsageRequest.parse_obj(obj)

        _obj = CreateUsageRequest.parse_obj({
            "account_id": obj.get("AccountId"),
            "account_number": obj.get("AccountNumber"),
            "subscription_id": obj.get("SubscriptionId"),
            "subscription_number": obj.get("SubscriptionNumber"),
            "charge_id": obj.get("ChargeId"),
            "charge_number": obj.get("ChargeNumber"),
            "start_date_time": obj.get("StartDateTime"),
            "end_date_time": obj.get("EndDateTime"),
            "uom": obj.get("UOM"),
            "quantity": obj.get("Quantity"),
            "description": obj.get("Description")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj


