# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from typing import List, Optional, Union
from pydantic import BaseModel, Field, StrictBool, StrictFloat, StrictInt, StrictStr, conlist
from com.zuora.models.credit_memo_item_from_invoice_item_finance_information import CreditMemoItemFromInvoiceItemFinanceInformation
from com.zuora.models.credit_memo_tax_item_from_invoice_tax_item_request import CreditMemoTaxItemFromInvoiceTaxItemRequest

class CreditMemoItemFromInvoiceItem(BaseModel):
    """
    CreditMemoItemFromInvoiceItem
    """
    amount: Union[StrictFloat, StrictInt] = Field(..., description="The amount of the credit memo item.")
    comment: Optional[StrictStr] = Field(None, description="Comments about the credit memo item. **Note**: This field is not available if you set the `zuora-version` request header to `257.0` or later.")
    description: Optional[StrictStr] = Field(None, description="The description of the credit memo item. **Note**: This field is only available if you set the `zuora-version` request header to `257.0` or later.")
    finance_information: Optional[CreditMemoItemFromInvoiceItemFinanceInformation] = Field(None, alias="financeInformation")
    invoice_item_id: StrictStr = Field(..., alias="invoiceItemId", description="The ID of the invoice item.")
    quantity: Optional[Union[StrictFloat, StrictInt]] = Field(None, description="The number of units for the credit memo item.")
    service_end_date: Optional[date] = Field(None, alias="serviceEndDate", description="The service end date of the credit memo item.")
    service_start_date: Optional[date] = Field(None, alias="serviceStartDate", description="The service start date of the credit memo item.")
    sku_name: StrictStr = Field(..., alias="skuName", description="The name of the charge associated with the invoice.")
    tax_items: Optional[conlist(CreditMemoTaxItemFromInvoiceTaxItemRequest)] = Field(None, alias="taxItems", description="Container for taxation items.")
    tax_mode: Optional[StrictStr] = Field(None, alias="taxMode")
    unit_of_measure: Optional[StrictStr] = Field(None, alias="unitOfMeasure", description="The definable unit that you measure when determining charges.")
    exclude_item_billing_from_revenue_accounting: Optional[StrictBool] = Field(None, alias="excludeItemBillingFromRevenueAccounting", description="The flag to exclude the credit memo item from revenue accounting.  **Note**: This field is only available if you have the Billing - Revenue Integration feature enabled. ")
    additional_properties: Dict[str, Any] = {}
    __properties = ["amount", "comment", "description", "financeInformation", "invoiceItemId", "quantity", "serviceEndDate", "serviceStartDate", "skuName", "taxItems", "taxMode", "unitOfMeasure", "excludeItemBillingFromRevenueAccounting"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CreditMemoItemFromInvoiceItem:
        """Create an instance of CreditMemoItemFromInvoiceItem from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of finance_information
        if self.finance_information:
            _dict['financeInformation'] = self.finance_information.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in tax_items (list)
        _items = []
        if self.tax_items:
            for _item in self.tax_items:
                if _item:
                    _items.append(_item.to_dict())
            _dict['taxItems'] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> CreditMemoItemFromInvoiceItem:
        """Create an instance of CreditMemoItemFromInvoiceItem from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return CreditMemoItemFromInvoiceItem.parse_obj(obj)

        _obj = CreditMemoItemFromInvoiceItem.parse_obj({
            "amount": obj.get("amount"),
            "comment": obj.get("comment"),
            "description": obj.get("description"),
            "finance_information": CreditMemoItemFromInvoiceItemFinanceInformation.from_dict(obj.get("financeInformation")) if obj.get("financeInformation") is not None else None,
            "invoice_item_id": obj.get("invoiceItemId"),
            "quantity": obj.get("quantity"),
            "service_end_date": obj.get("serviceEndDate"),
            "service_start_date": obj.get("serviceStartDate"),
            "sku_name": obj.get("skuName"),
            "tax_items": [CreditMemoTaxItemFromInvoiceTaxItemRequest.from_dict(_item) for _item in obj.get("taxItems")] if obj.get("taxItems") is not None else None,
            "tax_mode": obj.get("taxMode"),
            "unit_of_measure": obj.get("unitOfMeasure"),
            "exclude_item_billing_from_revenue_accounting": obj.get("excludeItemBillingFromRevenueAccounting")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj


