# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional, Union
from pydantic import BaseModel, Field, StrictBool, StrictFloat, StrictInt, StrictStr, conlist, validator
from com.zuora.models.get_payment_run_data_transaction_element_response import GetPaymentRunDataTransactionElementResponse

class GetPaymentRunDataElementResponse(BaseModel):
    """
    GetPaymentRunDataElementResponse
    """
    account_id: Optional[StrictStr] = Field(None, alias="accountId", description="The customer account ID specified in the `data` field when creating the payment run. ")
    account_number: Optional[StrictStr] = Field(None, alias="accountNumber", description="The customer account number specified in the `data` field when creating the payment run. ")
    amount: Optional[Union[StrictFloat, StrictInt]] = Field(None, description="The amount specified in the `data` field when creating the payment run. `null` is returned if it was not specified. ")
    amount_collected: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="amountCollected", description="The amount that is collected. ")
    amount_to_collect: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="amountToCollect", description="The amount to be collected. ")
    comment: Optional[StrictStr] = Field(None, description="The comment specified in the `data` field when creating the payment run. `null` is returned if it was not specified. ")
    currency: Optional[StrictStr] = Field(None, description="This field is only available if support for standalone payments is enabled.  The currency of the standalone payment. The currency of the standalone payment can be different from the payment currency defined in the customer account settings. ")
    document_id: Optional[StrictStr] = Field(None, alias="documentId", description="The billing document ID specified in the `data` field when creating the payment run. `null` is returned if it was not specified. ")
    document_number: Optional[StrictStr] = Field(None, alias="documentNumber", description="The billing document number specified in the `data` field when creating the payment run. `null` is returned if it was not specified. ")
    document_type: Optional[StrictStr] = Field(None, alias="documentType", description="The billing document type specified in the `data` field when creating the payment run. `null` is returned if it was not specified. ")
    error_code: Optional[StrictStr] = Field(None, alias="errorCode", description="The error code of the response. ")
    error_message: Optional[StrictStr] = Field(None, alias="errorMessage", description="The detailed information of the error response. ")
    payment_gateway_id: Optional[StrictStr] = Field(None, alias="paymentGatewayId", description="The payment gateway ID specified in the `data` field when creating the payment run. `null` is returned if it was not specified. ")
    payment_method_id: Optional[StrictStr] = Field(None, alias="paymentMethodId", description="The payment method ID specified in the `data` field when creating the payment run. `null` is returned if it was not specified. ")
    result: Optional[StrictStr] = Field(None, description="Indicates whether the data is processed successfully or not. ")
    standalone: Optional[StrictBool] = Field(None, description="This field is only available if the support for standalone payment is enabled.  The value `true` indicates this is a standalone payment that is created and processed in Zuora through Zuora gateway integration but will be settled outside of Zuora. No settlement data will be created. The standalone payment cannot be applied, unapplied, or transferred. ")
    transactions: Optional[conlist(GetPaymentRunDataTransactionElementResponse)] = Field(None, description="Container for transactions that apply to the current request. Each element contains an array of the settlement/payment applied to the record. ")
    additional_properties: Dict[str, Any] = {}
    __properties = ["accountId", "accountNumber", "amount", "amountCollected", "amountToCollect", "comment", "currency", "documentId", "documentNumber", "documentType", "errorCode", "errorMessage", "paymentGatewayId", "paymentMethodId", "result", "standalone", "transactions"]

    @validator('document_type')
    def document_type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('Invoice', 'DebitMemo'):
            raise ValueError("must be one of enum values ('Invoice', 'DebitMemo')")
        return value

    @validator('result')
    def result_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('Processed', 'Error'):
            raise ValueError("must be one of enum values ('Processed', 'Error')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> GetPaymentRunDataElementResponse:
        """Create an instance of GetPaymentRunDataElementResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in transactions (list)
        _items = []
        if self.transactions:
            for _item in self.transactions:
                if _item:
                    _items.append(_item.to_dict())
            _dict['transactions'] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> GetPaymentRunDataElementResponse:
        """Create an instance of GetPaymentRunDataElementResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return GetPaymentRunDataElementResponse.parse_obj(obj)

        _obj = GetPaymentRunDataElementResponse.parse_obj({
            "account_id": obj.get("accountId"),
            "account_number": obj.get("accountNumber"),
            "amount": obj.get("amount"),
            "amount_collected": obj.get("amountCollected"),
            "amount_to_collect": obj.get("amountToCollect"),
            "comment": obj.get("comment"),
            "currency": obj.get("currency"),
            "document_id": obj.get("documentId"),
            "document_number": obj.get("documentNumber"),
            "document_type": obj.get("documentType"),
            "error_code": obj.get("errorCode"),
            "error_message": obj.get("errorMessage"),
            "payment_gateway_id": obj.get("paymentGatewayId"),
            "payment_method_id": obj.get("paymentMethodId"),
            "result": obj.get("result"),
            "standalone": obj.get("standalone"),
            "transactions": [GetPaymentRunDataTransactionElementResponse.from_dict(_item) for _item in obj.get("transactions")] if obj.get("transactions") is not None else None
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj


