# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Dict, Optional
from pydantic import BaseModel, Field, StrictBool, StrictStr
from com.zuora.models.get_scheduled_event_response_parameters_value import GetScheduledEventResponseParametersValue

class GetScheduledEventResponse(BaseModel):
    """
    GetScheduledEventResponse
    """
    active: Optional[StrictBool] = Field(None, description="Indicate whether the scheduled event is active or inactive")
    api_field: Optional[StrictStr] = Field(None, alias="apiField", description="The base field of the base object in the `apiObject` field, should be in date or timestamp format. The scheduled event notifications are triggered based on this date and the event parameters (before or after a specified number of days) from notification definitions. Should be specified in the pattern: ^[A-Z][\\\\w\\\\-]*$")
    api_object: Optional[StrictStr] = Field(None, alias="apiObject", description="The base object that the scheduled event is defined upon. The base object should contain a date or timestamp format field. Should be specified in the pattern: ^[A-Z][\\\\w\\\\-]*$")
    condition: Optional[StrictStr] = Field(None, description="The filter rule conditions, written in [JEXL](http://commons.apache.org/proper/commons-jexl/). The scheduled event is triggered only if the condition is evaluated as true. The rule might contain event context merge fields and data source merge fields. Data source merge fields must be from [the base object of the event or from the joined objects of the base object](https://knowledgecenter.zuora.com/DC_Developers/M_Export_ZOQL#Data_Sources_and_Objects). Scheduled events with invalid merge fields will fail to evaluate, thus will not be triggered. For example, to trigger an invoice due date scheduled event to only invoices with an amount over 1000, you would define the following condition:  ```Invoice.Amount > 1000```  `Invoice.Amount` refers to the `Amount` field of the Zuora object `Invoice`. ")
    cron_expression: Optional[StrictStr] = Field(None, alias="cronExpression", description="The cron expression defines the time when scheduled event notifications will be sent.")
    description: Optional[StrictStr] = Field(None, description="The description of the scheduled event.")
    display_name: Optional[StrictStr] = Field(None, alias="displayName", description="The display name of the scheduled event.")
    id: Optional[StrictStr] = Field(None, description="Scheduled event ID.")
    name: Optional[StrictStr] = Field(None, description="The name of the scheduled event.")
    namespace: Optional[StrictStr] = Field(None, description="The namespace of the scheduled event name in the `name` field.")
    parameters: Optional[Dict[str, GetScheduledEventResponseParametersValue]] = Field(None, description="The parameter definitions of the filter rule. The names of the parameters must match with the filter rule and can't be duplicated. You should specify all the parameters when creating scheduled event notifications.")
    __properties = ["active", "apiField", "apiObject", "condition", "cronExpression", "description", "displayName", "id", "name", "namespace", "parameters"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> GetScheduledEventResponse:
        """Create an instance of GetScheduledEventResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each value in parameters (dict)
        _field_dict = {}
        if self.parameters:
            for _key in self.parameters:
                if self.parameters[_key]:
                    _field_dict[_key] = self.parameters[_key].to_dict()
            _dict['parameters'] = _field_dict
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> GetScheduledEventResponse:
        """Create an instance of GetScheduledEventResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return GetScheduledEventResponse.parse_obj(obj)

        _obj = GetScheduledEventResponse.parse_obj({
            "active": obj.get("active"),
            "api_field": obj.get("apiField"),
            "api_object": obj.get("apiObject"),
            "condition": obj.get("condition"),
            "cron_expression": obj.get("cronExpression"),
            "description": obj.get("description"),
            "display_name": obj.get("displayName"),
            "id": obj.get("id"),
            "name": obj.get("name"),
            "namespace": obj.get("namespace"),
            "parameters": dict(
                (_k, GetScheduledEventResponseParametersValue.from_dict(_v))
                for _k, _v in obj.get("parameters").items()
            )
            if obj.get("parameters") is not None
            else None
        })
        return _obj


