# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from typing import List, Optional, Union
from pydantic import BaseModel, Field, StrictBool, StrictFloat, StrictInt, StrictStr, conlist, constr
from com.zuora.models.account_basic_info import AccountBasicInfo
from com.zuora.models.contact import Contact
from com.zuora.models.externally_managed_by import ExternallyManagedBy
from com.zuora.models.failed_reason import FailedReason
from com.zuora.models.subscription_rate_plan import SubscriptionRatePlan
from com.zuora.models.subscription_status import SubscriptionStatus
from com.zuora.models.subscription_status_history import SubscriptionStatusHistory
from com.zuora.models.term_period_type import TermPeriodType

class GetSubscriptionResponse(BaseModel):
    """
    GetSubscriptionResponse
    """
    process_id: Optional[StrictStr] = Field(None, alias="processId", description="The Id of the process that handle the operation. ")
    request_id: Optional[StrictStr] = Field(None, alias="requestId", description="Unique request identifier. If you need to contact us about a specific request, providing the request identifier will ensure the fastest possible resolution. ")
    reasons: Optional[conlist(FailedReason)] = None
    success: Optional[StrictBool] = Field(None, description="Indicates whether the call succeeded. ")
    cpq_bundle_json_id__qt: Optional[constr(strict=True, max_length=32)] = Field(None, alias="CpqBundleJsonId__QT", description="The Bundle product structures from Zuora Quotes if you utilize Bundling in Salesforce. Do not change the value in this field. ")
    opportunity_close_date__qt: Optional[date] = Field(None, alias="OpportunityCloseDate__QT", description="The closing date of the Opportunity. This field is used in Zuora data sources to report on Subscription metrics. If the subscription originated from Zuora Quotes, the value is populated with the value from Zuora Quotes. ")
    opportunity_name__qt: Optional[constr(strict=True, max_length=100)] = Field(None, alias="OpportunityName__QT", description="The unique identifier of the Opportunity. This field is used in Zuora data sources to report on Subscription metrics. If the subscription originated from Zuora Quotes, the value is populated with the value from Zuora Quotes. ")
    quote_business_type__qt: Optional[constr(strict=True, max_length=32)] = Field(None, alias="QuoteBusinessType__QT", description="The specific identifier for the type of business transaction the Quote represents such as New, Upsell, Downsell, Renewal or Churn. This field is used in Zuora data sources to report on Subscription metrics. If the subscription originated from Zuora Quotes, the value is populated with the value from Zuora Quotes. ")
    quote_number__qt: Optional[constr(strict=True, max_length=32)] = Field(None, alias="QuoteNumber__QT", description="The unique identifier of the Quote. This field is used in Zuora data sources to report on Subscription metrics. If the subscription originated from Zuora Quotes, the value is populated with the value from Zuora Quotes. ")
    quote_type__qt: Optional[constr(strict=True, max_length=32)] = Field(None, alias="QuoteType__QT", description="The Quote type that represents the subscription lifecycle stage such as New, Amendment, Renew or Cancel. This field is used in Zuora data sources to report on Subscription metrics. If the subscription originated from Zuora Quotes, the value is populated with the value from Zuora Quotes. ")
    integration_id__ns: Optional[constr(strict=True, max_length=255)] = Field(None, alias="IntegrationId__NS", description="ID of the corresponding object in NetSuite. Only available if you have installed the [Zuora Connector for NetSuite](https://www.zuora.com/connect/app/?appId=265). ")
    integration_status__ns: Optional[constr(strict=True, max_length=255)] = Field(None, alias="IntegrationStatus__NS", description="Status of the subscription's synchronization with NetSuite. Only available if you have installed the [Zuora Connector for NetSuite](https://www.zuora.com/connect/app/?appId=265). ")
    project__ns: Optional[constr(strict=True, max_length=255)] = Field(None, alias="Project__NS", description="The NetSuite project that the subscription was created from. Only available if you have installed the [Zuora Connector for NetSuite](https://www.zuora.com/connect/app/?appId=265). ")
    sales_order__ns: Optional[constr(strict=True, max_length=255)] = Field(None, alias="SalesOrder__NS", description="The NetSuite sales order than the subscription was created from. Only available if you have installed the [Zuora Connector for NetSuite](https://www.zuora.com/connect/app/?appId=265). ")
    sync_date__ns: Optional[constr(strict=True, max_length=255)] = Field(None, alias="SyncDate__NS", description="Date when the subscription was synchronized with NetSuite. Only available if you have installed the [Zuora Connector for NetSuite](https://www.zuora.com/connect/app/?appId=265). ")
    id: Optional[StrictStr] = Field(None, description="Subscription ID. ")
    subscription_number: Optional[StrictStr] = Field(None, alias="subscriptionNumber", description="Subscription number.")
    account_id: Optional[StrictStr] = Field(None, alias="accountId", description="The ID of the account associated with this subscription.")
    account_name: Optional[StrictStr] = Field(None, alias="accountName", description="The name of the account associated with this subscription.")
    account_number: Optional[StrictStr] = Field(None, alias="accountNumber", description="The number of the account associated with this subscription.")
    auto_renew: Optional[StrictBool] = Field(None, alias="autoRenew", description="If `true`, the subscription automatically renews at the end of the term. Default is `false`. ")
    bill_to_contact: Optional[Contact] = Field(None, alias="billToContact")
    cancel_reason: Optional[StrictStr] = Field(None, alias="cancelReason", description="The reason for a subscription cancellation copied from the `changeReason` field of a Cancel Subscription order action.   This field contains valid value only if a subscription is cancelled through the Orders UI or API. Otherwise, the value for this field will always be `null`. ")
    contract_effective_date: Optional[date] = Field(None, alias="contractEffectiveDate", description="Effective contract date for this subscription, as yyyy-mm-dd. ")
    contracted_mrr: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="contractedMrr", description="Monthly recurring revenue of the subscription. ")
    current_term: Optional[StrictInt] = Field(None, alias="currentTerm", description="The length of the period for the current subscription term. ")
    current_term_period_type: Optional[TermPeriodType] = Field(None, alias="currentTermPeriodType")
    customer_acceptance_date: Optional[date] = Field(None, alias="customerAcceptanceDate", description="The date on which the services or products within a subscription have been accepted by the customer, as yyyy-mm-dd. ")
    currency: Optional[StrictStr] = Field(None, description="The currency of the subscription. ")
    create_time: Optional[StrictStr] = Field(None, alias="createTime", description="The date when the subscription was created, as yyyy-mm-dd HH:MM:SS. ")
    update_time: Optional[StrictStr] = Field(None, alias="updateTime", description="The date when the subscription was last updated, as yyyy-mm-dd HH:MM:SS. ")
    externally_managed_by: Optional[ExternallyManagedBy] = Field(None, alias="externallyManagedBy")
    initial_term: Optional[StrictInt] = Field(None, alias="initialTerm", description="The length of the period for the first subscription term. ")
    initial_term_period_type: Optional[TermPeriodType] = Field(None, alias="initialTermPeriodType")
    invoice_group_number: Optional[constr(strict=True, max_length=255)] = Field(None, alias="invoiceGroupNumber", description="The number of invoice group associated with the subscription.  **Note**: This field is available only if you have the <a href=\"https://knowledgecenter.zuora.com/Zuora_Billing/Bill_your_customers/Bill_customers_at_subscription_level/Flexible_Billing_Attributes\" target=\"_blank\">Flexible Billing Attributes</a> feature enabled. ")
    invoice_owner_account_id: Optional[StrictStr] = Field(None, alias="invoiceOwnerAccountId")
    invoice_owner_account_name: Optional[StrictStr] = Field(None, alias="invoiceOwnerAccountName")
    invoice_owner_account_number: Optional[StrictStr] = Field(None, alias="invoiceOwnerAccountNumber")
    invoice_schedule_id: Optional[StrictStr] = Field(None, alias="invoiceScheduleId", description="The ID of the invoice schedule associated with the subscription.  If multiple invoice schedules are created for different terms of a subscription, this field stores the latest invoice schedule.  **Note**: This field is available only if you have the <a href=\"https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/Billing_Schedule\" target=\"_blank\">Billing Schedule</a> feature in the **Early Adopter** phase enabled. ")
    invoice_separately: Optional[StrictStr] = Field(None, alias="invoiceSeparately", description="Separates a single subscription from other subscriptions and creates an invoice for the subscription.   If the value is `true`, the subscription is billed separately from other subscriptions. If the value is `false`, the subscription is included with other subscriptions in the account invoice. ")
    invoice_template_id: Optional[StrictStr] = Field(None, alias="invoiceTemplateId", description="The ID of the invoice template associated with the subscription.  **Note**:    - If you have the <a href=\"https://knowledgecenter.zuora.com/Zuora_Billing/Bill_your_customers/Bill_customers_at_subscription_level/Flexible_Billing_Attributes\" target=\"_blank\">Flexible Billing Attributes</a> feature disabled, this field is unavailable in the request body and the value of this field is `null` in the response body.    - If you have the Flexible Billing Attributes feature enabled, and you do not specify this field in the request or you select **Default Template from Account** for this field during subscription creation, the value of this field is automatically set to `null` in the response body. ")
    invoice_template_name: Optional[StrictStr] = Field(None, alias="invoiceTemplateName", description="The name of the invoice template associated with the subscription.  **Note**:    - If you have the <a href=\"https://knowledgecenter.zuora.com/Zuora_Billing/Bill_your_customers/Bill_customers_at_subscription_level/Flexible_Billing_Attributes\" target=\"_blank\">Flexible Billing Attributes</a> feature disabled, the value of this field is `null` in the response body.    - If you have the Flexible Billing Attributes feature enabled, and you do not specify the `invoiceTemplateId` field in the request or you select **Default Template from Account** for the `invoiceTemplateId` field during subscription creation, the value of the `invoiceTemplateName` field is automatically set to `null` in the response body.    ")
    is_latest_version: Optional[StrictBool] = Field(None, alias="isLatestVersion", description="If `true`, the current subscription object is the latest version.")
    last_booking_date: Optional[date] = Field(None, alias="lastBookingDate", description="The last booking date of the subscription object. This field is writable only when the subscription is newly created as a first version subscription. You can override the date value when creating a subscription through the Subscribe and Amend API or the subscription creation UI (non-Orders). Otherwise, the default value `today` is set per the user's timezone. The value of this field is as follows: * For a new subscription created by the [Subscribe and Amend APIs](https://knowledgecenter.zuora.com/Billing/Subscriptions/Orders/Orders_Harmonization/Orders_Migration_Guidance#Subscribe_and_Amend_APIs_to_Migrate), this field has the value of the subscription creation date. * For a subscription changed by an amendment, this field has the value of the amendment booking date. * For a subscription created or changed by an order, this field has the value of the order date. ")
    notes: Optional[StrictStr] = Field(None, description="A string of up to 65,535 characters. ")
    order_number: Optional[StrictStr] = Field(None, alias="orderNumber", description="The order number of the order in which the changes on the subscription are made.   **Note:** This field is only available if you have the [Order Metrics](https://knowledgecenter.zuora.com/BC_Subscription_Management/Orders/AA_Overview_of_Orders#Order_Metrics) feature enabled. If you wish to have access to the feature, submit a request at [Zuora Global Support](http://support.zuora.com/). We will investigate your use cases and data before enabling this feature for you. ")
    payment_term: Optional[StrictStr] = Field(None, alias="paymentTerm", description="The name of the payment term associated with the subscription. For example, `Net 30`. The payment term determines the due dates of invoices.  **Note**:    - If you have the <a href=\"https://knowledgecenter.zuora.com/Zuora_Billing/Bill_your_customers/Bill_customers_at_subscription_level/Flexible_Billing_Attributes\" target=\"_blank\">Flexible Billing Attributes</a> feature disabled, this field is unavailable in the request body and the value of this field is `null` in the response body.    - If you have the Flexible Billing Attributes feature enabled, and you do not specify this field in the request or you select **Default Term from Account** for this field during subscription creation, the value of this field is automatically set to `null` in the response body. ")
    rate_plans: Optional[conlist(SubscriptionRatePlan)] = Field(None, alias="ratePlans", description="Container for rate plans. ")
    renewal_setting: Optional[StrictStr] = Field(None, alias="renewalSetting", description="Specifies whether a termed subscription will remain `TERMED` or change to `EVERGREEN` when it is renewed.   Values are:  * `RENEW_WITH_SPECIFIC_TERM` (default) * `RENEW_TO_EVERGREEN` ")
    renewal_term: Optional[StrictInt] = Field(None, alias="renewalTerm", description="The length of the period for the subscription renewal term. ")
    renewal_term_period_type: Optional[TermPeriodType] = Field(None, alias="renewalTermPeriodType")
    revision: Optional[StrictStr] = Field(None, description="An auto-generated decimal value uniquely tagged with a subscription. The value always contains one decimal place, for example, the revision of a new subscription is 1.0. If a further version of the subscription is created, the revision value will be increased by 1. Also, the revision value is always incremental regardless of deletion of subscription versions. ")
    sequence_set_id: Optional[StrictStr] = Field(None, alias="sequenceSetId", description="The ID of the sequence set associated with the subscription.  **Note**:    - If you have the <a href=\"https://knowledgecenter.zuora.com/Zuora_Billing/Bill_your_customers/Bill_customers_at_subscription_level/Flexible_Billing_Attributes\" target=\"_blank\">Flexible Billing Attributes</a> feature disabled, this field is unavailable in the request body and the value of this field is `null` in the response body.    - If you have the Flexible Billing Attributes feature enabled, and you do not specify this field in the request or you select **Default Set from Account** for this field during subscription creation, the value of this field is automatically set to `null` in the response body. ")
    sequence_set_name: Optional[StrictStr] = Field(None, alias="sequenceSetName", description="The name of the sequence set associated with the subscription.  **Note**:    - If you have the <a href=\"https://knowledgecenter.zuora.com/Zuora_Billing/Bill_your_customers/Bill_customers_at_subscription_level/Flexible_Billing_Attributes\" target=\"_blank\">Flexible Billing Attributes</a> feature disabled, the value of this field is `null` in the response body.    - If you have the Flexible Billing Attributes feature enabled, and you do not specify the `sequenceSetId` field in the request or you select **Default Template from Account** for the `sequenceSetId` field during subscription creation, the value of the `sequenceSetName` field is automatically set to `null` in the response body. ")
    service_activation_date: Optional[date] = Field(None, alias="serviceActivationDate", description="The date on which the services or products within a subscription have been activated and access has been provided to the customer, as yyyy-mm-dd ")
    sold_to_contact: Optional[Contact] = Field(None, alias="soldToContact")
    status: Optional[SubscriptionStatus] = None
    status_history: Optional[conlist(SubscriptionStatusHistory)] = Field(None, alias="statusHistory", description="Container for status history. ")
    subscription_start_date: Optional[date] = Field(None, alias="subscriptionStartDate", description="Date the subscription becomes effective. ")
    subscription_end_date: Optional[date] = Field(None, alias="subscriptionEndDate", description="The date when the subscription term ends, where the subscription ends at midnight the day before. For example, if the `subscriptionEndDate` is 12/31/2016, the subscriptions ends at midnight (00:00:00 hours) on 12/30/2016. This date is the same as the term end date or the cancelation date, as appropriate. ")
    term_end_date: Optional[date] = Field(None, alias="termEndDate", description="Date the subscription term ends. If the subscription is evergreen, this is null or is the cancellation date (if one has been set). ")
    term_start_date: Optional[date] = Field(None, alias="termStartDate", description="Date the subscription term begins. If this is a renewal subscription, this date is different from the subscription start date. ")
    term_type: Optional[StrictStr] = Field(None, alias="termType", description="Possible values are: `TERMED`, `EVERGREEN`. ")
    total_contracted_value: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="totalContractedValue", description="Total contracted value of the subscription. ")
    version: Optional[StrictInt] = Field(None, description="This is the subscription version automatically generated by Zuora Billing. Each order or amendment creates a new version of the subscription, which incorporates the changes made in the order or amendment.")
    contracted_net_mrr: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="contractedNetMrr", description="Monthly recurring revenue of the subscription inclusive of all the discounts applicable, including the fixed-amount discounts and percentage discounts. ")
    as_of_day_gross_mrr: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="asOfDayGrossMrr", description="Monthly recurring revenue of the subscription exclusive of any discounts applicable as of specified day. ")
    as_of_day_net_mrr: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="asOfDayNetMrr", description="Monthly recurring revenue of the subscription inclusive of all the discounts applicable, including the fixed-amount discounts and percentage discounts as of specified day. ")
    account_owner_details: Optional[AccountBasicInfo] = Field(None, alias="accountOwnerDetails")
    invoice_owner_account_details: Optional[AccountBasicInfo] = Field(None, alias="invoiceOwnerAccountDetails")
    additional_properties: Dict[str, Any] = {}
    __properties = ["processId", "requestId", "reasons", "success", "CpqBundleJsonId__QT", "OpportunityCloseDate__QT", "OpportunityName__QT", "QuoteBusinessType__QT", "QuoteNumber__QT", "QuoteType__QT", "IntegrationId__NS", "IntegrationStatus__NS", "Project__NS", "SalesOrder__NS", "SyncDate__NS", "id", "subscriptionNumber", "accountId", "accountName", "accountNumber", "autoRenew", "billToContact", "cancelReason", "contractEffectiveDate", "contractedMrr", "currentTerm", "currentTermPeriodType", "customerAcceptanceDate", "currency", "createTime", "updateTime", "externallyManagedBy", "initialTerm", "initialTermPeriodType", "invoiceGroupNumber", "invoiceOwnerAccountId", "invoiceOwnerAccountName", "invoiceOwnerAccountNumber", "invoiceScheduleId", "invoiceSeparately", "invoiceTemplateId", "invoiceTemplateName", "isLatestVersion", "lastBookingDate", "notes", "orderNumber", "paymentTerm", "ratePlans", "renewalSetting", "renewalTerm", "renewalTermPeriodType", "revision", "sequenceSetId", "sequenceSetName", "serviceActivationDate", "soldToContact", "status", "statusHistory", "subscriptionStartDate", "subscriptionEndDate", "termEndDate", "termStartDate", "termType", "totalContractedValue", "version", "contractedNetMrr", "asOfDayGrossMrr", "asOfDayNetMrr", "accountOwnerDetails", "invoiceOwnerAccountDetails"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> GetSubscriptionResponse:
        """Create an instance of GetSubscriptionResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in reasons (list)
        _items = []
        if self.reasons:
            for _item in self.reasons:
                if _item:
                    _items.append(_item.to_dict())
            _dict['reasons'] = _items
        # override the default output from pydantic by calling `to_dict()` of bill_to_contact
        if self.bill_to_contact:
            _dict['billToContact'] = self.bill_to_contact.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in rate_plans (list)
        _items = []
        if self.rate_plans:
            for _item in self.rate_plans:
                if _item:
                    _items.append(_item.to_dict())
            _dict['ratePlans'] = _items
        # override the default output from pydantic by calling `to_dict()` of sold_to_contact
        if self.sold_to_contact:
            _dict['soldToContact'] = self.sold_to_contact.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in status_history (list)
        _items = []
        if self.status_history:
            for _item in self.status_history:
                if _item:
                    _items.append(_item.to_dict())
            _dict['statusHistory'] = _items
        # override the default output from pydantic by calling `to_dict()` of account_owner_details
        if self.account_owner_details:
            _dict['accountOwnerDetails'] = self.account_owner_details.to_dict()
        # override the default output from pydantic by calling `to_dict()` of invoice_owner_account_details
        if self.invoice_owner_account_details:
            _dict['invoiceOwnerAccountDetails'] = self.invoice_owner_account_details.to_dict()
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> GetSubscriptionResponse:
        """Create an instance of GetSubscriptionResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return GetSubscriptionResponse.parse_obj(obj)

        _obj = GetSubscriptionResponse.parse_obj({
            "process_id": obj.get("processId"),
            "request_id": obj.get("requestId"),
            "reasons": [FailedReason.from_dict(_item) for _item in obj.get("reasons")] if obj.get("reasons") is not None else None,
            "success": obj.get("success"),
            "cpq_bundle_json_id__qt": obj.get("CpqBundleJsonId__QT"),
            "opportunity_close_date__qt": obj.get("OpportunityCloseDate__QT"),
            "opportunity_name__qt": obj.get("OpportunityName__QT"),
            "quote_business_type__qt": obj.get("QuoteBusinessType__QT"),
            "quote_number__qt": obj.get("QuoteNumber__QT"),
            "quote_type__qt": obj.get("QuoteType__QT"),
            "integration_id__ns": obj.get("IntegrationId__NS"),
            "integration_status__ns": obj.get("IntegrationStatus__NS"),
            "project__ns": obj.get("Project__NS"),
            "sales_order__ns": obj.get("SalesOrder__NS"),
            "sync_date__ns": obj.get("SyncDate__NS"),
            "id": obj.get("id"),
            "subscription_number": obj.get("subscriptionNumber"),
            "account_id": obj.get("accountId"),
            "account_name": obj.get("accountName"),
            "account_number": obj.get("accountNumber"),
            "auto_renew": obj.get("autoRenew"),
            "bill_to_contact": Contact.from_dict(obj.get("billToContact")) if obj.get("billToContact") is not None else None,
            "cancel_reason": obj.get("cancelReason"),
            "contract_effective_date": obj.get("contractEffectiveDate"),
            "contracted_mrr": obj.get("contractedMrr"),
            "current_term": obj.get("currentTerm"),
            "current_term_period_type": obj.get("currentTermPeriodType"),
            "customer_acceptance_date": obj.get("customerAcceptanceDate"),
            "currency": obj.get("currency"),
            "create_time": obj.get("createTime"),
            "update_time": obj.get("updateTime"),
            "externally_managed_by": obj.get("externallyManagedBy"),
            "initial_term": obj.get("initialTerm"),
            "initial_term_period_type": obj.get("initialTermPeriodType"),
            "invoice_group_number": obj.get("invoiceGroupNumber"),
            "invoice_owner_account_id": obj.get("invoiceOwnerAccountId"),
            "invoice_owner_account_name": obj.get("invoiceOwnerAccountName"),
            "invoice_owner_account_number": obj.get("invoiceOwnerAccountNumber"),
            "invoice_schedule_id": obj.get("invoiceScheduleId"),
            "invoice_separately": obj.get("invoiceSeparately"),
            "invoice_template_id": obj.get("invoiceTemplateId"),
            "invoice_template_name": obj.get("invoiceTemplateName"),
            "is_latest_version": obj.get("isLatestVersion"),
            "last_booking_date": obj.get("lastBookingDate"),
            "notes": obj.get("notes"),
            "order_number": obj.get("orderNumber"),
            "payment_term": obj.get("paymentTerm"),
            "rate_plans": [SubscriptionRatePlan.from_dict(_item) for _item in obj.get("ratePlans")] if obj.get("ratePlans") is not None else None,
            "renewal_setting": obj.get("renewalSetting"),
            "renewal_term": obj.get("renewalTerm"),
            "renewal_term_period_type": obj.get("renewalTermPeriodType"),
            "revision": obj.get("revision"),
            "sequence_set_id": obj.get("sequenceSetId"),
            "sequence_set_name": obj.get("sequenceSetName"),
            "service_activation_date": obj.get("serviceActivationDate"),
            "sold_to_contact": Contact.from_dict(obj.get("soldToContact")) if obj.get("soldToContact") is not None else None,
            "status": obj.get("status"),
            "status_history": [SubscriptionStatusHistory.from_dict(_item) for _item in obj.get("statusHistory")] if obj.get("statusHistory") is not None else None,
            "subscription_start_date": obj.get("subscriptionStartDate"),
            "subscription_end_date": obj.get("subscriptionEndDate"),
            "term_end_date": obj.get("termEndDate"),
            "term_start_date": obj.get("termStartDate"),
            "term_type": obj.get("termType"),
            "total_contracted_value": obj.get("totalContractedValue"),
            "version": obj.get("version"),
            "contracted_net_mrr": obj.get("contractedNetMrr"),
            "as_of_day_gross_mrr": obj.get("asOfDayGrossMrr"),
            "as_of_day_net_mrr": obj.get("asOfDayNetMrr"),
            "account_owner_details": AccountBasicInfo.from_dict(obj.get("accountOwnerDetails")) if obj.get("accountOwnerDetails") is not None else None,
            "invoice_owner_account_details": AccountBasicInfo.from_dict(obj.get("invoiceOwnerAccountDetails")) if obj.get("invoiceOwnerAccountDetails") is not None else None
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj


