# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from typing import List, Optional
from pydantic import BaseModel, Field, StrictBool, StrictStr, condecimal, conlist, constr, validator
from com.zuora.models.billing_document_status import BillingDocumentStatus
from com.zuora.models.e_invoice_status import EInvoiceStatus
from com.zuora.models.failed_reason import FailedReason
from com.zuora.models.tax_status import TaxStatus
from com.zuora.models.transferred_to_accounting_status import TransferredToAccountingStatus

class InvoiceResponse(BaseModel):
    """
    InvoiceResponse
    """
    process_id: Optional[StrictStr] = Field(None, alias="processId", description="The Id of the process that handle the operation. ")
    request_id: Optional[StrictStr] = Field(None, alias="requestId", description="Unique request identifier. If you need to contact us about a specific request, providing the request identifier will ensure the fastest possible resolution. ")
    reasons: Optional[conlist(FailedReason)] = None
    success: Optional[StrictBool] = Field(None, description="Indicates whether the call succeeded. ")
    account_id: Optional[StrictStr] = Field(None, alias="accountId", description="The ID of the customer account associated with the invoice.")
    adjustment_amount: Optional[condecimal()] = Field(None, alias="adjustmentAmount", description="The amount of the invoice adjustments associated with the invoice.")
    amount: Optional[condecimal()] = Field(None, description="The total amount of the invoice.")
    amount_without_tax: Optional[condecimal()] = Field(None, alias="amountWithoutTax", description="The invoice amount excluding tax.")
    auto_pay: Optional[StrictBool] = Field(None, alias="autoPay", description="Whether invoices are automatically picked up for processing in the corresponding payment run.")
    balance: Optional[condecimal()] = Field(None, description="The remaining balance of the invoice after all payments, adjustments and refunds are applied.")
    bill_run_id: Optional[StrictStr] = Field(None, alias="billRunId", description="The id of bill run if the invoice is generated by a bill run.")
    bill_to_contact_id: Optional[StrictStr] = Field(None, alias="billToContactId", description="The ID of the bill-to contact associated with the invoice.")
    bill_to_contact_snapshot_id: Optional[StrictStr] = Field(None, alias="billToContactSnapshotId", description="The ID of the bill-to contact snapshot associated with the invoice.")
    comments: Optional[StrictStr] = Field(None, description="Comments about the invoice.")
    created_by_id: Optional[StrictStr] = Field(None, alias="createdById", description="The user ID of the person who created the invoice. If a bill run generated the invoice, then the value is the user ID of person who created the bill run.")
    created_date: Optional[StrictStr] = Field(None, alias="createdDate", description="The date and time when the invoice was created, in `yyyy-mm-dd hh:mm:ss` format. For example, 2017-03-01 15:31:10.")
    credit_balance_adjustment_amount: Optional[condecimal()] = Field(None, alias="creditBalanceAdjustmentAmount", description="The currency amount of the adjustment applied to the customer's credit balance.   **Note:** This field is only available if you have the Credit Balance feature enabled and the Invoice Settlement feature disabled. ")
    credit_memo_amount: Optional[condecimal()] = Field(None, alias="creditMemoAmount", description="The currency amount of all credit memos applied to this invoice. **Note:** This field is only available if you have [Invoice Settlement](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/Invoice_Settlement) enabled. The Invoice Settlement feature is generally available as of Zuora Billing Release 296 (March 2021). This feature includes Unapplied Payments, Credit and Debit Memo, and Invoice Item Settlement. If you want to enable Invoice Settlement, see [Invoice Settlement Enablement and Checklist Guide](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/Invoice_Settlement/Invoice_Settlement_Migration_Checklist_and_Guide) for more information.")
    currency: Optional[StrictStr] = Field(None, description="A currency defined in the web-based UI administrative settings.")
    discount: Optional[condecimal()] = Field(None, description="the invoice discount amount.")
    due_date: Optional[date] = Field(None, alias="dueDate", description="The date by which the payment for this invoice is due, in `yyyy-mm-dd` format.")
    e_invoice_status: Optional[StrictStr] = Field(None, alias="eInvoiceStatus")
    e_invoice_error_code: Optional[StrictStr] = Field(None, alias="eInvoiceErrorCode")
    e_invoice_error_message: Optional[StrictStr] = Field(None, alias="eInvoiceErrorMessage")
    e_invoice_file_id: Optional[StrictStr] = Field(None, alias="eInvoiceFileId")
    id: Optional[StrictStr] = Field(None, description="The unique ID of the invoice.")
    includes_one_time: Optional[StrictBool] = Field(None, alias="includesOneTime", description="Specifies whether the invoice includes one-time charges.")
    includes_recurring: Optional[StrictBool] = Field(None, alias="includesRecurring", description="Specifies whether the invoice includes recurring charges.")
    includes_usage: Optional[StrictBool] = Field(None, alias="includesUsage", description="Specifies whether the invoice includes usage charges.")
    invoice_date: Optional[date] = Field(None, alias="invoiceDate", description="The date that appears on the invoice being created.")
    invoice_group_number: Optional[constr(strict=True, max_length=255)] = Field(None, alias="invoiceGroupNumber", description="The number of invoice group associated with the invoice.  **Note**: This field is available only if you have the <a href=\"https://knowledgecenter.zuora.com/Zuora_Billing/Bill_your_customers/Bill_customers_at_subscription_level/Flexible_Billing_Attributes\" target=\"_blank\">Flexible Billing Attributes</a> feature enabled. ")
    invoice_number: Optional[StrictStr] = Field(None, alias="invoiceNumber", description="The unique identification number of the invoice.")
    last_email_sent_date: Optional[StrictStr] = Field(None, alias="lastEmailSentDate", description="The date when the invoice was last emailed.")
    payment_amount: Optional[condecimal()] = Field(None, alias="paymentAmount", description="The amount of payments applied to the invoice.")
    payment_term: Optional[StrictStr] = Field(None, alias="paymentTerm", description="The name of payment term associated with the invoice.")
    posted_by: Optional[StrictStr] = Field(None, alias="postedBy", description="The user ID of the person who moved the invoice to Posted status.")
    posted_date: Optional[date] = Field(None, alias="postedDate", description="The date when the invoice was posted.")
    refund_amount: Optional[condecimal()] = Field(None, alias="refundAmount", description="Specifies the amount of a refund that was applied against an earlier payment on the invoice.")
    sequence_set_id: Optional[StrictStr] = Field(None, alias="sequenceSetId", description="The ID of the sequence set associated with the invoice.")
    sold_to_contact_id: Optional[StrictStr] = Field(None, alias="soldToContactId", description="The ID of the sold-to contact associated with the invoice.")
    sold_to_contact_snapshot_id: Optional[StrictStr] = Field(None, alias="soldToContactSnapshotId", description="The ID of the sold-to contact snapshot associated with the invoice.")
    source: Optional[StrictStr] = Field(None, description="The source of the invoice.")
    source_id: Optional[StrictStr] = Field(None, alias="sourceId", description="The ID of the invoice source. If an invoice is generated from a bill run, the value is the number of the corresponding bill run.Otherwise, the value is `null`.")
    source_type: Optional[StrictStr] = Field(None, alias="sourceType", description="The type of the invoice source.")
    status: Optional[StrictStr] = None
    target_date: Optional[date] = Field(None, alias="targetDate", description="This date is used to determine which charges are to be billed. All charges that are to be billed on this date or prior will be included in this bill run.")
    tax_amount: Optional[condecimal()] = Field(None, alias="taxAmount", description="The amount of taxation.")
    tax_exempt_amount: Optional[condecimal()] = Field(None, alias="taxExemptAmount", description="The calculated tax amount excluded due to the exemption.")
    tax_message: Optional[StrictStr] = Field(None, alias="taxMessage", description="The message that the tax engine return if it calculates the taxes of this invoice fails.")
    tax_status: Optional[StrictStr] = Field(None, alias="taxStatus")
    template_id: Optional[StrictStr] = Field(None, alias="templateId", description="The ID of the invoice template.  - If you have the <a href=\"https://knowledgecenter.zuora.com/Zuora_Billing/Subscriptions/Flexible_Billing_Attributes\" target=\"_blank\">Flexible Billing Attributes</a> feature enabled, the value of this field depends on the configuration of the invoice template.    - If you specify an invoice template at the subscription level, the value of this field is automatically populated from the corresponding subscription.   - If you do not specify any invoice template at the subscription level, the value of this field is automatically populated from the corresponding account. - If you have the Flexible Billing Attributes feature disabled, the value of this field is `null`. ")
    transferred_to_accounting: Optional[StrictStr] = Field(None, alias="transferredToAccounting")
    updated_by_id: Optional[StrictStr] = Field(None, alias="updatedById", description="The ID of the Zuora user who last updated the invoice.")
    updated_date: Optional[StrictStr] = Field(None, alias="updatedDate", description="The date when the invoice was last updated.")
    integration_id__ns: Optional[constr(strict=True, max_length=255)] = Field(None, alias="IntegrationId__NS", description="ID of the corresponding object in NetSuite. Only available if you have installed the [Zuora Connector for NetSuite](https://www.zuora.com/connect/app/?appId=265). ")
    integration_status__ns: Optional[constr(strict=True, max_length=255)] = Field(None, alias="IntegrationStatus__NS", description="Status of the invoice's synchronization with NetSuite. Only available if you have installed the [Zuora Connector for NetSuite](https://www.zuora.com/connect/app/?appId=265). ")
    sync_date__ns: Optional[constr(strict=True, max_length=255)] = Field(None, alias="SyncDate__NS", description="Date when the invoice was synchronized with NetSuite. Only available if you have installed the [Zuora Connector for NetSuite](https://www.zuora.com/connect/app/?appId=265). ")
    additional_properties: Dict[str, Any] = {}
    __properties = ["processId", "requestId", "reasons", "success", "accountId", "adjustmentAmount", "amount", "amountWithoutTax", "autoPay", "balance", "billRunId", "billToContactId", "billToContactSnapshotId", "comments", "createdById", "createdDate", "creditBalanceAdjustmentAmount", "creditMemoAmount", "currency", "discount", "dueDate", "eInvoiceStatus", "eInvoiceErrorCode", "eInvoiceErrorMessage", "eInvoiceFileId", "id", "includesOneTime", "includesRecurring", "includesUsage", "invoiceDate", "invoiceGroupNumber", "invoiceNumber", "lastEmailSentDate", "paymentAmount", "paymentTerm", "postedBy", "postedDate", "refundAmount", "sequenceSetId", "soldToContactId", "soldToContactSnapshotId", "source", "sourceId", "sourceType", "status", "targetDate", "taxAmount", "taxExemptAmount", "taxMessage", "taxStatus", "templateId", "transferredToAccounting", "updatedById", "updatedDate", "IntegrationId__NS", "IntegrationStatus__NS", "SyncDate__NS"]

    @validator('source')
    def source_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('BillRun', 'API', 'ApiSubscribe', 'ApiAmend'):
            raise ValueError("must be one of enum values ('BillRun', 'API', 'ApiSubscribe', 'ApiAmend')")
        return value

    @validator('source_type')
    def source_type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('Subscription', 'Standalone', 'Order', 'Consolidation'):
            raise ValueError("must be one of enum values ('Subscription', 'Standalone', 'Order', 'Consolidation')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> InvoiceResponse:
        """Create an instance of InvoiceResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in reasons (list)
        _items = []
        if self.reasons:
            for _item in self.reasons:
                if _item:
                    _items.append(_item.to_dict())
            _dict['reasons'] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> InvoiceResponse:
        """Create an instance of InvoiceResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return InvoiceResponse.parse_obj(obj)

        _obj = InvoiceResponse.parse_obj({
            "process_id": obj.get("processId"),
            "request_id": obj.get("requestId"),
            "reasons": [FailedReason.from_dict(_item) for _item in obj.get("reasons")] if obj.get("reasons") is not None else None,
            "success": obj.get("success"),
            "account_id": obj.get("accountId"),
            "adjustment_amount": obj.get("adjustmentAmount"),
            "amount": obj.get("amount"),
            "amount_without_tax": obj.get("amountWithoutTax"),
            "auto_pay": obj.get("autoPay"),
            "balance": obj.get("balance"),
            "bill_run_id": obj.get("billRunId"),
            "bill_to_contact_id": obj.get("billToContactId"),
            "bill_to_contact_snapshot_id": obj.get("billToContactSnapshotId"),
            "comments": obj.get("comments"),
            "created_by_id": obj.get("createdById"),
            "created_date": obj.get("createdDate"),
            "credit_balance_adjustment_amount": obj.get("creditBalanceAdjustmentAmount"),
            "credit_memo_amount": obj.get("creditMemoAmount"),
            "currency": obj.get("currency"),
            "discount": obj.get("discount"),
            "due_date": obj.get("dueDate"),
            "e_invoice_status": obj.get("eInvoiceStatus"),
            "e_invoice_error_code": obj.get("eInvoiceErrorCode"),
            "e_invoice_error_message": obj.get("eInvoiceErrorMessage"),
            "e_invoice_file_id": obj.get("eInvoiceFileId"),
            "id": obj.get("id"),
            "includes_one_time": obj.get("includesOneTime"),
            "includes_recurring": obj.get("includesRecurring"),
            "includes_usage": obj.get("includesUsage"),
            "invoice_date": obj.get("invoiceDate"),
            "invoice_group_number": obj.get("invoiceGroupNumber"),
            "invoice_number": obj.get("invoiceNumber"),
            "last_email_sent_date": obj.get("lastEmailSentDate"),
            "payment_amount": obj.get("paymentAmount"),
            "payment_term": obj.get("paymentTerm"),
            "posted_by": obj.get("postedBy"),
            "posted_date": obj.get("postedDate"),
            "refund_amount": obj.get("refundAmount"),
            "sequence_set_id": obj.get("sequenceSetId"),
            "sold_to_contact_id": obj.get("soldToContactId"),
            "sold_to_contact_snapshot_id": obj.get("soldToContactSnapshotId"),
            "source": obj.get("source"),
            "source_id": obj.get("sourceId"),
            "source_type": obj.get("sourceType"),
            "status": obj.get("status"),
            "target_date": obj.get("targetDate"),
            "tax_amount": obj.get("taxAmount"),
            "tax_exempt_amount": obj.get("taxExemptAmount"),
            "tax_message": obj.get("taxMessage"),
            "tax_status": obj.get("taxStatus"),
            "template_id": obj.get("templateId"),
            "transferred_to_accounting": obj.get("transferredToAccounting"),
            "updated_by_id": obj.get("updatedById"),
            "updated_date": obj.get("updatedDate"),
            "integration_id__ns": obj.get("IntegrationId__NS"),
            "integration_status__ns": obj.get("IntegrationStatus__NS"),
            "sync_date__ns": obj.get("SyncDate__NS")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj


