# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from typing import List, Optional
from pydantic import BaseModel, Field, StrictBool, StrictStr, condecimal, conlist, constr
from com.zuora.models.invoice_schedule_item import InvoiceScheduleItem
from com.zuora.models.invoice_schedule_subscription import InvoiceScheduleSubscription

class InvoiceSchedule(BaseModel):
    """
    InvoiceSchedule
    """
    account_id: Optional[StrictStr] = Field(None, alias="accountId", description="The ID of the customer account that the invoice schedule belongs to. ")
    actual_amount: Optional[condecimal()] = Field(None, alias="actualAmount", description="The actual amount that needs to be billed during the processing of the invoice schedule.  By default, the actual amount is the same as the total amount. Even if order changes occur like Remove Product or Cancel Subscription, the value of the `totalAmount` field keeps unchanged. The value of the `actualAmount` field reflects the actual amount to be billed. ")
    additional_subscriptions_to_bill: Optional[conlist(StrictStr)] = Field(None, alias="additionalSubscriptionsToBill", description="A list of the numbers of the subscriptions that need to be billed together with the invoice schedule.   One invoice schedule can have at most 600 additional subscriptions. ")
    billed_amount: Optional[condecimal()] = Field(None, alias="billedAmount", description="The amount that has been billed during the processing of the invoice schedule. ")
    id: Optional[StrictStr] = Field(None, description="The unique ID of the invoice schedule. ")
    invoice_separately: Optional[StrictBool] = Field(None, alias="invoiceSeparately", description="Whether the invoice items created from the invoice schedule appears on a separate invoice when Zuora generates invoices. ")
    next_run_date: Optional[date] = Field(None, alias="nextRunDate", description="The run date of the next execution of invoice schedule. By default, the next run date is the same as run date of next pending invoice schedule item. It can be overwritten with a different date other than the default value. When the invoice schedule has completed the execution, the next run date is null. ")
    notes: Optional[constr(strict=True, max_length=255)] = Field(None, description="Comments on the invoice schedule. ")
    number: Optional[StrictStr] = Field(None, description="The sequence number of the invoice schedule. ")
    orders: Optional[conlist(StrictStr)] = Field(None, description="A list of the IDs or numbers of the orders associated with the invoice schedule. One invoice schedule can be associated with at most 10 orders. ")
    schedule_items: Optional[conlist(InvoiceScheduleItem)] = Field(None, alias="scheduleItems", description="Container for schedule items. One invoice schedule can have at most 50 invoice schedule items. ")
    specific_subscriptions: Optional[conlist(InvoiceScheduleSubscription)] = Field(None, alias="specificSubscriptions", description="A list of the numbers of specific subscriptions associated with the invoice schedule. ")
    status: Optional[StrictStr] = None
    total_amount: Optional[condecimal()] = Field(None, alias="totalAmount", description="The total amount that needs to be billed during the processing of the invoice schedule.   The value of this field keeps unchanged once invoice schedule items are created. ")
    unbilled_amount: Optional[condecimal()] = Field(None, alias="unbilledAmount", description="The amount that is waiting to be billed during the processing of the invoice schedule. ")
    additional_properties: Dict[str, Any] = {}
    __properties = ["accountId", "actualAmount", "additionalSubscriptionsToBill", "billedAmount", "id", "invoiceSeparately", "nextRunDate", "notes", "number", "orders", "scheduleItems", "specificSubscriptions", "status", "totalAmount", "unbilledAmount"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> InvoiceSchedule:
        """Create an instance of InvoiceSchedule from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in schedule_items (list)
        _items = []
        if self.schedule_items:
            for _item in self.schedule_items:
                if _item:
                    _items.append(_item.to_dict())
            _dict['scheduleItems'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in specific_subscriptions (list)
        _items = []
        if self.specific_subscriptions:
            for _item in self.specific_subscriptions:
                if _item:
                    _items.append(_item.to_dict())
            _dict['specificSubscriptions'] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> InvoiceSchedule:
        """Create an instance of InvoiceSchedule from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return InvoiceSchedule.parse_obj(obj)

        _obj = InvoiceSchedule.parse_obj({
            "account_id": obj.get("accountId"),
            "actual_amount": obj.get("actualAmount"),
            "additional_subscriptions_to_bill": obj.get("additionalSubscriptionsToBill"),
            "billed_amount": obj.get("billedAmount"),
            "id": obj.get("id"),
            "invoice_separately": obj.get("invoiceSeparately"),
            "next_run_date": obj.get("nextRunDate"),
            "notes": obj.get("notes"),
            "number": obj.get("number"),
            "orders": obj.get("orders"),
            "schedule_items": [InvoiceScheduleItem.from_dict(_item) for _item in obj.get("scheduleItems")] if obj.get("scheduleItems") is not None else None,
            "specific_subscriptions": [InvoiceScheduleSubscription.from_dict(_item) for _item in obj.get("specificSubscriptions")] if obj.get("specificSubscriptions") is not None else None,
            "status": obj.get("status"),
            "total_amount": obj.get("totalAmount"),
            "unbilled_amount": obj.get("unbilledAmount")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj


