# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, Optional
from pydantic import BaseModel, Field, StrictBool, StrictStr, conint, constr
from com.zuora.models.create_account_payment_method import CreateAccountPaymentMethod
from com.zuora.models.credit_card import CreditCard
from com.zuora.models.order_create_account_contact import OrderCreateAccountContact
from com.zuora.models.tax_info import TaxInfo

class OrderCreateAccount(BaseModel):
    """
    OrderCreateAccount
    """
    account_number: Optional[constr(strict=True, max_length=70)] = Field(None, alias="accountNumber")
    additional_email_addresses: Optional[constr(strict=True, max_length=1200)] = Field(None, alias="additionalEmailAddresses", description="List of additional email addresses to receive emailed invoices. Values should be a comma-separated list of email addresses. ")
    allow_invoice_edit: Optional[StrictBool] = Field(None, alias="allowInvoiceEdit", description="Indicates if associated invoices can be edited. Values are:   * `true` * `false` (default) ")
    auto_pay: Optional[StrictBool] = Field(None, alias="autoPay", description="Specifies whether future payments are to be automatically billed when they are due. Possible values are `true`, `false`.")
    batch: Optional[StrictStr] = None
    bill_cycle_day: conint(strict=True, le=31, ge=0) = Field(..., alias="billCycleDay", description="Day of the month that the account prefers billing periods to begin on. If set to 0, the bill cycle day will be set as \"AutoSet\".")
    bill_to_contact: OrderCreateAccountContact = Field(..., alias="billToContact")
    communication_profile_id: Optional[StrictStr] = Field(None, alias="communicationProfileId")
    credit_card: Optional[CreditCard] = Field(None, alias="creditCard")
    credit_memo_template_id: Optional[StrictStr] = Field(None, alias="creditMemoTemplateId", description="**Note:** This field is only available if you have [Invoice Settlement](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/Invoice_Settlement) enabled. The Invoice Settlement feature is generally available as of Zuora Billing Release 296 (March 2021). This feature includes Unapplied Payments, Credit and Debit Memo, and Invoice Item Settlement. If you want to enable Invoice Settlement, see [Invoice Settlement Enablement and Checklist Guide](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/Invoice_Settlement/Invoice_Settlement_Migration_Checklist_and_Guide) for more information.  The unique ID of the credit memo template, configured in **Billing Settings** > **Manage Billing Document Configuration** through the Zuora UI. For example, 2c92c08a6246fdf101626b1b3fe0144b. ")
    crm_id: Optional[constr(strict=True, max_length=100)] = Field(None, alias="crmId")
    currency: StrictStr = Field(..., description="3 uppercase character currency code.  For payment method authorization, if the `paymentMethod` > `currencyCode` field is specified, `currencyCode` is used. Otherwise, this `currency` field is used for payment method authorization. If no currency is specified for the account, the default currency of the account is then used. ")
    custom_fields: Optional[Dict[str, Any]] = Field(None, alias="customFields")
    customer_service_rep_name: Optional[constr(strict=True, max_length=50)] = Field(None, alias="customerServiceRepName", description="Name of the account's customer service representative, if applicable. ")
    debit_memo_template_id: Optional[StrictStr] = Field(None, alias="debitMemoTemplateId", description="**Note:** This field is only available if you have [Invoice Settlement](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/Invoice_Settlement) enabled. The Invoice Settlement feature is generally available as of Zuora Billing Release 296 (March 2021). This feature includes Unapplied Payments, Credit and Debit Memo, and Invoice Item Settlement. If you want to enable Invoice Settlement, see [Invoice Settlement Enablement and Checklist Guide](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/Invoice_Settlement/Invoice_Settlement_Migration_Checklist_and_Guide) for more information.  The unique ID of the debit memo template, configured in **Billing Settings** > **Manage Billing Document Configuration** through the Zuora UI. For example, 2c92c08d62470a8501626b19d24f19e2. ")
    hpm_credit_card_payment_method_id: Optional[StrictStr] = Field(None, alias="hpmCreditCardPaymentMethodId", description="The ID of the payment method associated with this account. The payment method specified for this field will be set as the default payment method of the account.  If the `autoPay` field is set to `true`, you must provide the credit card payment method ID for either this field or the `creditCard` field, but not both.  For the Credit Card Reference Transaction payment method, you can specify the payment method ID in this field or use the `paymentMethod` field to create a CC Reference Transaction payment method for an account. ")
    invoice_delivery_prefs_email: Optional[StrictBool] = Field(None, alias="invoiceDeliveryPrefsEmail", description="Specifies whether to turn on the invoice delivery method 'Email' for the new account.  Values are:   * `true` (default). Turn on the invoice delivery method 'Email' for the new account. * `false`. Turn off the invoice delivery method 'Email' for the new account. ")
    invoice_delivery_prefs_print: Optional[StrictBool] = Field(None, alias="invoiceDeliveryPrefsPrint", description="Specifies whether to turn on the invoice delivery method 'Print' for the new account. Values are:   * `true`. Turn on the invoice delivery method 'Print' for the new account. * `false` (default). Turn off the invoice delivery method 'Print' for the new account. ")
    invoice_template_id: Optional[StrictStr] = Field(None, alias="invoiceTemplateId")
    name: constr(strict=True, max_length=255) = Field(...)
    notes: Optional[constr(strict=True, max_length=65535)] = None
    parent_id: Optional[StrictStr] = Field(None, alias="parentId", description="Identifier of the parent customer account for this Account object. Use this field if you have <a href=\"https://knowledgecenter.zuora.com/Billing/Subscriptions/Customer_Accounts/A_Customer_Account_Introduction#Customer_Hierarchy\" target=\"_blank\">Customer Hierarchy</a> enabled.")
    partner_account: Optional[StrictBool] = Field(None, alias="partnerAccount", description="Whether the customer account is a partner, distributor, or reseller.  You can set this field to `true` if you have business with distributors or resellers, or operating in B2B model to manage numerous subscriptions through concurrent API requests. After this field is set to `true`, the calculation of account metrics is performed asynchronously during operations such as subscription creation, order changes, invoice generation, and payments. **Note**: This field is available only if you have the <a href=\"https://knowledgecenter.zuora.com/Zuora_Billing/Manage_customer_accounts/AAA_Overview_of_customer_accounts/Reseller_Account\" target=\"_blank\">Reseller Account</a> feature enabled. ")
    payment_gateway: Optional[constr(strict=True, max_length=40)] = Field(None, alias="paymentGateway")
    payment_method: Optional[CreateAccountPaymentMethod] = Field(None, alias="paymentMethod")
    payment_term: Optional[StrictStr] = Field(None, alias="paymentTerm", description="**Note**: If you want to specify a payment term when creating a new account, you must set a value in this field. If you do not set a value in this field, Zuora will use `Due Upon Receipt` as the value instead of the default value set in **Billing Settings** > **Payment Terms** from Zuora UI. ")
    purchase_order_number: Optional[constr(strict=True, max_length=100)] = Field(None, alias="purchaseOrderNumber", description="The number of the purchase order associated with this account. Purchase order information generally comes from customers. ")
    sales_rep: Optional[constr(strict=True, max_length=50)] = Field(None, alias="salesRep", description="The name of the sales representative associated with this account, if applicable. ")
    sequence_set_id: Optional[StrictStr] = Field(None, alias="sequenceSetId", description="The ID of the sequence set to assign to the customer account.   The billing documents to generate for this account will adopt the prefix and starting document number configured in the sequence set. ")
    sold_to_contact: Optional[OrderCreateAccountContact] = Field(None, alias="soldToContact")
    sold_to_same_as_bill_to: Optional[StrictBool] = Field(None, alias="soldToSameAsBillTo", description="Whether the sold-to contact and bill-to contact are the same entity.   The created account has the same bill-to contact and sold-to contact entity only when all the following conditions are met in the request body:  - This field is set to `true`.  - A bill-to contact is specified. - No sold-to contact is specified. ")
    tax_info: Optional[TaxInfo] = Field(None, alias="taxInfo")
    additional_properties: Dict[str, Any] = {}
    __properties = ["accountNumber", "additionalEmailAddresses", "allowInvoiceEdit", "autoPay", "batch", "billCycleDay", "billToContact", "communicationProfileId", "creditCard", "creditMemoTemplateId", "crmId", "currency", "customFields", "customerServiceRepName", "debitMemoTemplateId", "hpmCreditCardPaymentMethodId", "invoiceDeliveryPrefsEmail", "invoiceDeliveryPrefsPrint", "invoiceTemplateId", "name", "notes", "parentId", "partnerAccount", "paymentGateway", "paymentMethod", "paymentTerm", "purchaseOrderNumber", "salesRep", "sequenceSetId", "soldToContact", "soldToSameAsBillTo", "taxInfo"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> OrderCreateAccount:
        """Create an instance of OrderCreateAccount from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of bill_to_contact
        if self.bill_to_contact:
            _dict['billToContact'] = self.bill_to_contact.to_dict()
        # override the default output from pydantic by calling `to_dict()` of credit_card
        if self.credit_card:
            _dict['creditCard'] = self.credit_card.to_dict()
        # override the default output from pydantic by calling `to_dict()` of payment_method
        if self.payment_method:
            _dict['paymentMethod'] = self.payment_method.to_dict()
        # override the default output from pydantic by calling `to_dict()` of sold_to_contact
        if self.sold_to_contact:
            _dict['soldToContact'] = self.sold_to_contact.to_dict()
        # override the default output from pydantic by calling `to_dict()` of tax_info
        if self.tax_info:
            _dict['taxInfo'] = self.tax_info.to_dict()
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> OrderCreateAccount:
        """Create an instance of OrderCreateAccount from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return OrderCreateAccount.parse_obj(obj)

        _obj = OrderCreateAccount.parse_obj({
            "account_number": obj.get("accountNumber"),
            "additional_email_addresses": obj.get("additionalEmailAddresses"),
            "allow_invoice_edit": obj.get("allowInvoiceEdit"),
            "auto_pay": obj.get("autoPay"),
            "batch": obj.get("batch"),
            "bill_cycle_day": obj.get("billCycleDay"),
            "bill_to_contact": OrderCreateAccountContact.from_dict(obj.get("billToContact")) if obj.get("billToContact") is not None else None,
            "communication_profile_id": obj.get("communicationProfileId"),
            "credit_card": CreditCard.from_dict(obj.get("creditCard")) if obj.get("creditCard") is not None else None,
            "credit_memo_template_id": obj.get("creditMemoTemplateId"),
            "crm_id": obj.get("crmId"),
            "currency": obj.get("currency"),
            "custom_fields": obj.get("customFields"),
            "customer_service_rep_name": obj.get("customerServiceRepName"),
            "debit_memo_template_id": obj.get("debitMemoTemplateId"),
            "hpm_credit_card_payment_method_id": obj.get("hpmCreditCardPaymentMethodId"),
            "invoice_delivery_prefs_email": obj.get("invoiceDeliveryPrefsEmail"),
            "invoice_delivery_prefs_print": obj.get("invoiceDeliveryPrefsPrint"),
            "invoice_template_id": obj.get("invoiceTemplateId"),
            "name": obj.get("name"),
            "notes": obj.get("notes"),
            "parent_id": obj.get("parentId"),
            "partner_account": obj.get("partnerAccount"),
            "payment_gateway": obj.get("paymentGateway"),
            "payment_method": CreateAccountPaymentMethod.from_dict(obj.get("paymentMethod")) if obj.get("paymentMethod") is not None else None,
            "payment_term": obj.get("paymentTerm"),
            "purchase_order_number": obj.get("purchaseOrderNumber"),
            "sales_rep": obj.get("salesRep"),
            "sequence_set_id": obj.get("sequenceSetId"),
            "sold_to_contact": OrderCreateAccountContact.from_dict(obj.get("soldToContact")) if obj.get("soldToContact") is not None else None,
            "sold_to_same_as_bill_to": obj.get("soldToSameAsBillTo"),
            "tax_info": TaxInfo.from_dict(obj.get("taxInfo")) if obj.get("taxInfo") is not None else None
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj


