# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, StrictStr, conlist
from com.zuora.models.time_sliced_elp_net_metrics import TimeSlicedElpNetMetrics
from com.zuora.models.time_sliced_metrics import TimeSlicedMetrics
from com.zuora.models.time_sliced_net_metrics import TimeSlicedNetMetrics
from com.zuora.models.time_sliced_tcb_net_metrics import TimeSlicedTcbNetMetrics

class OrderMetric(BaseModel):
    """
    The set of order metrics for an order action.  # noqa: E501
    """
    charge_number: Optional[StrictStr] = Field(None, alias="chargeNumber")
    elp: Optional[conlist(TimeSlicedElpNetMetrics)] = Field(None, description="The extended list price which is calculated by the original product catalog list price multiplied by the delta quantity.  The `elp` nested field is only available to existing Orders customers who already have access to the field.  **Note:** The following Order Metrics have been deprecated. Any new customers who onboard on [Orders](https://knowledgecenter.zuora.com/Billing/Subscriptions/Orders/AA_Overview_of_Orders) or [Orders Harmonization](https://knowledgecenter.zuora.com/Billing/Subscriptions/Orders/Orders_Harmonization/Orders_Harmonization) will not get these metrics. * The Order ELP and Order Item objects  * The \"Generated Reason\" and \"Order Item ID\" fields in the Order MRR, Order TCB, Order TCV, and Order Quantity objects  Existing Orders customers who have these metrics will continue to be supported. ")
    mrr: Optional[conlist(TimeSlicedNetMetrics)] = None
    origin_rate_plan_id: Optional[StrictStr] = Field(None, alias="originRatePlanId")
    product_rate_plan_charge_id: Optional[StrictStr] = Field(None, alias="productRatePlanChargeId")
    product_rate_plan_id: Optional[StrictStr] = Field(None, alias="productRatePlanId")
    subscription_rate_plan_number: Optional[StrictStr] = Field(None, alias="subscriptionRatePlanNumber")
    quantity: Optional[conlist(TimeSlicedMetrics)] = None
    tcb: Optional[conlist(TimeSlicedTcbNetMetrics)] = Field(None, description="Total contracted billing which is the forecast value for the total invoice amount.")
    tcv: Optional[conlist(TimeSlicedNetMetrics)] = Field(None, description="Total contracted value.")
    __properties = ["chargeNumber", "elp", "mrr", "originRatePlanId", "productRatePlanChargeId", "productRatePlanId", "subscriptionRatePlanNumber", "quantity", "tcb", "tcv"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> OrderMetric:
        """Create an instance of OrderMetric from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in elp (list)
        _items = []
        if self.elp:
            for _item in self.elp:
                if _item:
                    _items.append(_item.to_dict())
            _dict['elp'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in mrr (list)
        _items = []
        if self.mrr:
            for _item in self.mrr:
                if _item:
                    _items.append(_item.to_dict())
            _dict['mrr'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in quantity (list)
        _items = []
        if self.quantity:
            for _item in self.quantity:
                if _item:
                    _items.append(_item.to_dict())
            _dict['quantity'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in tcb (list)
        _items = []
        if self.tcb:
            for _item in self.tcb:
                if _item:
                    _items.append(_item.to_dict())
            _dict['tcb'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in tcv (list)
        _items = []
        if self.tcv:
            for _item in self.tcv:
                if _item:
                    _items.append(_item.to_dict())
            _dict['tcv'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> OrderMetric:
        """Create an instance of OrderMetric from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return OrderMetric.parse_obj(obj)

        _obj = OrderMetric.parse_obj({
            "charge_number": obj.get("chargeNumber"),
            "elp": [TimeSlicedElpNetMetrics.from_dict(_item) for _item in obj.get("elp")] if obj.get("elp") is not None else None,
            "mrr": [TimeSlicedNetMetrics.from_dict(_item) for _item in obj.get("mrr")] if obj.get("mrr") is not None else None,
            "origin_rate_plan_id": obj.get("originRatePlanId"),
            "product_rate_plan_charge_id": obj.get("productRatePlanChargeId"),
            "product_rate_plan_id": obj.get("productRatePlanId"),
            "subscription_rate_plan_number": obj.get("subscriptionRatePlanNumber"),
            "quantity": [TimeSlicedMetrics.from_dict(_item) for _item in obj.get("quantity")] if obj.get("quantity") is not None else None,
            "tcb": [TimeSlicedTcbNetMetrics.from_dict(_item) for _item in obj.get("tcb")] if obj.get("tcb") is not None else None,
            "tcv": [TimeSlicedNetMetrics.from_dict(_item) for _item in obj.get("tcv")] if obj.get("tcv") is not None else None
        })
        return _obj


