# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional, Union
from pydantic import BaseModel, Field, StrictBool, StrictFloat, StrictInt, StrictStr, conlist
from com.zuora.models.get_payment_finance_information import GetPaymentFinanceInformation
from com.zuora.models.payment_schedule_payment_option_fields import PaymentSchedulePaymentOptionFields

class Payment(BaseModel):
    """
    Payment
    """
    account_id: Optional[StrictStr] = Field(None, alias="accountId", description="The ID of the customer account that the payment is for. ")
    account_number: Optional[StrictStr] = Field(None, alias="accountNumber", description="The number of the customer account that the payment is for. ")
    amount: Optional[Union[StrictFloat, StrictInt]] = Field(None, description="The total amount of the payment. ")
    applied_amount: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="appliedAmount", description="The applied amount of the payment. ")
    auth_transaction_id: Optional[StrictStr] = Field(None, alias="authTransactionId", description="The authorization transaction ID from the payment gateway. ")
    bank_identification_number: Optional[StrictStr] = Field(None, alias="bankIdentificationNumber", description="The first six or eight digits of the credit card or debit card used for the payment, when applicable. ")
    cancelled_on: Optional[StrictStr] = Field(None, alias="cancelledOn", description="The date and time when the payment was cancelled, in `yyyy-mm-dd hh:mm:ss` format. ")
    comment: Optional[StrictStr] = Field(None, description="Comments about the payment. ")
    created_by_id: Optional[StrictStr] = Field(None, alias="createdById", description="The ID of the Zuora user who created the payment. ")
    created_date: Optional[StrictStr] = Field(None, alias="createdDate", description="The date and time when the payment was created, in `yyyy-mm-dd hh:mm:ss` format. For example, 2017-03-01 15:31:10. ")
    credit_balance_amount: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="creditBalanceAmount", description="The amount that the payment transfers to the credit balance. The value is not `0` only for those payments that come from legacy payment operations performed without the Invoice Settlement feature. ")
    currency: Optional[StrictStr] = Field(None, description="When Standalone Payment is not enabled, the `currency` of the payment must be the same as the payment currency defined in the customer account settings through Zuora UI.  When Standalone Payment is enabled and `standalone` is `true`, the `currency` of the standalone payment can be different from the payment currency defined in the customer account settings. The amount will not be summed up to the account balance or key metrics regardless of currency. ")
    effective_date: Optional[StrictStr] = Field(None, alias="effectiveDate", description="The date and time when the payment takes effect, in `yyyy-mm-dd hh:mm:ss` format. ")
    finance_information: Optional[GetPaymentFinanceInformation] = Field(None, alias="financeInformation")
    gateway_id: Optional[StrictStr] = Field(None, alias="gatewayId", description="The ID of the gateway instance that processes the payment. ")
    gateway_order_id: Optional[StrictStr] = Field(None, alias="gatewayOrderId", description="A merchant-specified natural key value that can be passed to the electronic payment gateway when a payment is created.  If not specified, the payment number will be passed in instead. ")
    gateway_reconciliation_reason: Optional[StrictStr] = Field(None, alias="gatewayReconciliationReason", description="The reason of gateway reconciliation. ")
    gateway_reconciliation_status: Optional[StrictStr] = Field(None, alias="gatewayReconciliationStatus", description="The status of gateway reconciliation. ")
    gateway_response: Optional[StrictStr] = Field(None, alias="gatewayResponse", description="The message returned from the payment gateway for the payment. This message is gateway-dependent. ")
    gateway_response_code: Optional[StrictStr] = Field(None, alias="gatewayResponseCode", description="The code returned from the payment gateway for the payment. This code is gateway-dependent. ")
    gateway_state: Optional[StrictStr] = Field(None, alias="gatewayState")
    id: Optional[StrictStr] = Field(None, description="The unique ID of the created payment. For example, 4028905f5a87c0ff015a87eb6b75007f. ")
    marked_for_submission_on: Optional[StrictStr] = Field(None, alias="markedForSubmissionOn", description="The date and time when a payment was marked and waiting for batch submission to the payment process, in `yyyy-mm-dd hh:mm:ss` format. ")
    number: Optional[StrictStr] = Field(None, description="The unique identification number of the payment. For example, P-00000001. ")
    payment_method_id: Optional[StrictStr] = Field(None, alias="paymentMethodId", description="The unique ID of the payment method that the customer used to make the payment. ")
    payment_method_snapshot_id: Optional[StrictStr] = Field(None, alias="paymentMethodSnapshotId", description="The unique ID of the payment method snapshot which is a copy of the particular Payment Method used in a transaction. ")
    payment_option: Optional[conlist(PaymentSchedulePaymentOptionFields)] = Field(None, alias="paymentOption", description="Container for the paymentOption items, which describe the transactional level rules for processing payments. Currently, only the Gateway Options type is supported.  `paymentOption` of the payment schedule takes precedence over `paymentOption` of the payment schedule item.  This field is only available if `zuora-version` is set to 337.0 or later. ")
    payment_schedule_key: Optional[StrictStr] = Field(None, alias="paymentScheduleKey", description="The unique ID or the number of the payment schedule that is linked to the payment. See [Link payments to payment schedules](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/Payment_Schedules/Link_payments_with_payment_schedules) for more information.")
    payout_id: Optional[StrictStr] = Field(None, alias="payoutId", description="The payout ID of the payment from the gateway side. ")
    prepayment: Optional[StrictBool] = Field(None, description="Indicates whether the payment is used as a reserved payment. See [Prepaid Cash with Drawdown](https://knowledgecenter.zuora.com/Zuora_Billing/Billing_and_Invoicing/JA_Advanced_Consumption_Billing/Prepaid_Cash_with_Drawdown) for more information. ")
    reference_id: Optional[StrictStr] = Field(None, alias="referenceId", description="The transaction ID returned by the payment gateway. Use this field to reconcile payments between your gateway and Zuora Payments. ")
    refund_amount: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="refundAmount", description="The amount of the payment that is refunded. ")
    second_payment_reference_id: Optional[StrictStr] = Field(None, alias="secondPaymentReferenceId", description="The transaction ID returned by the payment gateway if there is an additional transaction for the payment. Use this field to reconcile payments between your gateway and Zuora Payments. ")
    settled_on: Optional[StrictStr] = Field(None, alias="settledOn", description="The date and time when the payment was settled in the payment processor, in `yyyy-mm-dd hh:mm:ss` format. This field is used by the Spectrum gateway only and not applicable to other gateways. ")
    soft_descriptor: Optional[StrictStr] = Field(None, alias="softDescriptor", description="A payment gateway-specific field that maps to Zuora for the gateways, Orbital, Vantiv and Verifi. ")
    soft_descriptor_phone: Optional[StrictStr] = Field(None, alias="softDescriptorPhone", description="A payment gateway-specific field that maps to Zuora for the gateways, Orbital, Vantiv and Verifi. ")
    standalone: Optional[StrictBool] = Field(False, description="This field is only available if the support for standalone payment is enabled. This field is not available for transferring, applying, or unapplying a payment.  The value `true` indicates this is a standalone payment that is created and processed in Zuora through Zuora gateway integration but will be settled outside of Zuora. No settlement data will be created. The standalone payment cannot be applied, unapplied, or transferred.  The value `false` indicates this is an ordinary payment that is created, processed, and settled in Zuora. ")
    status: Optional[StrictStr] = None
    submitted_on: Optional[StrictStr] = Field(None, alias="submittedOn", description="The date and time when the payment was submitted, in `yyyy-mm-dd hh:mm:ss` format. ")
    success: Optional[StrictBool] = Field(None, description="Returns `true` if the request was processed successfully.")
    type: Optional[StrictStr] = None
    unapplied_amount: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="unappliedAmount", description="The unapplied amount of the payment. ")
    updated_by_id: Optional[StrictStr] = Field(None, alias="updatedById", description="The ID of the Zuora user who last updated the payment. ")
    updated_date: Optional[StrictStr] = Field(None, alias="updatedDate", description="The date and time when the payment was last updated, in `yyyy-mm-dd hh:mm:ss` format. For example, 2017-03-02 15:36:10. ")
    additional_properties: Dict[str, Any] = {}
    __properties = ["accountId", "accountNumber", "amount", "appliedAmount", "authTransactionId", "bankIdentificationNumber", "cancelledOn", "comment", "createdById", "createdDate", "creditBalanceAmount", "currency", "effectiveDate", "financeInformation", "gatewayId", "gatewayOrderId", "gatewayReconciliationReason", "gatewayReconciliationStatus", "gatewayResponse", "gatewayResponseCode", "gatewayState", "id", "markedForSubmissionOn", "number", "paymentMethodId", "paymentMethodSnapshotId", "paymentOption", "paymentScheduleKey", "payoutId", "prepayment", "referenceId", "refundAmount", "secondPaymentReferenceId", "settledOn", "softDescriptor", "softDescriptorPhone", "standalone", "status", "submittedOn", "success", "type", "unappliedAmount", "updatedById", "updatedDate"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Payment:
        """Create an instance of Payment from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of finance_information
        if self.finance_information:
            _dict['financeInformation'] = self.finance_information.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in payment_option (list)
        _items = []
        if self.payment_option:
            for _item in self.payment_option:
                if _item:
                    _items.append(_item.to_dict())
            _dict['paymentOption'] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> Payment:
        """Create an instance of Payment from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return Payment.parse_obj(obj)

        _obj = Payment.parse_obj({
            "account_id": obj.get("accountId"),
            "account_number": obj.get("accountNumber"),
            "amount": obj.get("amount"),
            "applied_amount": obj.get("appliedAmount"),
            "auth_transaction_id": obj.get("authTransactionId"),
            "bank_identification_number": obj.get("bankIdentificationNumber"),
            "cancelled_on": obj.get("cancelledOn"),
            "comment": obj.get("comment"),
            "created_by_id": obj.get("createdById"),
            "created_date": obj.get("createdDate"),
            "credit_balance_amount": obj.get("creditBalanceAmount"),
            "currency": obj.get("currency"),
            "effective_date": obj.get("effectiveDate"),
            "finance_information": GetPaymentFinanceInformation.from_dict(obj.get("financeInformation")) if obj.get("financeInformation") is not None else None,
            "gateway_id": obj.get("gatewayId"),
            "gateway_order_id": obj.get("gatewayOrderId"),
            "gateway_reconciliation_reason": obj.get("gatewayReconciliationReason"),
            "gateway_reconciliation_status": obj.get("gatewayReconciliationStatus"),
            "gateway_response": obj.get("gatewayResponse"),
            "gateway_response_code": obj.get("gatewayResponseCode"),
            "gateway_state": obj.get("gatewayState"),
            "id": obj.get("id"),
            "marked_for_submission_on": obj.get("markedForSubmissionOn"),
            "number": obj.get("number"),
            "payment_method_id": obj.get("paymentMethodId"),
            "payment_method_snapshot_id": obj.get("paymentMethodSnapshotId"),
            "payment_option": [PaymentSchedulePaymentOptionFields.from_dict(_item) for _item in obj.get("paymentOption")] if obj.get("paymentOption") is not None else None,
            "payment_schedule_key": obj.get("paymentScheduleKey"),
            "payout_id": obj.get("payoutId"),
            "prepayment": obj.get("prepayment"),
            "reference_id": obj.get("referenceId"),
            "refund_amount": obj.get("refundAmount"),
            "second_payment_reference_id": obj.get("secondPaymentReferenceId"),
            "settled_on": obj.get("settledOn"),
            "soft_descriptor": obj.get("softDescriptor"),
            "soft_descriptor_phone": obj.get("softDescriptorPhone"),
            "standalone": obj.get("standalone") if obj.get("standalone") is not None else False,
            "status": obj.get("status"),
            "submitted_on": obj.get("submittedOn"),
            "success": obj.get("success"),
            "type": obj.get("type"),
            "unapplied_amount": obj.get("unappliedAmount"),
            "updated_by_id": obj.get("updatedById"),
            "updated_date": obj.get("updatedDate")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj


