# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictBool, StrictStr

class PaymentRun(BaseModel):
    """
    PaymentRun
    """
    account_id: Optional[StrictStr] = Field(None, alias="accountId", description="The ID of the customer account associated with the payment run. ")
    apply_credit_balance: Optional[StrictBool] = Field(None, alias="applyCreditBalance", description="**Note:** This field is only available if you have the Credit Balance feature enabled and the Invoice Settlement feature disabled.  Whether to apply credit balances in the payment run. This field is only available when you have Invoice Settlement feature disabled. ")
    auto_apply_credit_memo: Optional[StrictBool] = Field(None, alias="autoApplyCreditMemo", description="**Note:** This field is only available if you have [Invoice Settlement](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/Invoice_Settlement) enabled. The Invoice Settlement feature is generally available as of Zuora Billing Release 296 (March 2021). This feature includes Unapplied Payments, Credit and Debit Memo, and Invoice Item Settlement. If you want to enable Invoice Settlement, see [Invoice Settlement Enablement and Checklist Guide](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/Invoice_Settlement/Invoice_Settlement_Migration_Checklist_and_Guide) for more information.  Whether to automatically apply a posted credit memo to one or more receivables in the payment run. ")
    auto_apply_unapplied_payment: Optional[StrictBool] = Field(None, alias="autoApplyUnappliedPayment", description="**Note:** This field is only available if you have [Invoice Settlement](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/Invoice_Settlement) enabled. The Invoice Settlement feature is generally available as of Zuora Billing Release 296 (March 2021). This feature includes Unapplied Payments, Credit and Debit Memo, and Invoice Item Settlement. If you want to enable Invoice Settlement, see [Invoice Settlement Enablement and Checklist Guide](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/Invoice_Settlement/Invoice_Settlement_Migration_Checklist_and_Guide) for more information.  Whether to automatically apply unapplied payments to  one or more receivables in the payment run. ")
    batch: Optional[StrictStr] = Field(None, description="The alias name given to a batch. ")
    bill_cycle_day: Optional[StrictStr] = Field(None, alias="billCycleDay", description="The billing cycle day (BCD), the day of the month when a bill run generates invoices for the account.  ")
    billing_run_id: Optional[StrictStr] = Field(None, alias="billingRunId", description="The ID of the bill run. ")
    collect_payment: Optional[StrictBool] = Field(None, alias="collectPayment", description="Whether to process electronic payments during the execution of payment runs.  ")
    completed_on: Optional[StrictStr] = Field(None, alias="completedOn", description="The date and time when the payment run is completed, in `yyyy-mm-dd hh:mm:ss` format. For example, 2017-03-01 11:39:58. ")
    consolidated_payment: Optional[StrictBool] = Field(None, alias="consolidatedPayment", description="**Note:** The **Process Electronic Payment** permission also needs to be allowed for a Manage Payment Runs role to work. See [Payments Roles](https://knowledgecenter.zuora.com/CF_Users_and_Administrators/A_Administrator_Settings/User_Roles/e_Payments_Roles) for more information.   Whether to process a single payment for all receivables that are due on an account. ")
    created_by_id: Optional[StrictStr] = Field(None, alias="createdById", description="The ID of the Zuora user who created the payment run. ")
    created_date: Optional[StrictStr] = Field(None, alias="createdDate", description="The date and time when the payment run was created, in `yyyy-mm-dd hh:mm:ss` format. For example, 2017-03-01 15:31:10. ")
    currency: Optional[StrictStr] = Field(None, description="A currency defined in the web-based UI administrative settings. ")
    executed_on: Optional[StrictStr] = Field(None, alias="executedOn", description="The date and time when the payment run is executed, in `yyyy-mm-dd hh:mm:ss` format. For example, 2017-03-01 11:30:37. ")
    id: Optional[StrictStr] = Field(None, description="The ID of the payment run. ")
    number: Optional[StrictStr] = Field(None, description="The identification number of the payment run. ")
    payment_gateway_id: Optional[StrictStr] = Field(None, alias="paymentGatewayId", description="The ID of the gateway instance that processes the payment. ")
    process_payment_with_closed_pm: Optional[StrictBool] = Field(None, alias="processPaymentWithClosedPM", description="**Note:** The **Process Electronic Payment** permission also needs to be allowed for a Manage Payment Runs role to work. See [Payments Roles](https://knowledgecenter.zuora.com/CF_Users_and_Administrators/A_Administrator_Settings/User_Roles/e_Payments_Roles) for more information.   Whether to process payments even if the default payment method is closed. ")
    run_date: Optional[StrictStr] = Field(None, alias="runDate", description="The date and time when the scheduled payment run is to be executed for collecting payments. ")
    payment_gateway_number: Optional[StrictStr] = Field(None, alias="paymentGatewayNumber")
    status: Optional[StrictStr] = None
    target_date: Optional[StrictStr] = Field(None, alias="targetDate", description="The target date used to determine which receivables to be collected in the payment run.  ")
    updated_by_id: Optional[StrictStr] = Field(None, alias="updatedById", description="The ID of the Zuora user who last updated the payment run. ")
    updated_date: Optional[StrictStr] = Field(None, alias="updatedDate", description="The date and time when the payment run was last updated, in `yyyy-mm-dd hh:mm:ss` format. For example, 2017-03-02 15:36:10. ")
    organization_labels: Optional[StrictStr] = Field(None, alias="organizationLabels")
    __properties = ["accountId", "applyCreditBalance", "autoApplyCreditMemo", "autoApplyUnappliedPayment", "batch", "billCycleDay", "billingRunId", "collectPayment", "completedOn", "consolidatedPayment", "createdById", "createdDate", "currency", "executedOn", "id", "number", "paymentGatewayId", "processPaymentWithClosedPM", "runDate", "paymentGatewayNumber", "status", "targetDate", "updatedById", "updatedDate", "organizationLabels"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> PaymentRun:
        """Create an instance of PaymentRun from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> PaymentRun:
        """Create an instance of PaymentRun from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return PaymentRun.parse_obj(obj)

        _obj = PaymentRun.parse_obj({
            "account_id": obj.get("accountId"),
            "apply_credit_balance": obj.get("applyCreditBalance"),
            "auto_apply_credit_memo": obj.get("autoApplyCreditMemo"),
            "auto_apply_unapplied_payment": obj.get("autoApplyUnappliedPayment"),
            "batch": obj.get("batch"),
            "bill_cycle_day": obj.get("billCycleDay"),
            "billing_run_id": obj.get("billingRunId"),
            "collect_payment": obj.get("collectPayment"),
            "completed_on": obj.get("completedOn"),
            "consolidated_payment": obj.get("consolidatedPayment"),
            "created_by_id": obj.get("createdById"),
            "created_date": obj.get("createdDate"),
            "currency": obj.get("currency"),
            "executed_on": obj.get("executedOn"),
            "id": obj.get("id"),
            "number": obj.get("number"),
            "payment_gateway_id": obj.get("paymentGatewayId"),
            "process_payment_with_closed_pm": obj.get("processPaymentWithClosedPM"),
            "run_date": obj.get("runDate"),
            "payment_gateway_number": obj.get("paymentGatewayNumber"),
            "status": obj.get("status"),
            "target_date": obj.get("targetDate"),
            "updated_by_id": obj.get("updatedById"),
            "updated_date": obj.get("updatedDate"),
            "organization_labels": obj.get("organizationLabels")
        })
        return _obj


