# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic import BaseModel, Field, StrictBool, StrictStr, constr, validator

class PostServiceProviderRequest(BaseModel):
    """
    PostServiceProviderRequest
    """
    name: constr(strict=True, max_length=100) = Field(..., description="The name of the e-invoicing service provider. ")
    test: Optional[StrictBool] = Field(None, description="Whether the e-invoicing service provider's configuration is intended for testing.   - If you set this field to `true`, requests are directed to the testing integration endpoints. If you set this field to `false`, requests are directed to the production integration endpoints. ")
    provider: StrictStr = Field(..., description="The name of the e-invoicing service provider that can help you generate e-invoice files for billing documents. ")
    company_identifier: Optional[StrictStr] = Field(None, alias="companyIdentifier", description="The Company Identifier is used to create a SenderSystemId, which serves to identify the system from which the transactions are sent. ")
    api_key: Optional[StrictStr] = Field(None, alias="apiKey", description="The API key is used to authenticate the e-invoicing service provider's requests. ")
    secret_key: Optional[StrictStr] = Field(None, alias="secretKey", description="The Secret Key is used to authenticate the e-invoicing service provider's requests. ")
    client_certificate: Optional[StrictStr] = Field(None, alias="clientCertificate", description="The client certificate is used to authenticate the e-invoicing service provider's requests, which should be in base64 encoded format. ")
    client_certificate_type: Optional[StrictStr] = Field(None, alias="clientCertificateType", description="The client certificate type is used to specify the type of the client certificate. The default value is `PKCS12`. ")
    client_certificate_password: Optional[StrictStr] = Field(None, alias="clientCertificatePassword", description="The client certificate password is the password to protect the client certificate. ")
    __properties = ["name", "test", "provider", "companyIdentifier", "apiKey", "secretKey", "clientCertificate", "clientCertificateType", "clientCertificatePassword"]

    @validator('provider')
    def provider_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('Sovos'):
            raise ValueError("must be one of enum values ('Sovos')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> PostServiceProviderRequest:
        """Create an instance of PostServiceProviderRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> PostServiceProviderRequest:
        """Create an instance of PostServiceProviderRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return PostServiceProviderRequest.parse_obj(obj)

        _obj = PostServiceProviderRequest.parse_obj({
            "name": obj.get("name"),
            "test": obj.get("test"),
            "provider": obj.get("provider"),
            "company_identifier": obj.get("companyIdentifier"),
            "api_key": obj.get("apiKey"),
            "secret_key": obj.get("secretKey"),
            "client_certificate": obj.get("clientCertificate"),
            "client_certificate_type": obj.get("clientCertificateType"),
            "client_certificate_password": obj.get("clientCertificatePassword")
        })
        return _obj


