# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, Optional, Union
from pydantic import BaseModel, Field, StrictBool, StrictFloat, StrictInt, StrictStr, constr
from com.zuora.models.charge_override_billing import ChargeOverrideBilling
from com.zuora.models.charge_override_pricing import ChargeOverridePricing
from com.zuora.models.end_conditions import EndConditions
from com.zuora.models.trigger_params import TriggerParams

class PreviewOrderChargeOverride(BaseModel):
    """
    Charge associated with a rate plan.   # noqa: E501
    """
    billing: Optional[ChargeOverrideBilling] = None
    charge_number: Optional[constr(strict=True, max_length=50)] = Field(None, alias="chargeNumber", description="Charge number of the charge. For example, C-00000307.  If you do not set this field, Zuora will generate the charge number. ")
    custom_fields: Optional[Dict[str, Dict[str, Any]]] = Field(None, alias="customFields", description="Container for custom fields of a Rate Plan Charge object. ")
    description: Optional[constr(strict=True, max_length=500)] = Field(None, description="Description of the charge. ")
    drawdown_rate: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="drawdownRate", description="**Note**: This field is only available if you have the [Prepaid with Drawdown](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/J_Billing_Operations/Prepaid_with_Drawdown) feature enabled.  The [conversion rate](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/J_Billing_Operations/Prepaid_with_Drawdown/Create_drawdown_charge#UOM_Conversion) between Usage UOM and Drawdown UOM for a [drawdown charge](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/J_Billing_Operations/Prepaid_with_Drawdown/Create_drawdown_charge). Must be a positive number (>0). ")
    end_date: Optional[EndConditions] = Field(None, alias="endDate")
    exclude_item_billing_from_revenue_accounting: Optional[StrictBool] = Field(None, alias="excludeItemBillingFromRevenueAccounting", description="The flag to exclude rate plan charge related invoice items, invoice item adjustments, credit memo items, and debit memo items from revenue accounting.  **Note**: This field is only available if you have the [Zuora Billing - Revenue Integration](https://knowledgecenter.zuora.com/Zuora_Revenue/Zuora_Billing_-_Revenue_Integration) feature enabled. ")
    exclude_item_booking_from_revenue_accounting: Optional[StrictBool] = Field(None, alias="excludeItemBookingFromRevenueAccounting", description="The flag to exclude rate plan charges from revenue accounting.  **Note**: This field is only available if you have the [Zuora Billing - Revenue Integration](https://knowledgecenter.zuora.com/Zuora_Revenue/Zuora_Billing_-_Revenue_Integration) feature enabled. ")
    is_allocation_eligible: Optional[StrictBool] = Field(None, alias="isAllocationEligible", description="This field is used to identify if the charge segment is allocation eligible in revenue recognition.  **Note**: This feature is in the **Early Adopter** phase. If you want to use the feature, submit a request at <a href=\"https://support.zuora.com/\" target=\"_blank\">Zuora Global Support</a>, and we will evaluate whether the feature is suitable for your use cases. ")
    is_rollover: Optional[StrictBool] = Field(None, alias="isRollover", description="**Note**: This field is only available if you have the [Prepaid with Drawdown](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/J_Billing_Operations/Prepaid_with_Drawdown) feature enabled.  To use this field, you must set the `X-Zuora-WSDL-Version` request header to 114 or higher. Otherwise, an error occurs.  The value is either \"True\" or \"False\". It determines whether the rollover fields are needed. ")
    is_unbilled: Optional[StrictBool] = Field(None, alias="isUnbilled", description="This field is used to dictate how to perform the accounting during revenue recognition.  **Note**: This feature is in the **Early Adopter** phase. If you want to use the feature, submit a request at <a href=\"https://support.zuora.com/\" target=\"_blank\">Zuora Global Support</a>, and we will evaluate whether the feature is suitable for your use cases. ")
    prepaid_quantity: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="prepaidQuantity", description="**Note**: This field is only available if you have the [Prepaid with Drawdown](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/J_Billing_Operations/Prepaid_with_Drawdown) feature enabled.  The number of units included in a [prepayment charge](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/J_Billing_Operations/Prepaid_with_Drawdown/Create_prepayment_charge). Must be a positive number (>0). ")
    pricing: Optional[ChargeOverridePricing] = None
    product_rate_plan_charge_id: StrictStr = Field(..., alias="productRatePlanChargeId", description="Internal identifier of the product rate plan charge that the charge is based on. ")
    product_rate_plan_charge_number: Optional[StrictStr] = Field(None, alias="productRatePlanChargeNumber", description="Number of a product rate-plan charge for this subscription. ")
    rev_rec_code: Optional[constr(strict=True, max_length=70)] = Field(None, alias="revRecCode", description="Revenue Recognition Code ")
    rev_rec_trigger_condition: Optional[StrictStr] = Field(None, alias="revRecTriggerCondition")
    revenue_recognition_rule_name: Optional[StrictStr] = Field(None, alias="revenueRecognitionRuleName", description="Specifies the revenue recognition rule, such as `Recognize upon invoicing` or `Recognize daily over time`. ")
    rollover_apply: Optional[StrictStr] = Field(None, alias="rolloverApply")
    rollover_periods: Optional[Union[StrictFloat, StrictInt]] = Field(None, alias="rolloverPeriods", description="**Note**: This field is only available if you have the [Prepaid with Drawdown](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/J_Billing_Operations/Prepaid_with_Drawdown) feature enabled.  To use this field, you must set the `X-Zuora-WSDL-Version` request header to 114 or higher. Otherwise, an error occurs.  This field defines the number of rollover periods, it is restricted to 3. ")
    start_date: Optional[TriggerParams] = Field(None, alias="startDate")
    unique_token: Optional[constr(strict=True, max_length=50)] = Field(None, alias="uniqueToken", description="Unique identifier for the charge. This identifier enables you to refer to the charge before the charge has an internal identifier in Zuora.  For instance, suppose that you want to use a single order to add a product to a subscription and later update the same product. When you add the product, you can set a unique identifier for the charge. Then when you update the product, you can use the same unique identifier to specify which charge to modify. ")
    validity_period_type: Optional[StrictStr] = Field(None, alias="validityPeriodType")
    __properties = ["billing", "chargeNumber", "customFields", "description", "drawdownRate", "endDate", "excludeItemBillingFromRevenueAccounting", "excludeItemBookingFromRevenueAccounting", "isAllocationEligible", "isRollover", "isUnbilled", "prepaidQuantity", "pricing", "productRatePlanChargeId", "productRatePlanChargeNumber", "revRecCode", "revRecTriggerCondition", "revenueRecognitionRuleName", "rolloverApply", "rolloverPeriods", "startDate", "uniqueToken", "validityPeriodType"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> PreviewOrderChargeOverride:
        """Create an instance of PreviewOrderChargeOverride from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of billing
        if self.billing:
            _dict['billing'] = self.billing.to_dict()
        # override the default output from pydantic by calling `to_dict()` of end_date
        if self.end_date:
            _dict['endDate'] = self.end_date.to_dict()
        # override the default output from pydantic by calling `to_dict()` of pricing
        if self.pricing:
            _dict['pricing'] = self.pricing.to_dict()
        # override the default output from pydantic by calling `to_dict()` of start_date
        if self.start_date:
            _dict['startDate'] = self.start_date.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> PreviewOrderChargeOverride:
        """Create an instance of PreviewOrderChargeOverride from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return PreviewOrderChargeOverride.parse_obj(obj)

        _obj = PreviewOrderChargeOverride.parse_obj({
            "billing": ChargeOverrideBilling.from_dict(obj.get("billing")) if obj.get("billing") is not None else None,
            "charge_number": obj.get("chargeNumber"),
            "custom_fields": obj.get("customFields"),
            "description": obj.get("description"),
            "drawdown_rate": obj.get("drawdownRate"),
            "end_date": EndConditions.from_dict(obj.get("endDate")) if obj.get("endDate") is not None else None,
            "exclude_item_billing_from_revenue_accounting": obj.get("excludeItemBillingFromRevenueAccounting"),
            "exclude_item_booking_from_revenue_accounting": obj.get("excludeItemBookingFromRevenueAccounting"),
            "is_allocation_eligible": obj.get("isAllocationEligible"),
            "is_rollover": obj.get("isRollover"),
            "is_unbilled": obj.get("isUnbilled"),
            "prepaid_quantity": obj.get("prepaidQuantity"),
            "pricing": ChargeOverridePricing.from_dict(obj.get("pricing")) if obj.get("pricing") is not None else None,
            "product_rate_plan_charge_id": obj.get("productRatePlanChargeId"),
            "product_rate_plan_charge_number": obj.get("productRatePlanChargeNumber"),
            "rev_rec_code": obj.get("revRecCode"),
            "rev_rec_trigger_condition": obj.get("revRecTriggerCondition"),
            "revenue_recognition_rule_name": obj.get("revenueRecognitionRuleName"),
            "rollover_apply": obj.get("rolloverApply"),
            "rollover_periods": obj.get("rolloverPeriods"),
            "start_date": TriggerParams.from_dict(obj.get("startDate")) if obj.get("startDate") is not None else None,
            "unique_token": obj.get("uniqueToken"),
            "validity_period_type": obj.get("validityPeriodType")
        })
        return _obj


