# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, List, Optional
from pydantic import BaseModel, Field, StrictBool, StrictStr, conlist, constr
from com.zuora.models.create_order_rate_plan_feature_override import CreateOrderRatePlanFeatureOverride
from com.zuora.models.preview_order_charge_override import PreviewOrderChargeOverride

class PreviewOrderRatePlanOverride(BaseModel):
    """
    Rate plan associated with a subscription.   # noqa: E501
    """
    charge_overrides: Optional[conlist(PreviewOrderChargeOverride)] = Field(None, alias="chargeOverrides", description="List of charges associated with the rate plan. ")
    clearing_existing_features: Optional[StrictBool] = Field(None, alias="clearingExistingFeatures", description="Specifies whether all features in the rate plan will be cleared. ")
    custom_fields: Optional[Dict[str, Dict[str, Any]]] = Field(None, alias="customFields", description="Container for custom fields of the Rate Plan object. The custom fields of  the Rate Plan object are used when rate plans are subscribed. ")
    product_rate_plan_id: StrictStr = Field(..., alias="productRatePlanId", description="Internal identifier of the product rate plan that the rate plan is based on. ")
    product_rate_plan_number: Optional[StrictStr] = Field(None, alias="productRatePlanNumber", description="Number of a product rate plan for this subscription. ")
    subscription_product_features: Optional[conlist(CreateOrderRatePlanFeatureOverride)] = Field(None, alias="subscriptionProductFeatures", description="List of features associated with the rate plan. The system compares the `subscriptionProductFeatures` and `featureId` fields in the request with the counterpart fields in a rate plan. The comparison results are as follows: * If there is no `subscriptionProductFeatures` field or the field is empty, features in the rate plan remain unchanged. But if the `clearingExistingFeatures` field is additionally set to true, all features in the rate plan are cleared. * If the `subscriptionProductFeatures` field contains the `featureId` nested fields, as well as the optional `description` and `customFields` nested fields, the features indicated by the featureId nested fields in the request overwrite all features in the rate plan. ")
    unique_token: Optional[constr(strict=True, max_length=50)] = Field(None, alias="uniqueToken", description="Unique identifier for the rate plan. This identifier enables you to refer to the rate plan before the rate plan has an internal identifier in Zuora.  For instance, suppose that you want to use a single order to add a product to a subscription and later update the same product. When you add the product, you can set a unique identifier for the rate plan. Then when you update the product, you can use the same unique identifier to specify which rate plan to modify. ")
    subscription_rate_plan_number: Optional[constr(strict=True, max_length=50)] = Field(None, alias="subscriptionRatePlanNumber", description="Number of a subscription rate plan for this subscription. ")
    __properties = ["chargeOverrides", "clearingExistingFeatures", "customFields", "productRatePlanId", "productRatePlanNumber", "subscriptionProductFeatures", "uniqueToken", "subscriptionRatePlanNumber"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> PreviewOrderRatePlanOverride:
        """Create an instance of PreviewOrderRatePlanOverride from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in charge_overrides (list)
        _items = []
        if self.charge_overrides:
            for _item in self.charge_overrides:
                if _item:
                    _items.append(_item.to_dict())
            _dict['chargeOverrides'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in subscription_product_features (list)
        _items = []
        if self.subscription_product_features:
            for _item in self.subscription_product_features:
                if _item:
                    _items.append(_item.to_dict())
            _dict['subscriptionProductFeatures'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> PreviewOrderRatePlanOverride:
        """Create an instance of PreviewOrderRatePlanOverride from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return PreviewOrderRatePlanOverride.parse_obj(obj)

        _obj = PreviewOrderRatePlanOverride.parse_obj({
            "charge_overrides": [PreviewOrderChargeOverride.from_dict(_item) for _item in obj.get("chargeOverrides")] if obj.get("chargeOverrides") is not None else None,
            "clearing_existing_features": obj.get("clearingExistingFeatures"),
            "custom_fields": obj.get("customFields"),
            "product_rate_plan_id": obj.get("productRatePlanId"),
            "product_rate_plan_number": obj.get("productRatePlanNumber"),
            "subscription_product_features": [CreateOrderRatePlanFeatureOverride.from_dict(_item) for _item in obj.get("subscriptionProductFeatures")] if obj.get("subscriptionProductFeatures") is not None else None,
            "unique_token": obj.get("uniqueToken"),
            "subscription_rate_plan_number": obj.get("subscriptionRatePlanNumber")
        })
        return _obj


