# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, Optional
from pydantic import BaseModel, Field, StrictBool, StrictStr, conint, constr
from com.zuora.models.create_account_payment_method import CreateAccountPaymentMethod
from com.zuora.models.credit_card import CreditCard
from com.zuora.models.order_create_account_contact import OrderCreateAccountContact
from com.zuora.models.tax_info import TaxInfo

class PreviewOrderSubscriptionOwnerAccount(BaseModel):
    """
    Information about a new account that will own the subscription. Only available if you have enabled the Owner Transfer feature.  **Note:** The Owner Transfer feature is in **Limited Availability**. If you wish to have access to the feature, submit a request at [Zuora Global Support](http://support.zuora.com/).  If you do not set this field or the `subscriptionOwnerAccountNumber` field, the account that owns the order will also own the subscription. Zuora will return an error if you set this field and the `subscriptionOwnerAccountNumber` field.   # noqa: E501
    """
    account_number: Optional[constr(strict=True, max_length=70)] = Field(None, alias="accountNumber", description="Account number. For example, A00000001. ")
    additional_email_addresses: Optional[constr(strict=True, max_length=1200)] = Field(None, alias="additionalEmailAddresses", description="List of additional email addresses to receive emailed invoices. Values should be a comma-separated list of email addresses. ")
    allow_invoice_edit: Optional[StrictBool] = Field(None, alias="allowInvoiceEdit", description="Indicates if associated invoices can be edited. Values are:   * `true` * `false` (default) ")
    auto_pay: Optional[StrictBool] = Field(None, alias="autoPay", description="Specifies whether future payments are automatically billed when they are due. ")
    batch: Optional[StrictStr] = Field(None, description="Name of the billing batch that the account belongs to. For example, Batch1. ")
    bill_cycle_day: conint(strict=True, le=31, ge=0) = Field(..., alias="billCycleDay", description="Day of the month that the account prefers billing periods to begin on. If set to 0, the bill cycle day will be set as \"AutoSet\". ")
    bill_to_contact: OrderCreateAccountContact = Field(..., alias="billToContact")
    communication_profile_id: Optional[StrictStr] = Field(None, alias="communicationProfileId", description="Internal identifier of the communication profile that Zuora uses when sending notifications to the account's contacts. ")
    credit_card: Optional[CreditCard] = Field(None, alias="creditCard")
    credit_memo_template_id: Optional[StrictStr] = Field(None, alias="creditMemoTemplateId", description="**Note:** This field is only available if you have [Invoice Settlement](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/Invoice_Settlement) enabled. The Invoice Settlement feature is generally available as of Zuora Billing Release 296 (March 2021). This feature includes Unapplied Payments, Credit and Debit Memo, and Invoice Item Settlement. If you want to enable Invoice Settlement, see [Invoice Settlement Enablement and Checklist Guide](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/Invoice_Settlement/Invoice_Settlement_Migration_Checklist_and_Guide) for more information.  The unique ID of the credit memo template, configured in **Billing Settings** > **Manage Billing Document Configuration** through the Zuora UI. For example, 2c92c08a6246fdf101626b1b3fe0144b. ")
    crm_id: Optional[constr(strict=True, max_length=100)] = Field(None, alias="crmId", description="External identifier of the account in a CRM system. ")
    currency: StrictStr = Field(..., description="ISO 3-letter currency code (uppercase). For example, USD. ")
    custom_fields: Optional[Dict[str, Dict[str, Any]]] = Field(None, alias="customFields", description="Container for custom fields of an Account object. ")
    customer_service_rep_name: Optional[constr(strict=True, max_length=50)] = Field(None, alias="customerServiceRepName", description="Name of the account's customer service representative, if applicable. ")
    debit_memo_template_id: Optional[StrictStr] = Field(None, alias="debitMemoTemplateId", description="**Note:** This field is only available if you have [Invoice Settlement](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/Invoice_Settlement) enabled. The Invoice Settlement feature is generally available as of Zuora Billing Release 296 (March 2021). This feature includes Unapplied Payments, Credit and Debit Memo, and Invoice Item Settlement. If you want to enable Invoice Settlement, see [Invoice Settlement Enablement and Checklist Guide](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/Invoice_Settlement/Invoice_Settlement_Migration_Checklist_and_Guide) for more information.  The unique ID of the debit memo template, configured in **Billing Settings** > **Manage Billing Document Configuration** through the Zuora UI. For example, 2c92c08d62470a8501626b19d24f19e2. ")
    hpm_credit_card_payment_method_id: Optional[StrictStr] = Field(None, alias="hpmCreditCardPaymentMethodId", description="The ID of the payment method associated with this account. The payment method specified for this field will be set as the default payment method of the account.  If the `autoPay` field is set to `true`, you must provide the credit card payment method ID for either this field or the `creditCard` field, but not both.  For a specified credit card payment method, it is recommended that [the support for stored credential transactions](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/L_Payment_Methods/Stored_credential_transactions) for this payment method is already enabled. ")
    invoice_delivery_prefs_email: Optional[StrictBool] = Field(None, alias="invoiceDeliveryPrefsEmail", description="Specifies whether to turn on the invoice delivery method 'Email' for the new account.  Values are:   * `true` (default). Turn on the invoice delivery method 'Email' for the new account. * `false`. Turn off the invoice delivery method 'Email' for the new account.          ")
    invoice_delivery_prefs_print: Optional[StrictBool] = Field(None, alias="invoiceDeliveryPrefsPrint", description="Specifies whether to turn on the invoice delivery method 'Print' for the new account. Values are:   * `true`. Turn on the invoice delivery method 'Print' for the new account. * `false` (default). Turn off the invoice delivery method 'Print' for the new account. ")
    invoice_template_id: Optional[StrictStr] = Field(None, alias="invoiceTemplateId", description="Internal identifier of the invoice template that Zuora uses when generating invoices for the account. ")
    name: constr(strict=True, max_length=255) = Field(..., description="Account name. ")
    notes: Optional[constr(strict=True, max_length=65535)] = Field(None, description="Notes about the account. These notes are only visible to Zuora users. ")
    parent_id: Optional[StrictStr] = Field(None, alias="parentId", description="Identifier of the parent customer account for this Account object. Use this field if you have <a href=\"https://knowledgecenter.zuora.com/Billing/Subscriptions/Customer_Accounts/A_Customer_Account_Introduction#Customer_Hierarchy\" target=\"_blank\">Customer Hierarchy</a> enabled.")
    payment_gateway: Optional[constr(strict=True, max_length=40)] = Field(None, alias="paymentGateway", description="The payment gateway that Zuora uses when processing electronic payments and refunds for the account. If you do not specify this field or if the value of this field is null, Zuora uses your default payment gateway. ")
    payment_method: Optional[CreateAccountPaymentMethod] = Field(None, alias="paymentMethod")
    payment_term: Optional[StrictStr] = Field(None, alias="paymentTerm", description="Name of the payment term associated with the account. For example, \"Net 30\". The payment term determines the due dates of invoices. ")
    purchase_order_number: Optional[constr(strict=True, max_length=100)] = Field(None, alias="purchaseOrderNumber", description="The number of the purchase order associated with this account. Purchase order information generally comes from customers. ")
    sales_rep: Optional[constr(strict=True, max_length=50)] = Field(None, alias="salesRep", description="The name of the sales representative associated with this account, if applicable. ")
    sold_to_contact: Optional[OrderCreateAccountContact] = Field(None, alias="soldToContact")
    tax_info: Optional[TaxInfo] = Field(None, alias="taxInfo")
    __properties = ["accountNumber", "additionalEmailAddresses", "allowInvoiceEdit", "autoPay", "batch", "billCycleDay", "billToContact", "communicationProfileId", "creditCard", "creditMemoTemplateId", "crmId", "currency", "customFields", "customerServiceRepName", "debitMemoTemplateId", "hpmCreditCardPaymentMethodId", "invoiceDeliveryPrefsEmail", "invoiceDeliveryPrefsPrint", "invoiceTemplateId", "name", "notes", "parentId", "paymentGateway", "paymentMethod", "paymentTerm", "purchaseOrderNumber", "salesRep", "soldToContact", "taxInfo"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> PreviewOrderSubscriptionOwnerAccount:
        """Create an instance of PreviewOrderSubscriptionOwnerAccount from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of bill_to_contact
        if self.bill_to_contact:
            _dict['billToContact'] = self.bill_to_contact.to_dict()
        # override the default output from pydantic by calling `to_dict()` of credit_card
        if self.credit_card:
            _dict['creditCard'] = self.credit_card.to_dict()
        # override the default output from pydantic by calling `to_dict()` of payment_method
        if self.payment_method:
            _dict['paymentMethod'] = self.payment_method.to_dict()
        # override the default output from pydantic by calling `to_dict()` of sold_to_contact
        if self.sold_to_contact:
            _dict['soldToContact'] = self.sold_to_contact.to_dict()
        # override the default output from pydantic by calling `to_dict()` of tax_info
        if self.tax_info:
            _dict['taxInfo'] = self.tax_info.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> PreviewOrderSubscriptionOwnerAccount:
        """Create an instance of PreviewOrderSubscriptionOwnerAccount from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return PreviewOrderSubscriptionOwnerAccount.parse_obj(obj)

        _obj = PreviewOrderSubscriptionOwnerAccount.parse_obj({
            "account_number": obj.get("accountNumber"),
            "additional_email_addresses": obj.get("additionalEmailAddresses"),
            "allow_invoice_edit": obj.get("allowInvoiceEdit"),
            "auto_pay": obj.get("autoPay"),
            "batch": obj.get("batch"),
            "bill_cycle_day": obj.get("billCycleDay"),
            "bill_to_contact": OrderCreateAccountContact.from_dict(obj.get("billToContact")) if obj.get("billToContact") is not None else None,
            "communication_profile_id": obj.get("communicationProfileId"),
            "credit_card": CreditCard.from_dict(obj.get("creditCard")) if obj.get("creditCard") is not None else None,
            "credit_memo_template_id": obj.get("creditMemoTemplateId"),
            "crm_id": obj.get("crmId"),
            "currency": obj.get("currency"),
            "custom_fields": obj.get("customFields"),
            "customer_service_rep_name": obj.get("customerServiceRepName"),
            "debit_memo_template_id": obj.get("debitMemoTemplateId"),
            "hpm_credit_card_payment_method_id": obj.get("hpmCreditCardPaymentMethodId"),
            "invoice_delivery_prefs_email": obj.get("invoiceDeliveryPrefsEmail"),
            "invoice_delivery_prefs_print": obj.get("invoiceDeliveryPrefsPrint"),
            "invoice_template_id": obj.get("invoiceTemplateId"),
            "name": obj.get("name"),
            "notes": obj.get("notes"),
            "parent_id": obj.get("parentId"),
            "payment_gateway": obj.get("paymentGateway"),
            "payment_method": CreateAccountPaymentMethod.from_dict(obj.get("paymentMethod")) if obj.get("paymentMethod") is not None else None,
            "payment_term": obj.get("paymentTerm"),
            "purchase_order_number": obj.get("purchaseOrderNumber"),
            "sales_rep": obj.get("salesRep"),
            "sold_to_contact": OrderCreateAccountContact.from_dict(obj.get("soldToContact")) if obj.get("soldToContact") is not None else None,
            "tax_info": TaxInfo.from_dict(obj.get("taxInfo")) if obj.get("taxInfo") is not None else None
        })
        return _obj


