# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from typing import List, Optional
from pydantic import BaseModel, Field, StrictBool, StrictInt, StrictStr, conlist
from com.zuora.models.create_subscription_rate_plan import CreateSubscriptionRatePlan
from com.zuora.models.preview_subscription_account_info import PreviewSubscriptionAccountInfo

class PreviewSubscriptionRequest(BaseModel):
    """
    PreviewSubscriptionRequest
    """
    account_key: Optional[StrictStr] = Field(None, alias="accountKey", description=" Customer account number or ID.  You must specify the account information either in this field or in the `previewAccountInfo` field with the following conditions:           * If you already have a customer account, specify the account number or ID in this field. * If you do not have a customer account, provide account information in the `previewAccountInfo` field. ")
    contract_effective_date: date = Field(..., alias="contractEffectiveDate", description="Effective contract date for this subscription, as yyyy-mm-dd. ")
    customer_acceptance_date: Optional[date] = Field(None, alias="customerAcceptanceDate", description="The date on which the services or products within a subscription have been accepted by the customer, as yyyy-mm-dd.  Default value is dependent on the value of other fields. See **Notes** section for more details. ")
    document_date: Optional[date] = Field(None, alias="documentDate", description="The date of the billing document, in `yyyy-mm-dd` format. It represents the invoice date for invoices, credit memo date for credit memos, and debit memo date for debit memos.  - If this field is specified, the specified date is used as the billing document date.  - If this field is not specified, the date specified in the `targetDate` is used as the billing document date. ")
    include_existing_draft_doc_items: Optional[StrictBool] = Field(None, alias="includeExistingDraftDocItems", description="Specifies whether to include draft invoice items in subscription previews. Values are:  * `true` (default). Includes draft invoice items in the preview result. * `false`. Excludes draft invoice items in the preview result.  **Note:** This field is in Zuora REST API version control. Supported minor versions are 207.0 or later. To use this field in the method, you must set the **zuora-version** parameter to the minor version number in the request header. See [Zuora REST API Versions](https://www.zuora.com/developer/api-references/api/overview/#section/API-Versions) for more information. ")
    include_existing_draft_invoice_items: Optional[StrictBool] = Field(None, alias="includeExistingDraftInvoiceItems", description="Specifies whether to include draft invoice items in previews. Values are:  * `true` (default). Includes draft invoice items in the preview result. * `false`. Excludes draft invoice items in the preview result.  **Note:** This field is in Zuora REST API version control. Supported minor versions are 186.0, 187.0, 188.0, 189.0, 196.0, and 206.0. See [Zuora REST API Versions](https://www.zuora.com/developer/api-references/api/overview/#section/API-Versions) for more information. ")
    initial_term: Optional[StrictInt] = Field(None, alias="initialTerm", description="Duration of the first term of the subscription, in whole months. If `termType` is `TERMED`, then this field is required, and the value must be greater than `0`. If `termType` is `EVERGREEN`, this field is ignored.     ")
    initial_term_period_type: Optional[StrictStr] = Field(None, alias="initialTermPeriodType", description="The period type of the initial term.   Supported values are:  * `Month` * `Year` * `Day` * `Week`   The default period type is `Month`.  ")
    invoice_owner_account_key: Optional[StrictStr] = Field(None, alias="invoiceOwnerAccountKey", description="Invoice owner account number or ID.  **Note:** This feature is in **Limited Availability**. If you wish to have access to the feature, submit a request at [Zuora Global Support](http://support.zuora.com/). ")
    invoice_target_date: Optional[date] = Field(None, alias="invoiceTargetDate", description="Date through which to calculate charges if an invoice is generated, as yyyy-mm-dd. Default is current date.  **Note:** This field is in Zuora REST API version control. Supported minor versions are 186.0, 187.0, 188.0, 189.0, 196.0, and 206.0. See [Zuora REST API Versions](https://www.zuora.com/developer/api-references/api/overview/#section/API-Versions) for more information. ")
    notes: Optional[StrictStr] = Field(None, description="String of up to 500 characters.")
    preview_account_info: Optional[PreviewSubscriptionAccountInfo] = Field(None, alias="previewAccountInfo")
    preview_type: Optional[StrictStr] = Field(None, alias="previewType", description="The type of preview you will receive.   This field is in Zuora REST API version control. The supported values of this field depend on the REST API minor version you specified in the request header.   * If you do not specify the REST API minor version or specify the minor version number to one of following values in the request header:     * 186.0   * 187.0   * 188.0   * 189.0   * 196.0   * 206.0      The following values are supported in the **previewType** field:    * InvoiceItem   * ChargeMetrics   * InvoiceItemChargeMetrics      The default value is InvoiceItem.  * If you specify the REST API minor version to 207.0 or later in the request header, the following values are supported in the **previewType** field:    - LegalDoc   - ChargeMetrics   - LegalDocChargeMetrics    The default value is LegalDoc.  See [Zuora REST API Versions](https://www.zuora.com/developer/api-references/api/overview/#section/API-Versions) for more information. ")
    service_activation_date: Optional[date] = Field(None, alias="serviceActivationDate", description="The date on which the services or products within a subscription have been activated and access has been provided to the customer, as yyyy-mm-dd.  Default value is dependent on the value of other fields. See **Notes** section for more details. ")
    subscribe_to_rate_plans: conlist(CreateSubscriptionRatePlan) = Field(..., alias="subscribeToRatePlans", description="Container for one or more rate plans for this subscription. ")
    target_date: Optional[date] = Field(None, alias="targetDate", description="Date through which to calculate charges if an invoice is generated, as yyyy-mm-dd. Default is current date.  **Note:** This field is in Zuora REST API version control. Supported minor versions are 207.0 or later. To use this field in the method, you must set the **zuora-version** parameter to the minor version number in the request header. See [Zuora REST API Versions](https://www.zuora.com/developer/api-references/api/overview/#section/API-Versions) for more information. ")
    term_start_date: Optional[date] = Field(None, alias="termStartDate", description="The date on which the subscription term begins, as yyyy-mm-dd. If this is a renewal subscription, this date is different from the subscription start date. ")
    term_type: StrictStr = Field(..., alias="termType", description="Possible values are: `TERMED`, `EVERGREEN`. ")
    additional_properties: Dict[str, Any] = {}
    __properties = ["accountKey", "contractEffectiveDate", "customerAcceptanceDate", "documentDate", "includeExistingDraftDocItems", "includeExistingDraftInvoiceItems", "initialTerm", "initialTermPeriodType", "invoiceOwnerAccountKey", "invoiceTargetDate", "notes", "previewAccountInfo", "previewType", "serviceActivationDate", "subscribeToRatePlans", "targetDate", "termStartDate", "termType"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> PreviewSubscriptionRequest:
        """Create an instance of PreviewSubscriptionRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of preview_account_info
        if self.preview_account_info:
            _dict['previewAccountInfo'] = self.preview_account_info.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in subscribe_to_rate_plans (list)
        _items = []
        if self.subscribe_to_rate_plans:
            for _item in self.subscribe_to_rate_plans:
                if _item:
                    _items.append(_item.to_dict())
            _dict['subscribeToRatePlans'] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> PreviewSubscriptionRequest:
        """Create an instance of PreviewSubscriptionRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return PreviewSubscriptionRequest.parse_obj(obj)

        _obj = PreviewSubscriptionRequest.parse_obj({
            "account_key": obj.get("accountKey"),
            "contract_effective_date": obj.get("contractEffectiveDate"),
            "customer_acceptance_date": obj.get("customerAcceptanceDate"),
            "document_date": obj.get("documentDate"),
            "include_existing_draft_doc_items": obj.get("includeExistingDraftDocItems"),
            "include_existing_draft_invoice_items": obj.get("includeExistingDraftInvoiceItems"),
            "initial_term": obj.get("initialTerm"),
            "initial_term_period_type": obj.get("initialTermPeriodType"),
            "invoice_owner_account_key": obj.get("invoiceOwnerAccountKey"),
            "invoice_target_date": obj.get("invoiceTargetDate"),
            "notes": obj.get("notes"),
            "preview_account_info": PreviewSubscriptionAccountInfo.from_dict(obj.get("previewAccountInfo")) if obj.get("previewAccountInfo") is not None else None,
            "preview_type": obj.get("previewType"),
            "service_activation_date": obj.get("serviceActivationDate"),
            "subscribe_to_rate_plans": [CreateSubscriptionRatePlan.from_dict(_item) for _item in obj.get("subscribeToRatePlans")] if obj.get("subscribeToRatePlans") is not None else None,
            "target_date": obj.get("targetDate"),
            "term_start_date": obj.get("termStartDate"),
            "term_type": obj.get("termType")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj


