# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Dict, Optional
from pydantic import BaseModel, Field, StrictBool, conint, constr
from com.zuora.models.post_scheduled_event_request_parameters_value import PostScheduledEventRequestParametersValue

class PutScheduledEventRequest(BaseModel):
    """
    PutScheduledEventRequest
    """
    active: Optional[StrictBool] = Field(None, description="Indicate whether the scheduled event is active or inactive")
    condition: Optional[constr(strict=True, max_length=65535)] = Field(None, description="The filter rule conditions, written in [JEXL](http://commons.apache.org/proper/commons-jexl/). The scheduled event is triggered only if the condition is evaluated as true. The rule might contain event context merge fields and data source merge fields. Data source merge fields must be from [the base object of the event or from the joined objects of the base object](https://knowledgecenter.zuora.com/DC_Developers/M_Export_ZOQL#Data_Sources_and_Objects). Scheduled events with invalid merge fields will fail to evaluate, thus will not be triggered. For example, to trigger an invoice due date scheduled event to only invoices with an amount over 1000, you would define the following condition:  ```Invoice.Amount > 1000```  `Invoice.Amount` refers to the `Amount` field of the Zuora object `Invoice`. ")
    description: Optional[constr(strict=True, max_length=1000)] = Field(None, description="The description of the scheduled event.")
    display_name: Optional[constr(strict=True, max_length=500, min_length=1)] = Field(None, alias="displayName", description="The display name of the scheduled event.")
    hours: Optional[conint(strict=True, le=23, ge=0)] = Field(None, description="The scheduled time (hour) that the scheduled event notifications are sent. This time is based on the localized timezone of your tenant.")
    minutes: Optional[conint(strict=True, le=59, ge=0)] = Field(None, description="The scheduled time (minute) that the scheduled event notifications are sent. This time is based on the localized timezone of your tenant.")
    parameters: Optional[Dict[str, PostScheduledEventRequestParametersValue]] = Field(None, description="The parameters of the filter rule. The names of the parameters must match with the filter rule and can't be duplicated.")
    __properties = ["active", "condition", "description", "displayName", "hours", "minutes", "parameters"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> PutScheduledEventRequest:
        """Create an instance of PutScheduledEventRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each value in parameters (dict)
        _field_dict = {}
        if self.parameters:
            for _key in self.parameters:
                if self.parameters[_key]:
                    _field_dict[_key] = self.parameters[_key].to_dict()
            _dict['parameters'] = _field_dict
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> PutScheduledEventRequest:
        """Create an instance of PutScheduledEventRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return PutScheduledEventRequest.parse_obj(obj)

        _obj = PutScheduledEventRequest.parse_obj({
            "active": obj.get("active"),
            "condition": obj.get("condition"),
            "description": obj.get("description"),
            "display_name": obj.get("displayName"),
            "hours": obj.get("hours"),
            "minutes": obj.get("minutes"),
            "parameters": dict(
                (_k, PostScheduledEventRequestParametersValue.from_dict(_v))
                for _k, _v in obj.get("parameters").items()
            )
            if obj.get("parameters") is not None
            else None
        })
        return _obj


