# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from typing import List, Optional, Union
from pydantic import BaseModel, Field, StrictBool, StrictFloat, StrictInt, StrictStr, conlist, constr
from com.zuora.models.credit_memo_item_from_invoice_item_finance_information import CreditMemoItemFromInvoiceItemFinanceInformation
from com.zuora.models.update_credit_memo_tax_item_request import UpdateCreditMemoTaxItemRequest

class UpdateCreditMemoItem(BaseModel):
    """
    UpdateCreditMemoItem
    """
    amount: Optional[Union[StrictFloat, StrictInt]] = Field(None, description="The amount of the credit memo item. For tax-inclusive credit memo items, the amount indicates the credit memo item amount including tax. For tax-exclusive credit memo items, the amount indicates the credit memo item amount excluding tax")
    comment: Optional[StrictStr] = Field(None, description="Comments about the credit memo item.")
    exclude_item_billing_from_revenue_accounting: Optional[StrictBool] = Field(None, alias="excludeItemBillingFromRevenueAccounting", description="The flag to exclude the credit memo item from revenue accounting.  **Note**: This field is only available if you have the Billing - Revenue Integration feature enabled.  ")
    finance_information: Optional[CreditMemoItemFromInvoiceItemFinanceInformation] = Field(None, alias="financeInformation")
    id: constr(strict=True, max_length=32, min_length=32) = Field(..., description="The ID of the credit memo item.")
    quantity: Optional[Union[StrictFloat, StrictInt]] = Field(None, description="The number of units for the credit memo item.")
    service_end_date: Optional[date] = Field(None, alias="serviceEndDate", description="The service end date of the credit memo item.")
    service_start_date: Optional[date] = Field(None, alias="serviceStartDate", description="The service start date of the credit memo item.")
    sku_name: Optional[StrictStr] = Field(None, alias="skuName", description="The name of the SKU.")
    tax_items: Optional[conlist(UpdateCreditMemoTaxItemRequest)] = Field(None, alias="taxItems", description="Container for credit memo taxation items.")
    unit_of_measure: Optional[StrictStr] = Field(None, alias="unitOfMeasure", description="The definable unit that you measure when determining charges.")
    additional_properties: Dict[str, Any] = {}
    __properties = ["amount", "comment", "excludeItemBillingFromRevenueAccounting", "financeInformation", "id", "quantity", "serviceEndDate", "serviceStartDate", "skuName", "taxItems", "unitOfMeasure"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> UpdateCreditMemoItem:
        """Create an instance of UpdateCreditMemoItem from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of finance_information
        if self.finance_information:
            _dict['financeInformation'] = self.finance_information.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in tax_items (list)
        _items = []
        if self.tax_items:
            for _item in self.tax_items:
                if _item:
                    _items.append(_item.to_dict())
            _dict['taxItems'] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> UpdateCreditMemoItem:
        """Create an instance of UpdateCreditMemoItem from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return UpdateCreditMemoItem.parse_obj(obj)

        _obj = UpdateCreditMemoItem.parse_obj({
            "amount": obj.get("amount"),
            "comment": obj.get("comment"),
            "exclude_item_billing_from_revenue_accounting": obj.get("excludeItemBillingFromRevenueAccounting"),
            "finance_information": CreditMemoItemFromInvoiceItemFinanceInformation.from_dict(obj.get("financeInformation")) if obj.get("financeInformation") is not None else None,
            "id": obj.get("id"),
            "quantity": obj.get("quantity"),
            "service_end_date": obj.get("serviceEndDate"),
            "service_start_date": obj.get("serviceStartDate"),
            "sku_name": obj.get("skuName"),
            "tax_items": [UpdateCreditMemoTaxItemRequest.from_dict(_item) for _item in obj.get("taxItems")] if obj.get("taxItems") is not None else None,
            "unit_of_measure": obj.get("unitOfMeasure")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj


