# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date, datetime
from typing import Optional
from pydantic import BaseModel, Field, StrictBool, StrictStr

class UpdatePaymentRunRequest(BaseModel):
    """
    UpdatePaymentRunRequest
    """
    account_id: Optional[StrictStr] = Field(None, alias="accountId", description="The ID of the customer account associated with the payment run.  This field conflicts with each of the `batch`, `billCycleDay`, `currency`, `paymentGatewayId`, and `billingRunId` fields. If there are such conflicts, an error occurs and an error message is returned. ")
    apply_credit_balance: Optional[StrictBool] = Field(None, alias="applyCreditBalance", description="**Note:** This field is only available if you have the Credit Balance feature enabled and the Invoice Settlement feature disabled.  Whether to apply credit balances in the payment run. This field is only available when you have Invoice Settlement feature disabled. ")
    auto_apply_credit_memo: Optional[StrictBool] = Field(None, alias="autoApplyCreditMemo", description="**Note:** This field is only available if you have [Invoice Settlement](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/Invoice_Settlement) enabled. The Invoice Settlement feature is generally available as of Zuora Billing Release 296 (March 2021). This feature includes Unapplied Payments, Credit and Debit Memo, and Invoice Item Settlement. If you want to enable Invoice Settlement, see [Invoice Settlement Enablement and Checklist Guide](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/Invoice_Settlement/Invoice_Settlement_Migration_Checklist_and_Guide) for more information.  Whether to automatically apply a posted credit memo to one or more receivables in the payment run. ")
    auto_apply_unapplied_payment: Optional[StrictBool] = Field(None, alias="autoApplyUnappliedPayment", description="**Note:** This field is only available if you have [Invoice Settlement](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/Invoice_Settlement) enabled. The Invoice Settlement feature is generally available as of Zuora Billing Release 296 (March 2021). This feature includes Unapplied Payments, Credit and Debit Memo, and Invoice Item Settlement. If you want to enable Invoice Settlement, see [Invoice Settlement Enablement and Checklist Guide](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/Invoice_Settlement/Invoice_Settlement_Migration_Checklist_and_Guide) for more information.  Whether to automatically apply unapplied payments to  one or more receivables in the payment run. ")
    batch: Optional[StrictStr] = Field(None, description="The alias name given to a batch. The batch name is a string of 50 characters or less.  This field conflicts with the `accountId` field. If they are both specified in the request body, an error occurs and an error message is returned.  ")
    bill_cycle_day: Optional[StrictStr] = Field(None, alias="billCycleDay", description="The billing cycle day (BCD), the day of the month when a bill run generates invoices for the account. The value must be equal to or less then 31, and 31 is mean the EOM.  This field conflicts with the `accountId` field. If they are both specified in the request body, an error occurs and an error message is returned. ")
    billing_run_id: Optional[StrictStr] = Field(None, alias="billingRunId", description="The ID of a bill run.  This field conflicts with the `accountId` field. If they are both specified in the request body, an error occurs and an error message is returned. ")
    collect_payment: Optional[StrictBool] = Field(None, alias="collectPayment", description="Whether to process electronic payments during the execution of payment runs.   If the Payment user permission \"Process Electronic Payment\" is disabled, this field will be ignored. ")
    consolidated_payment: Optional[StrictBool] = Field(None, alias="consolidatedPayment", description="**Note:** The **Process Electronic Payment** permission also needs to be allowed for a Manage Payment Runs role to work. See [Payments Roles](https://knowledgecenter.zuora.com/CF_Users_and_Administrators/A_Administrator_Settings/User_Roles/e_Payments_Roles) for more information.   Whether to process a single payment for all receivables that are due on an account. ")
    currency: Optional[StrictStr] = Field(None, description="A currency defined in the web-based UI administrative settings.  This field conflicts with the `accountId` field. If they are both specified in the request body, an error occurs and an error message is returned. ")
    payment_gateway_id: Optional[StrictStr] = Field(None, alias="paymentGatewayId", description="The ID of the gateway instance that processes the payment.  This field conflicts with the `accountId` field. If they are both specified in the request body, an error occurs and an error message is returned. ")
    process_payment_with_closed_pm: Optional[StrictBool] = Field(None, alias="processPaymentWithClosedPM", description="**Note:** The **Process Electronic Payment** permission also needs to be allowed for a Manage Payment Runs role to work. See [Payments Roles](https://knowledgecenter.zuora.com/CF_Users_and_Administrators/A_Administrator_Settings/User_Roles/e_Payments_Roles) for more information.   Whether to process payments even if the default payment method is closed. ")
    run_date: Optional[datetime] = Field(None, alias="runDate", description="The date and time when the scheduled payment run is to be executed, in `yyyy-mm-dd hh:mm:ss` format. The backend will ignore mintues and seconds in the field value. For example, if you specify `2017-03-01 11:30:37` for this value, this payment run will be run at 2017-03-01 11:00:00. ")
    target_date: Optional[date] = Field(None, alias="targetDate", description="The target date used to determine which receivables to be paid in the payment run. The payments are collected for all receivables with the due date no later than the target date. ")
    __properties = ["accountId", "applyCreditBalance", "autoApplyCreditMemo", "autoApplyUnappliedPayment", "batch", "billCycleDay", "billingRunId", "collectPayment", "consolidatedPayment", "currency", "paymentGatewayId", "processPaymentWithClosedPM", "runDate", "targetDate"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> UpdatePaymentRunRequest:
        """Create an instance of UpdatePaymentRunRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> UpdatePaymentRunRequest:
        """Create an instance of UpdatePaymentRunRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return UpdatePaymentRunRequest.parse_obj(obj)

        _obj = UpdatePaymentRunRequest.parse_obj({
            "account_id": obj.get("accountId"),
            "apply_credit_balance": obj.get("applyCreditBalance"),
            "auto_apply_credit_memo": obj.get("autoApplyCreditMemo"),
            "auto_apply_unapplied_payment": obj.get("autoApplyUnappliedPayment"),
            "batch": obj.get("batch"),
            "bill_cycle_day": obj.get("billCycleDay"),
            "billing_run_id": obj.get("billingRunId"),
            "collect_payment": obj.get("collectPayment"),
            "consolidated_payment": obj.get("consolidatedPayment"),
            "currency": obj.get("currency"),
            "payment_gateway_id": obj.get("paymentGatewayId"),
            "process_payment_with_closed_pm": obj.get("processPaymentWithClosedPM"),
            "run_date": obj.get("runDate"),
            "target_date": obj.get("targetDate")
        })
        return _obj


